<?php
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Declaration.php');

/**
 * @package ELC
 * @subpackage ExportValidator
 * @author Nick Loeve <nick@basesoft.com>
 * @since 28/08/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
class ELCExportValidator
{
	var $db;
	var $filename;
	var $crso;
	var $type;

	/**
	 * This is used to strip the first line if it is field headers
	 */
	var $stripFirstLine = true;

	function ELCExportValidator($db, $filename, $crso, $type=null)
	{
		$this->db = $db;
		$this->filename = $filename;
		$this->type = $type;
		$this->crso = $crso;
		if(is_null($this->type))
			$this->type = $this->autoDetectType($filename);
		if(is_null($this->type))
			die('Could not detect the type of the export file!');
		if(!defined('ELC_EXPORT_PATH'))
			die('No ELC_EXPORT_PATH defined!');
	}

	/**
	 * @static
	 */
	function autoDetectType($filename)
	{
		if(preg_match("/[0-9]+_TRASALES.txt/", $filename))
			return EXPORT_FILE_DECLARATION;
		else if(preg_match("/[0-9]+_TRASALES_RR.txt/", $filename))
			return EXPORT_FILE_REFUND_REQUEST;
		else if(preg_match("/[0-9]+_MVEND.txt/", $filename))
			return EXPORT_FILE_MASTER_DATA;
		return null;
	}

	function factory($db, $filename, $crso)
	{
		$type = ELCExportValidator::autoDetectType($filename);
		if(is_null($type))
			die('Could not detect the type of the export file!');
		switch($type)
		{
			case EXPORT_FILE_DECLARATION:
				$validator = new ELCDeclarationExportValidator($db, $filename, $crso);
				break;
			case EXPORT_FILE_MASTER_DATA:
				$validator = new ELCMasterDataExportValidator($db, $filename, $crso);
				break;
			case EXPORT_FILE_REFUND_REQUEST:
				$validator = new ELCRefundRequestExportValidator($db, $filename, $crso);
				break;
		}
		return $validator;
	}

	function validate()
	{
		$lines =& $this->getLines();
		$lineIndex = $this->stripFirstLine ? 1: 0;
		foreach($lines as $line)
		{
			$this->validateLine($line, $lineIndex);
			$lineIndex++;
		}
	}

	function getLines()
	{
		$lines = file(ELC_EXPORT_PATH.$this->crso.'/'.$this->filename);
		if($this->stripFirstLine)
			array_shift($lines);
		return $lines;
	}

	function validateLine($line, $lineIndex)
	{
		//die('This is an abstract class. Please define your validateLine function in a subclass.');
	}

	function getFields($line, $fieldDelim="/(?<=#)\|/", $textDelim='#')
	{
		$fields = array();
		$tmpFields = preg_split($fieldDelim, $line);
		//try and detect textDelim
		foreach($tmpFields as $field)
		{
			if(strlen($field) && $field[0] == $textDelim && $field[strlen($field) - 1] == $textDelim)
				$fields[] = $this->ripData($field);
			else
				$fields[] = $field;
		}
		return $fields;
	}

	function ripData($field)
	{
		return substr($field, 1, strlen($field) - 2);
	}
}

define('EXPORT_FILE_DECLARATION', 1);
define('EXPORT_FILE_MASTER_DATA', 2);
define('EXPORT_FILE_REFUND_REQUEST', 3);




class ELCDeclarationExportValidator extends ELCExportValidator
{
		/**
		 1: No.
		 2: Line No.
		 3: Journal Template
		 4: Customer No.
		 5: Customer Name
		 6: Posting Date
		 7: Document Date
		 8: External Doc No.
		 9: Currency Code
		 10: Type
		 11: G/L Account No.
		 12: Description
		 13: Quantity
		 14: Unit Cost
		 15: Line Amount excl VAT
		 16: Total Amount excl VAT
		 17: Total Amount Incl VAT
		 18: Total VAT Amount
		 19: Navision VAT Code
		 20: VAT Extended text
		 21: Dimension Code 1
		 22: Dimension Code 2
		 23: Dimension Code 3
		 24: Dimension Code 4
		 25: Dimension Code 5
		 26: Dimension Code 6
		 27: Dimension Code 7
		 28: Dimension Code 8
		 29: Tariff No.
		 30: Net Weight
		 31: Transaction type
		 32: Exit Point
		 33: Transport Method
		*/

	/**
	 * Validate an exported declaration line
	 */
	function validateLine($line, $lineIndex)
	{

		$fields = $this->getFields($line);
		debug(print_r($fields, true));

		//first we get the crso id and the participant id and check they exist
		$crso_id = (int)substr($fields[3], 3, 8);
		$participant_id = (int)substr($fields[3], 11, 8);
		$crso = new CRSO($this->db, array('relation' => new Relation($this->db, $crso_id)));
		if(!$crso->set())
			print(sprintf("ERROR: CRSO [%s] does not exist in line [%s]<br />\n", $crso_id, $lineIndex));

		$participant = new Participant($this->db, array('relation' => new Relation($this->db, $participant_id)));
		if(!$participant->set())
			print(sprintf("ERROR: Participant [%s] does not exist in line [%s]<br />\n", $participant_id, $lineIndex));


		//now we see if the credit/debit exists and that the declaration exists and is exported
		$tbl = $fields[2] == 'C' ? 'CREDIT' : 'DEBIT';
		$q = $this->db->query('SELECT * FROM TBL_DECLARATION_'.$tbl.' WHERE DECLARATION_'.$tbl.'_ID=?');
		$q->bind_values(substr($fields[0], 2));
		if(!$q->execute())
			die('Problem executing query.');
		if(!($r = $q->fetch()))
			print(sprintf("ERROR: Declaration %s [%s] does not exist in line [%s]<br />\n", $tbl, substr($fields[0], 2), $lineIndex));
		if(!$r)
			print(sprintf("ERROR: Declaration %s [%s] has multiple entries in TBL_DECLARATION_%s in line [%s]<br />\n", $tbl, substr($fields[0], 2), $tbl, $lineIndex));
		$dec = new Declaration($this->db, $r->DECLARATION_ID);
		if(!$dec->set())
			print(sprintf("ERROR: Declaration [%s] does not exist in line [%s]<br />\n", $r->DECLARATION_ID, $lineIndex));
		if($dec->crso->relation->id != $crso_id)
			print(sprintf("ERROR: CRSO [%s] does not match CRSO [%s] in Declaration [%s] in line [%s]<br />\n", $crso_id, $dec->crso->relation->id, $r->DECLARATION_ID, $lineIndex));
		if($dec->participant->relation->id != $participant_id)
			print(sprintf("ERROR: Participant [%s] does not match Participant [%s] in Declaration [%s] in line [%s]<br />\n", $participant_id, $dec->participant->relation->id, $r->DECLARATION_ID, $lineIndex));
		if($dec->is_exported != 2)
			print(sprintf("ERROR: Declaration [%s] is not marked as exported in line [%s]<br />\n", $r->DECLARATION_ID, $lineIndex));

		$q = $this->db->query('SELECT EXPORTED_AMOUNT FROM TBL_DECLARATION_LINE WHERE DECLARATION_ID=? AND LINE_INDEX=?');
		$q->bind_values($r->DECLARATION_ID, $fields[1]);
		if(!$q->execute())
			die('Error executing query');
		if(!($r2 = $q->fetch()))
			print(sprintf("ERROR: DeclarationLine [%s, %s] does not exist in line [%s]<br />\n", $r->DECLARATION_ID, $fields[1], $lineIndex));
		if(number_format($r2->EXPORTED_AMOUNT, 2) != number_format(str_replace(',', '.', $fields[14]), 2))
			print(sprintf("ERROR: DeclarationLine [%s, %s] EXPORTED_AMOUNT does not match in line [%s]<br />\n", $r->DECLARATION_ID, $fields[1], $lineIndex));
	}
}
?>