<?php
/**
 * ELC SOAP Handler contains all SOAP methods for the participants
 *
 * @author Martijn van den Broek <martijn@xs4all.nl>
 * @copyright 2006 BaseSoft B.V.
 */

require_once(ELC_CLASSES.'WEEEController.php');

class ELCSoapHandler
{
	private $weeeController;

	function __construct()
	{
		$this->db = IWRegistry::registry('db');
		$this->langIf = IWRegistry::registry('langIf');
		$this->sessionHandler = IWRegistry::registry('sessionHandler');

		require_once(IWARE.'FormBehaviour.php');
		require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
		$this->fb = new FormBehaviour(IWRegistry::registry('langIf'), IWRegistry::registry('db'));
		$this->fb->setValues(array()); // we dont want FormBehaviour to work on $_POST data

		$crso_id = isset($_SESSION['selected_crso_id']) ? $_SESSION['selected_crso_id'] : null;
		$participant_id = isset($_SESSION['selected_participant_id']) ? $_SESSION['selected_participant_id'] : null;
		$product_id = isset($_SESSION['selected_product_id']) ? $_SESSION['selected_product_id'] : null;

		// make a cpp object, the controllers need this, Note that id's might not have been set. authenticate() on the controller should check all
		$cpp = new CRSOParticipantProduct($this->db);
		$cpp->crso = new CRSO($this->db, array('relation'=>new Relation($this->db, $crso_id)));
		$cpp->participant = new Participant($this->db, array('relation'=>new Relation($this->db, $participant_id)));
		$cpp->product = new Product($this->db, $product_id);

		// user
		$user = new ELCUser($this->db, $_SESSION['session_user_id']);

		// force english locale
		setlocale(LC_NUMERIC, 'en_GB.UTF-8');
		$_SESSION['locales'][LC_NUMERIC] = 'en_GB.UTF-8';

		// init the weee controller
		$this->weeeController = WEEEController::instance($cpp, $user);
	}

	/**
	 * Basic echo function to test functionality
	 */
	function echoString($inputString)
	{
		$this->fb->addValue('inputString', $inputString, 'required');

		if ( !$this->fb->validate() )
			throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

		return $inputString;
	}

	/**
	 * This operation will list all products processed by WEEE Blackbox, and
	 * provide an id for each product which can then be used in further
	 * operations.
	 *
	 * @return array of SoapProduct
	 */
	function getProducts()
	{
		try
		{
			require_once(ELC_CLASSES.'Product.php');

			$p = new Product($this->db);
			$q = $p->getProducts();
			$rv = array();

			while ($r = $q->fetch())
				$rv[] = new SoapProduct($r->{'PRODUCT_ID'}, $r->{'PRODUCT_NAME'});

			return $rv;
		}
		catch(Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unknown error while getting products');
		}
	}

	/**
	 * This operation will provide a list of product categories and an id for
	 * each category for use in later operations. Product categories are defined
	 * by a CRSO, so the crsoVAT is the VAT number of the CRSO the
	 * producer/refund requester deals with. The productId is an integer
	 * identifying which product you are acting on.
	 *
	 * @return array of SoapProductCategory
	 */
	function getProductCategories($crsoVAT, $productId)
	{
		try
		{
			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'Product.php');
			require_once(ELC_CLASSES.'ProductCategory.php');

			$this->fb->addValue('crsoVAT', $crsoVAT, 'required');
			$this->fb->addValue('productId', $productId, 'required;integer');

			if ( !$this->fb->validate() )
				throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

			$crso = new CRSO($this->db);
			if (!$crso->setByVAT($crsoVAT))
				throw new SoapFault('SOAP-ENV:Client', sprintf('CRSO with VAT [%s] does not exist', $crsoVAT));

			$product = new Product($this->db);
			if (!$product->set($productId))
				throw new SoapFault('SOAP-ENV:Client', sprintf('Product [%s] does not exist', $productId));

			$pc = new ProductCategory($this->db);
			$q = $pc->selectOverview($productId, $crso->relation->id);

			$rv = array();
			while ($r = $q->fetch())
				$rv[] = new SoapProductCategory($r->{'PRODUCT_CATEGORY_ID'}, $r->{'CATEGORY_NAME'});

			return $rv;
		}
		catch(SoapFault $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw $e;
		}
		catch(Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unknown error while getting product categories');
		}
	}

	/**
	 * This operation will provide a list of brands and an id for each category
	 * for use in later operations. Brands are defined by CRSO and assigned to
	 * producers, so you must provide the VAT number for both parties. The
	 * productId is an integer identifying which product you are acting on.
	 */
	function getBrands($crsoVAT, $producerVAT, $productId)
	{
		try
		{
			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'Participant.php');
			require_once(ELC_CLASSES.'Product.php');
			require_once(ELC_CLASSES.'ParticipantBrand.php');

			$this->fb->addValue('crsoVAT', $crsoVAT, 'required');
			$this->fb->addValue('producerVAT', $producerVAT, 'required');
			$this->fb->addValue('productId', $productId, 'required;integer');

			if ( !$this->fb->validate() )
				throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

			$crso = new CRSO($this->db);
			if (!$crso->setByVAT($crsoVAT))
				throw new SoapFault('SOAP-ENV:Client', sprintf('CRSO with VAT [%s] does not exist', $crsoVAT));

			$participant = new Participant($this->db);
			if (!$participant->setByVAT($producerVAT))
				throw new SoapFault('SOAP-ENV:Client', sprintf('Participant with VAT [%s] does not exist', $producerVAT));

			$product = new Product($this->db);
			if (!$product->set($productId))
				throw new SoapFault('SOAP-ENV:Client', sprintf('Product [%s] does not exist', $productId));

			$b = new ParticipantBrand($this->db);
			$q = $b->getBrands($participant->relation->id, $crso->relation->id, $productId);

			$rv = array();
			while ($r = $q->fetch())
				$rv[] = new SoapBrand($r->{'BRAND_ID'}, $r->{'NAME'});

			return $rv;
		}
		catch(SoapFault $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw $e;
		}
		catch(Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unknown error while getting brands');
		}
	}

	/**
	 * This operation supplies you with a list of countries. Each item in the
	 * list will provide a country name, country code, and country id, for use
	 * in later operations.
	 *
	 * @return ArrayOfCountry
	 */
	function getCountries()
	{
		try
		{
			require_once(ELC_CLASSES.'Country.php');

			$c = new Country($this->db);
			$q = $c->select();

			$rv = array();
			while ($r = $q->fetch())
				$rv[] = new SoapCountry($r->{'COUNTRY_ID'}, $r->{'NAME'}, $r->{'CODE'});

			return $rv;
		}
		catch(Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unknown error while getting countries');
		}
	}

	/**
	 * This soap operation will authenticate a user for further submission
	 * of data. The username anRd password will be the same as login via www.
	 * weeeblackbox.pwc.com, both as strings. The VAT is the VAT of the
	 * producer/refund requester that the user is operating as. The productId is
	 * an integer identifying which product you are acting on (initially all
	 * operations will be on Lamps).
	 *
	 * @return boolean true on success
	 */
	function authenticate($username, $password, $participantVAT, $crsoVAT, $productId)
	{
		try
		{
			require_once(ELC_CLASSES.'ELCSessionHandlerDecorator.php');
			require_once(ELC_CLASSES.'Participant.php');
			require_once(ELC_CLASSES.'RelationUser.php');
			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'CRSOParticipantProduct.php');

			$this->fb->addValue('username', $username, 'required');
			$this->fb->addValue('password', $username, 'required');
			$this->fb->addValue('participantVAT', $participantVAT, 'required');
			$this->fb->addValue('crsoVAT', $crsoVAT, 'required');
			$this->fb->addValue('productId', $productId, 'required;integer');

			if ( !$this->fb->validate() )
				throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

			$sd = new ELCIWSessionHandlerDecorator($this->sessionHandler);

			// Try to login with username and password, this will set $_SESSION['session_user_id']
			$errStr = ''; if(!$sd->login($username, $password, $errStr))
				throw new SoapFault('SOAP-ENV:Client', $errStr);
			$userId = $_SESSION['session_user_id'];

			// Set participant by VAT
			$participant = new Participant($this->db);
			if (!$participant->setByVAT($participantVAT))
				throw new SoapFault('SOAP-ENV:Client', sprintf('Participant with VAT [%s] does not exist', $participantVAT));

			// Try to change the default relation.
			$ru = new RelationUser($this->db);
			try {
				$ru->setDefaultRelation($userId, $participant->relation->id);
			} catch (Exception $e) {
				throw new SoapFault('SOAP-ENV:Client', 'Failed to change default relation');
			}

			$ru->set($userId, $participant->relation->id);
			$_SESSION['selected_participant_id'] = $participant->relation->id;
			info(sprintf('Setting selected_participant_id to %s for user %s', $participant->relation->id, $userId));

			// Try to set crso and save session
			$crso = new CRSO($this->db);
			if (!$crso->setByVAT($crsoVAT))
				throw new SoapFault('SOAP-ENV:Client', sprintf('CRSO with VAT [%s] does not exist', $crsoVAT));
			$_SESSION['selected_crso_id'] = $crso->relation->id;
			info(sprintf('Setting selected_crso_id to %s for user %s', $crso->relation->id, $userId));

			// Change product, this will set $_SESSION['selected_product_id'].
			try {
				$sd->changeProduct($productId);
			} catch (IWException $e) {
				throw new SoapFault('SOAP-ENV:Client', sprintf('Failed to change product %s', $productId));
			}
			info(sprintf('Setting selected_product_id to %s for user %s', $productId, $userId));

			// See if the participant is active for this product and crso
			$cp = new CRSOParticipantProduct($this->db);
			if(!$cp->set($crso->relation->id,$participant->relation->id,$productId))
				throw new SoapFault(sprintf('SOAP-ENV:Client', 'Participant [%s] is not active for CRSO [%s] / Product [%s]',
					$participant->relation->id, $crso->relation->id, $productId));

			return true;
		}
		catch(SoapFault $e)
		{
			session_destroy(); // logout
			throw $e;
		}
		catch(Exception $e)
		{
			session_destroy(); // logout
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unable to authenticate');
		}
	}

	 /**
	  * Makes a ELCDocument from the given soapDocument
	  * @param SoapDocument
	  * @return Document
	  */
	 private function makeDocument($soapDocument)
	 {
	 	try
	 	{
			// temp save the document
			$tmpfile = '/tmp/soap_upload_'.rand();
			if (!file_put_contents($tmpfile, $soapDocument->payload))
				throw new SoapFault('SOAP-ENV:Server', 'Error while saving document');

			// create elc document
			require_once(ELC_CLASSES.'Document.php');
			$document = Document::createDocument($this->db);
			$document->name = $soapDocument->fileName;
			$document->file_name = $soapDocument->fileName;
			$document->temp_name = $tmpfile;
			$document->insert();
			$document->makePermanent();

			// delete tmp file
			unlink($tmpfile);

			// make relation document
			require_once(ELC_CLASSES.'RelationDocument.php');
			$rd = new RelationDocument($this->db);
			$rd->relation = $this->weeeController->getCpp()->participant->relation;
			$rd->document = $document;
			$rd->insert();

		 	return $document;
		}
		catch(Exception $e)
		{
			trigger_error('Unable to save document: '.$e->getMessage(), E_USER_NOTICE);
			throw new IWException('Unable to save Document');
			unlink($tmpfile);
		}
	 }

	/**
	 * This operation will allow an refund requester to submit a proof of
	 * payment of visible fee. An accompanying document must still be provided
	 * (as with submission at www.weeeblackbox.pwc.com), and will be a base64
	 * encoded text block allowing for simple appendage to the SOAP operation
	 * request. YOU MUST BE AUTHENTICATED TO USE THIS OPERATION.
	 *
	 * @returns boolean true on success
	 */
	 function proofOfPayment($producer, $soapPopLines, $soapDocument, $soapSupplyChain)
	 {
	 	require_once(ELC_CLASSES.'ProofOfPaymentController.php');
	 	$controller = ProofOfPaymentController::instance($this->weeeController->getCpp(), $this->weeeController->getUser());

	 	try
	 	{
	 		// soap document can be optional
	 		$document = $soapDocument ? $this->makeDocument($soapDocument) : null;

			// Convert lines
			require_once(ELC_CLASSES.'ProofOfPayment.php');
			$popLines = array();
			foreach ($soapPopLines as $soapPopLine)
				$popLines[] = ProofOfPaymentLine::fromSoapProofOfPaymentLine($this->db, $soapPopLine);

			// convert supplyChain if given
	 		$supplyChain = array();
	 		if (is_array($soapSupplyChain))
	 		{
		 		require_once(ELC_CLASSES.'ProofOfPayment.php');
		 		foreach ($soapSupplyChain as $soapSupplier)
		 			$supplyChain[] = ProofOfPaymentSupplyChain::fromSoapSupplier($this->db, $soapSupplier);
	 		}

			$producer_relation = null;
			if (isset($producer->producerName) || isset($producer->producerVAT)) {
				$producer_relation = new Relation($this->db, array('name'=>$producer->producerName,'vat_no'=>$producer->producerVAT));
			}

		 	return (bool) $controller->proofOfPayment(null, $producer_relation, $popLines, $document, $supplyChain);
	 	}
	 	catch (WEEEAuthenticateException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
	 	}
	 	catch (FormBehaviourException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (WEEEException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (Exception $e)
	 	{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error while creating the Proof of Payment. An error report has been sent.');
	 	}
	 }

	/**
	 * FIXME: documentation
	 */
	 public function proofOfExportEU(Array $soapRefundLines, $exportDate, SoapCountry $soapExportCountry, $importerEmail, $importerVAT, $importerCompanyName, Array $additionalDocuments = null)
	 {
	 	require_once(ELC_CLASSES.'ProofOfExportController.php');
	 	$controller = ProofOfExportController::instance($this->weeeController->getCpp(), $this->weeeController->getUser());

	 	try
	 	{
	 		// Convert SoapCountry to Country
			require_once(ELC_CLASSES.'Country.php');
			$exportCountry = new Country($this->db, $soapExportCountry->countryId);
			if (!$exportCountry->set())
				throw new SoapFault('SOAP-ENV:Client', sprintf('Country [%s] does not exist', $soapExportCountry->countryId));

			// Convert refundLines
			require_once(ELC_CLASSES.'RefundRequest.php');
			$refundLines = array();
			foreach ($soapRefundLines as $soapRefundLine)
				$refundLines[] = RefundRequestLine::fromSoapRefundLine($this->db, $soapRefundLine);

			//additional documents
			if($additionalDocuments)
			{
				$additionalDocuments = $this->getFromSoapAdditionalDocuments($additionalDocuments);
			}

		 	return (bool) $controller->proofOfExportEU(null, $refundLines, $exportDate, $exportCountry, $importerEmail, $importerVAT, $importerCompanyName, $additionalDocuments);
	 	}
	 	catch (WEEEAuthenticateException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
	 	}
	 	catch (FormBehaviourException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (WEEEException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (Exception $e)
	 	{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error while creating the Proof of Export. A error report has been sent.');
	 	}
	 }


	/**
	 * FIXME: documentation
	 */
	public function proofOfExportNonEU(Array $soapRefundLines, $exportDate, SoapCountry $soapExportCountry, $identificationNo, $office, $commodityCode, $soapDocument, Array $additionalDocuments = null)
	{
	 	require_once(ELC_CLASSES.'ProofOfExportController.php');
	 	$controller = ProofOfExportController::instance($this->weeeController->getCpp(), $this->weeeController->getUser());

	 	try
	 	{
	 		// Convert Country
			require_once(ELC_CLASSES.'Country.php');
			$exportCountry = new Country($this->db, $soapExportCountry->countryId);
			if (!$exportCountry->set())
				throw new SoapFault('SOAP-ENV:Client', sprintf('Country [%s] does not exist', $soapExportCountry->countryId));

			// Convert refundLines
			require_once(ELC_CLASSES.'RefundRequest.php');
			$refundLines = array();
			foreach ($soapRefundLines as $soapRefundLine)
				$refundLines[] = RefundRequestLine::fromSoapRefundLine($this->db, $soapRefundLine);

	 		// soap document can be optional
	 		$document = $soapDocument ? $this->makeDocument($soapDocument) : null;

	 		//additional documents
			if($additionalDocuments)
			{
				$additionalDocuments = $this->getFromSoapAdditionalDocuments($additionalDocuments);
			}

		 	return (bool) $controller->proofOfExportNonEu(null, $refundLines, $exportDate, $exportCountry, $identificationNo, $office, $commodityCode, $document, $additionalDocuments);
	 	}
	 	catch (WEEEAuthenticateException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
	 	}
	 	catch (FormBehaviourException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (WEEEException $e)
	 	{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
	 	}
	 	catch (Exception $e)
	 	{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error while creating the Proof of Export. A error report has been sent.');
	 	}
	 }

	 private function getFromSoapAdditionalDocuments($additionalDocuments)
	 {
	 	require_once(ELC_CLASSES.'RefundRequestAdditionalDocument.php');
	 	$rv = array();
	 	foreach($additionalDocuments as $add_doc)
	 	{
	 		$doc = new RefundRequestAdditionalDocument($this->db);
	 		$doc->document = $this->makeDocument($add_doc->document);
	 		$doc->invoice_number = $add_doc->invoiceNo;
	 		$doc->invoice_date = $add_doc->invoiceDate;
	 		$rv[] = $doc;
	 	}
	 	return $rv;
	 }
}


?>
