<?php
/**
 * ELC SOAP Handler contains all SOAP methods for the accountants
 *
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 */

require_once(ELC_CLASSES.'WEEEController.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'ELCMessageHandler.php');

class ELCSoapHandlerAccountant
{
	private $weeeController;

	function __construct()
	{
		$this->db = IWRegistry::registry('db');
		$this->langIf = IWRegistry::registry('langIf');
		$this->sessionHandler = IWRegistry::registry('sessionHandler');

		require_once(IWARE.'FormBehaviour.php');
		require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
		$this->fb = new FormBehaviour(IWRegistry::registry('langIf'), IWRegistry::registry('db'));
		$this->fb->setValues(array()); // we dont want FormBehaviour to work on $_POST data

		// user
		$user = new ELCUser($this->db, $_SESSION['session_user_id']);
	}

	/**
	 * Basic echo function to test functionality
	 */
	function echoString($inputString)
	{
		info(sprintf('SOAP: echoString: %s', $inputString));

		$this->fb->addValue('inputString', $inputString, 'required');

		if ( !$this->fb->validate() )
		throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

		return $inputString;
	}

	function authenticate($username, $password) {
		try
		{
			require_once(ELC_CLASSES.'ELCSessionHandlerDecorator.php');
			require_once(ELC_CLASSES.'Participant.php');
			require_once(ELC_CLASSES.'RelationUser.php');
			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'CRSOParticipantProduct.php');

			info(sprintf('SOAP: authenticate: %s', $username));

			$this->fb->addValue('username', $username, 'required');
			$this->fb->addValue('password', $username, 'required');

			if ( !$this->fb->validate() )
				throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

			$sd = new ELCIWSessionHandlerDecorator($this->sessionHandler);

			// Try to login with username and password, this will set $_SESSION['session_user_id']
			$errStr = '';
			if(!$sd->login($username, $password, $errStr))
				throw new SoapFault('SOAP-ENV:Client', $errStr);

			$userId = $_SESSION['session_user_id'];

			info(sprintf(' - logged in as %s', $username));

			// force english locale
			setlocale(LC_NUMERIC, 'en_GB.UTF-8');
			$_SESSION['locales'][LC_NUMERIC] = 'en_GB.UTF-8';

			return true;

		}
		catch(SoapFault $e)
		{
			session_destroy(); // logout
			throw $e;
		}
		catch(Exception $e)
		{
			session_destroy(); // logout
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'Unable to authenticate');
		}

	}

	/**
	 * returns the current logged in accountant relation
	 */
	function _currentAccountantRelationId() {
		$accountant_id = isset($_SESSION['accountant_relation_id']) ? $_SESSION['accountant_relation_id'] : null;

		if (!$accountant_id) {
			alert('ELCSoapHandlerAccountant: Not logged in');
			throw new WEEEAuthenticateException('Not logged in');
		}
		return $accountant_id;
	}

	/**
	 * checks if the current accountnat is allowed to retrieve data fro the given crso
	 * @param $accountant_id
	 * @param $crso_id
	 */
	function _checkAllowedCrso($crso_id, $participant_id=null) {

		// checks if logged in as accountant and returns the current accountant_id
		$accountant_id = $this->_currentAccountantRelationId();

		require_once(ELC_CLASSES.'CRSO.php');
		$c = new CRSO($this->db);
		if (!$c->set($crso_id)) {
			throw new WEEEException(sprintf('ELCSoapHandlerAccountant: No such CRSO [%s]', $crso_id));
		}

		// check if accountant is active at crsos
		require_once(ELC_CLASSES.'AccountantCRSO.php');
		$ac = new AccountantCRSO($this->db);
		if (!$ac->setByAccountantAndCRSO($accountant_id, $crso_id)) {
			alert(sprintf('ELCSoapHandlerAccountant: Accountant [%s] not allowed to get masterdata for CRSO [%s]', $accountant_id, $crso_id));
			throw new WEEEException(sprintf('Permission denied accesing crso [%s]', $crso_id));
		}

		// if an participant_id is passed, check if the participant/crso combination is ok
		if (!is_null($participant_id)) {
			require_once(ELC_CLASSES.'CRSOParticipant.php');
			$cp = new CRSOParticipant($this->db);
			if (!$cp->set($crso_id, $participant_id))
				throw new WEEEException(sprintf('Participant [%s] is not part of CRSO [%s]', $participant_id, $crso_id));
		}

	}

	function getCrsoMasterData() {
		try {

			info(sprintf('SOAP: getCrsoMasterData()'));

			require_once(ELC_CLASSES.'AccountantCRSO.php');
			$ac = new AccountantCRSO($this->db);
			$q = $ac->selectAccountantCRSOMasterData($this->_currentAccountantRelationId());

			$rv = array();
			while ($r= $q->fetch()) {
				$c = new SoapCrsoMasterData();

				$c->crsoId = $r->CRSO_RELATION_ID;
				$c->crsoName = $r->NAME;
				$c->crsoAddress = $r->ADDRESS;
				$c->crsoPostalCode = $r->ADDRESS_ZIPCODE;
				$c->crsoCity = $r->CITY_NAME;
				$c->crsoContact = $r->CONTACT_NAME;
				$c->crsoPhone = $r->PHONE;
				$c->crsoFax = $r->FAX;
				$c->crsoLanguageCode = $r->LANGUAGE_CODE;
				$c->crsoCountryCode = $r->COUNTRY_CODE;
				$c->crsoVatNumber = $r->VAT_NO;
				$c->crsoEmail = $r->EMAIL;
				$c->crsoHomepage = $r->WEBSITE;
				$c->crsoBankAccountNumber = $r->BANK_ACCOUNT_NUMBER;
				$c->crsoIbanNumber = $r->IBAN_NO;
				$c->crsoSwiftNumber = $r->SWIFT_BIC_CODE;
				$c->crsoDdBankAccountNumber = $r->DD_BANK_ACCOUNT;
				$c->crsoDdIbanNumber = $r->DD_IBAN_NO;
				$c->crsoDdSwiftNumber = $r->DD_SWIFT_BIC_CODE;
				$c->crsoPoNumber = $r->PO_NO;

				$rv[] = $c;
			}

			return $rv;
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}
	}


	function getParticipantMasterData($crso_id) {

		try {
			info(sprintf('SOAP: getParticipantMasterData(): '.$crso_id));

			require_once(ELC_CLASSES.'Participant.php');

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');
			if ( !$this->fb->validate() )
				throw new SoapFault('SOAP-ENV:Client', $this->fb->getFailure() );

			// check if allowed at this crso else exception
			$this->_checkAllowedCrso($crso_id);

			// retrieve participant masterdata
			$ac = new Participant($this->db);
			$q = $ac->selectMasterData($crso_id);

			$rv = array();
			while ($r= $q->fetch()) {
				$p = new SoapParticipantMasterData();

				$p->participantId = $r->PARTICIPANT_RELATION_ID;
				$p->participantName = $r->NAME;
				$p->participantAddress = $r->ADDRESS;
				$p->participantPostalcode = $r->ADDRESS_ZIPCODE;
				$p->participantCity = $r->CITY_NAME;
				$p->participantContactName = $r->CONTACT_NAME;
				$p->participantPhone = $r->PHONE;
				$p->participantFax = $r->FAX;
				$p->participantLanguageCode = $r->LANGUAGE_CODE;
				$p->participantCountryCode = $r->COUNTRY_CODE;
				$p->participantVatNumber = $r->VAT_NO;
				$p->participantEmail = $r->EMAIL;
				$p->participantHomePage = $r->WEBSITE;
				$p->participantSendToName = $r->SEND_TO_NAME;
				$p->participantSendToAddress = $r->SEND_TO_ADDRESS;
				$p->participantSendToPostalcode = $r->SEND_TO_ZIPCODE;
				$p->participantSendToCity = $r->SEND_TO_CITY_NAME;
				$p->participantSendToCountryCode = $r->SEND_TO_COUNTRY_CODE;
				$p->participantSendToContact = $r->SEND_TO_CONTACT_NAME;
				$p->participantBankAccountNumber = $r->BANK_ACCOUNT_NUMBER;
				$p->participantIbanNumber = $r->IBAN_NO;
				$p->participantSwiftNumber = $r->SWIFT_BIC_CODE;
				$p->participantDdBankAccountNumber = $r->DD_BANK_ACCOUNT;
				$p->participantDdIbanNumber = $r->DD_IBAN_NO;
				$p->participantDdSwiftNumber = $r->DD_SWIFT_BIC_CODE;
				$p->participantRegistration = $r->PRODUCER_REG_NO;
				$p->participantPoNumber = $r->PO_NO;

				$rv[] = $p;
			}

			return $rv;
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}
	}

	protected function _getDeclarationStatus($status) {
		$map = array(
			DECLARATION_OPEN => 1,
			DECLARATION_NORMAL => 2,
			DECLARATION_FORFEIT => 3,
			DECLARATION_LATE => 4,
			DECLARATION_DECLARED => 5,
			DECLARATION_RESUBMIT => 6,
			DECLARATION_CREDIT => 7
		);

		if (!isset($map[$status]))
			throw new Exception('getDeclarationStatus: No such status: '.$status);
		return $map[$status];
	}

	function getDeclarations($crso_id, $year, $month) {

		try {
			info(sprintf(sprintf('SOAP: getDeclarations(): %s: %s-%s', $crso_id, $year, $month)));

			// check if allowed at this crso else exception
			$this->_checkAllowedCrso($crso_id);

			// get declarations for given arguments
			require_once(ELC_CLASSES.'Declaration.php');
			$d = new Declaration($this->db);
			$q = $d->selectExportedDeclarations($crso_id, $year, $month);

			// dataset is a 2d redundant array
			// so keep track of changes using some vars
			$inv = null;
			$dec = null;
			$invoiceNr = null;
			$decId = null;
			$declarations = null;
			$lines = null;

			// loopy
			$rv = array();
			while ($r= $q->fetch()) {

				// debug(print_r($r, true));

				// invoiceis unique per invoice number
				$_invoiceNr = $r->INVOICE_YEAR.$r->TYPE.$r->INVOICE_ID;
				if ($invoiceNr != $_invoiceNr) {
					$invoiceNr = $_invoiceNr;

					// create new invoice
					$inv = new SoapDeclarationInvoice();
					$inv->invoiceId = $r->INVOICE_ID;
					$inv->invoiceYear = $r->INVOICE_YEAR;
					$inv->type = $r->TYPE; // credit / debit
					$inv->participantId = $r->PARTICIPANT_RELATION_ID;
					$inv->documentDate = $r->DOCUMENT_DATE;
					$inv->declarations = array();
					$inv->totalAmount = $r->TOTAL_AMOUNT;

					// add invoice to resultset
					$rv[] = $inv;
				}

				// declaration
				if ($decId != $r->DECLARATION_ID) {
					$decId = $r->DECLARATION_ID;

					$dec = new SoapDeclaration();
					$dec->declarationId = $r->DECLARATION_ID;
					$dec->product = $r->PRODUCT_NAME;
					$dec->month = $r->MONTH;
					$dec->year = $r->YEAR;
					$dec->postingDate = $r->POSTING_DATE;
					$dec->originalInvoiceYear= $r->ORIGINAL_INVOICE_YEAR ? $r->ORIGINAL_INVOICE_YEAR : null;
					$dec->originalInvoiceId= $r->ORIGINAL_INVOICE_ID ? $r->ORIGINAL_INVOICE_ID : null;
					$dec->lines = array();

					// add declaration to invoice
					$inv->declarations[] = $dec;
				}

				$line = new SoapDeclarationLine();
				$line->quantity = $r->QUANTITY == '' ? null : $r->QUANTITY;
				$line->unitCost = $r->UNIT_COST;
				$line->weight = $r->WEIGHT == '' ? null : $r->WEIGHT;
				$line->weightDec = $r->WEIGHT_DEC == '' ? null : $r->WEIGHT_DEC;
				$line->productCategory = $r->CATEGORY_NAME;
				$line->brand = $r->BRAND_NAME;
				$line->dimension = $r->CPD_NAME;
				$line->amount = is_null($r->LINE_AMOUNT) ? 0 : $r->LINE_AMOUNT;
				$line->status = $this->_getDeclarationStatus($r->LINE_STATUS);

				$line->glAccount = $d->getGlAccount($r->DECLARATION_ID, $r->STATUS);

				// add line to declaration
				$dec->lines[] = $line;
			}

			return $rv;
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error while processing your request. An error report has been sent.');
		}
	}


	function getRefundRequests($crso_id, $year, $month) {

		try {
			info(sprintf(sprintf('SOAP: getRefundRequests(): %s: %s-%s', $crso_id, $year, $month)));

			// check if allowed at this crso else exception
			$this->_checkAllowedCrso($crso_id);

			require_once(ELC_CLASSES.'RefundExport.php');
			$re = new RefundExport($this->db);
			$q = $re->selectExportedRefundRequests($crso_id, $year, $month);

			$invoiceNr = null;
			$rrId = null;
			$refundExportId = null;

			$rv = array();
			while ($r= $q->fetch()) {

				// invoiceis unique per invoice number
				$_invoiceNr = $r->INVOICE_YEAR.$r->TYPE.$r->INVOICE_ID;
				if ($invoiceNr != $_invoiceNr) {
					$invoiceNr = $_invoiceNr;

					// create new invoice
					$inv = new SoapRefundRequestInvoice();
					$inv->invoiceId = $r->INVOICE_ID;
					$inv->invoiceYear = $r->INVOICE_YEAR;
					$inv->type = $r->TYPE; // FIXME: enum?
					$inv->participantId = $r->PARTICIPANT_RELATION_ID;
					$inv->documentDate = $r->DOCUMENT_DATE;
					$inv->billToParticipantId = $r->BILL_TO_PARTICIPANT_ID;
					$inv->totalAmount = $r->TOTAL_AMOUNT_EXCL_VAT;

					$inv->refundRequests = array();
					$inv->documentReviews = array();
					$inv->refundFees = array();

					// add invoice to resultset
					$rv[] = $inv;
				}

				// for each different refund_export_id..
				if ($refundExportId != $r->REFUND_EXPORT_ID) {
					$refundExportId = $r->REFUND_EXPORT_ID;

					// doc reviews
					$q2 = $re->getExportedDocumentReviews($r->REFUND_EXPORT_ID);
					while ($r2 = $q2->fetch()) {
						$docReview = new SoapRefundDocumentReview();
						$docReview->product = $r2->PRODUCT_NAME;
						$docReview->nrDocuments = $r2->DOCUMENT_COUNT;
						$docReview->amount = $inv->type == 'credit' ? $r2->REVIEW_COST * -1 : $r2->REVIEW_COST;
						$docReview->glAccount = $r->DOCUMENT_REVIEW_GLACCOUNT;
						$inv->documentReviews[] = $docReview;
					}

					// refund fee's
					$q2 = $re->getExportedRefundFees($r->REFUND_EXPORT_ID);
					while ($r2 = $q2->fetch()) {
						$refundFee = new SoapRefundFee();
						$refundFee->product = $r2->PRODUCT_NAME;
						$refundFee->fee = $r2->REFUND_COST;
						$refundFee->amount = $inv->type == 'credit' ? $r2->REFUND_COST * -1 : $r2->REFUND_COST;
						$refundFee->glAccount = $r2->GL_ACCOUNT_NO;
						$inv->refundFees[] = $refundFee;
					}
				}

				// dont specify 0 quantities
				if($r->QUANTITY === '0')
					continue;

				// refundRequest
				if ($rrId != $r->REFUND_REQUEST_ID) {
					$rrId = $r->REFUND_REQUEST_ID;

					$originalInvoice = explode('-', $r->ORIGINAL_INVOICE_DATA);

					$rr = new SoapRefundRequest();
					$rr->refundRequestId = $r->REFUND_REQUEST_ID;
					$rr->product = $r->PRODUCT_NAME;
					$rr->postingDate = $r->POSTING_DATE;
					$rr->originalInvoiceYear = isset($originalInvoice[0]) && $originalInvoice[0] ? $originalInvoice[0] : null;
					$rr->originalInvoiceId = isset($originalInvoice[1]) && $originalInvoice[1] ? $originalInvoice[1] : null;
					$rr->lines = array();

					// add declaration to invoice
					$inv->refundRequests[] = $rr;
				}

				$line = new SoapRefundRequestLine();
				$line->quantity = $r->QUANTITY;
				$line->unitCost = $r->UNIT_COST;
				$line->weight = $r->WEIGHT;
				$line->productCategory = $r->CATEGORY_NAME;
				$line->brand = $r->BRAND_NAME;
				$line->glAccount = $r->REFUND_GLACCOUNT;
				$line->amount = $r->LINE_AMOUNT_EXCL_VAT;

				if ($r->UNIT_COST == '0.13000000')
					debug(print_r($r, true));

				// add line to declaration
				$rr->lines[] = $line;


			}

			return $rv;


		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}
	}


	 /**
	  * Makes a ELCDocument from the given soapDocument
	  * @param SoapDocument
	  * @return Document
	  */
	 private function makeDocument($soapDocument, $relation, $dry_run)
	 {
	 	try
	 	{
	 		if (!$soapDocument->payload)
	 			throw new Exception('makeDocument: document has no payload');

			// temp save the document
			$tmpfile = '/tmp/soap_upload_'.rand();
			if (!file_put_contents($tmpfile, $soapDocument->payload))
				throw new SoapFault('SOAP-ENV:Server', 'Error while saving document');

			// create elc document
			require_once(ELC_CLASSES.'Document.php');
			$document = Document::createDocument($this->db);
			$document->name = $soapDocument->fileName;
			$document->file_name = $soapDocument->fileName;
			$document->temp_name = $tmpfile;
			$dry_run || $document->insert();
			$dry_run || $document->makePermanent();

			// delete tmp file
			unlink($tmpfile);

			// make relation document
			require_once(ELC_CLASSES.'RelationDocument.php');
			$rd = new RelationDocument($this->db);
			$rd->relation = $relation;
			$rd->document = $document;
			$dry_run || $rd->insert();

		 	return $document;
		}
		catch(Exception $e)
		{
			trigger_error('Unable to save document: '.$e->getMessage(), E_USER_NOTICE);
			throw new WEEEException('Invalid document supplied');
			unlink($tmpfile);
		}
	 }

	function sendDeclarationInvoice($crso_id, $participant_id, $type, $year, $invoice_id, $document, $dry_run)
	{
		$this->_sendDeclarationRefundDocument('Declaration', $crso_id, $participant_id, $type, $year, $invoice_id, $document, $dry_run);
	}

	function sendRefundInvoice($crso_id, $participant_id, $type, $year, $invoice_id, $document, $dry_run)
	{
		$this->_sendDeclarationRefundDocument('RefundExport', $crso_id, $participant_id, $type, $year, $invoice_id, $document, $dry_run);
	}

	protected function _sendDeclarationRefundDocument($class, $crso_id, $participant_id, $type, $year, $invoice_id, $document, $dry_run) {
		try
		{
			info(sprintf('SOAP: _sendDeclarationRefundDocument: %s, %s, %s, %s, %s, %s, %s, [%s]', $class, $crso_id, $participant_id, $type, $year, $invoice_id, $document->fileName, md5($document->payload)));
			info(' - dry run '.$dry_run.': '.($dry_run ? 'enabled' : 'disabled'));

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');
			$this->fb->addValue('participant_id', $participant_id, 'required;integer');
			$this->fb->addValue('type', $type, 'required');
			$this->fb->addValue('year', $year, 'required;integer');
			$this->fb->addValue('invoice_id', $invoice_id, 'required;integer');

			if (!$this->fb->validate())
				throw new FormBehaviourException($this->fb->getFailures());

			// check if current accountant is allowed to operate for this crso
			$this->_checkAllowedCrso($crso_id, $participant_id);

			if ($type != 'credit' && $type != 'debit')
				throw new WEEEException('invalid type: '.$type);

			$participant = new Participant($this->db, array('relation'=> new Relation($this->db, $participant_id)));

			$docname = ($class=='RefundExport')?'invoice_document':'document';
			$tbl = ($class=='RefundExport')?'REFUND_EXPORT':'DECLARATION';

			require_once(ELC_CLASSES.$class.'.php');
			$obj = new $class($this->db);

			$lookup_id = $invoice_id;
			$invoice_type = $type == 'credit' ? 'Credit' : 'Debit';

			// Check for Requested Invoices/Credits
			// In case this is a requested $invoice_type id on this object,
			//  change the object to RequestedInvoice to link the document to it.
			if (!($ids = $obj->{'getBy'.$invoice_type.'ID'}($lookup_id, $crso_id,$year)))
			{
				info(" - No $class found for $invoice_type '$lookup_id' - validating RequestedInvoice ");

				require_once(ELC_CLASSES.'RequestedInvoice.php');

				$obj = new RequestedInvoice($this->db);
				$docname = 'document';
				$obj->setTbl($tbl);
				if (!($ids = $obj->{'getBy'.$invoice_type.'ID'}($lookup_id, $crso_id,$year)))
				{
					throw new WEEEException("No RequestedInvoice ($class) found for $invoice_type '$lookup_id' - Bailing out");
				}
			}

	        $this->db->begin();

			$doc = $this->makeDocument($document, $participant->relation, $dry_run);

			foreach($ids as $id)
			{
				$obj->set($id);

				$obj->participant->relation->set();
				info(sprintf(" - Participant: id: [%s], name: [%s]",  $obj->participant->relation->id, $obj->participant->relation->name));

				if($participant_id != $obj->participant->relation->id)
				{
					throw new WEEEException(sprintf("participant id [%s] does not match obj participant id [%s]", $participant_id, $obj->participant->relation->id));
				}

				if($crso_id != $obj->crso->relation->id)
				{
					throw new WEEEException(sprintf("crso id in file [%s] does not match obj crso id [%s]", $crso_id, $obj->crso->relation->id));
				}
				if ($obj->$docname && $obj->$docname->id)
				{
					throw new WEEEException("$class '".$obj->id."' already has an invoice");
				}

				$obj->$docname = clone $doc;
				$dry_run || $obj->update();

				info(sprintf(' - updated object %s with document %s', $obj, $doc));
			}

			info(" - ".($dry_run? 'Dryrun ' : '')."Import successful for Participant [".$obj->participant->relation->id."] and CRSO [".$obj->crso->relation->id."]");

	        $this->db->commit();

			$messageType = array(
				'RefundExport'=>NEW_RR_INVOICE,
				'Declaration'=>NEW_INVOICE
			);

			$attachments = array();
			$attachments[] = array(
				'path'=>DOCUMENT_REPOSITORY.$doc->getFileName(),
				'name'=>$doc->name,
				'type'=>'application/pdf'
			);

			$this->sendEmail($obj, $messageType[$class], $attachments, $dry_run);
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}

	}

	/**
	 * Sends a message of the right type by email
	 *
	 * Expects an Object that has participant and crso instances
	 * @param mixed &$obj
	 */
	protected function sendEmail(&$obj,$messageType, $attachments, $dry_run)
	{
		require_once(ELC_CLASSES.'RelationInvoice.php');
		$ri = new RelationInvoice($this->db, array('relation'=>$obj->participant->relation), true);

		// changing the language from the info person to the relation invoice_language_id
		$obj->participant->relation->set();
		$ri->info_person->set();
		$ri->info_person->language_id = $obj->participant->relation->invoice_language->id;

		// only send attachments if crso has country setting
		$country = new Country($this->db,$obj->crso->getCountry(),true);
		if(!$country->has(HasSendInvoicesAsAttachment, null))
			$attachments = array();

		MessageEvent::send($messageType,array(
			'info_person'=>$ri->info_person,
			'crso'=>$obj->crso,
			'participant'=>$obj->participant,
			'attachments'=>$attachments),
			'ELCMessageHandler',
			$dry_run ); // dry-run
	}

	function sendReminderDocument($crso_id, $participant_id, $reminder_level, $invoice_date, $document, $dry_run) {
		try
		{
			info(sprintf('SOAP: sendReminderDocument: %s, %s, %s, %s, %s, [%s]', $crso_id, $participant_id, $reminder_level, $invoice_date, $document->fileName, md5($document->payload)));
			info(' - dry run '.$dry_run.': '.($dry_run ? 'enabled' : 'disabled'));

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');
			$this->fb->addValue('participant_id', $participant_id, 'required;integer');
			$this->fb->addValue('reminder_level', $reminder_level, 'required;integer');
			$this->fb->addValue('invoice_date', $invoice_date, 'required');

			$invoice_date = new IWDateTime($invoice_date);
			$invoice_date = $invoice_date->format("Y-m-d");

			if (!$this->fb->validate())
				throw new FormBehaviourException($this->fb->getFailures());

			$this->_checkAllowedCrso($crso_id, $participant_id);

			require_once(ELC_CLASSES.'CRSOParticipantReminder.php');

			$cpr = new CRSOParticipantReminder($this->db);

			$cpr->crso = new CRSO($this->db,array('relation'=>new Relation($this->db, $crso_id)));
			$cpr->participant = new Participant($this->db,array('relation'=>new Relation($this->db, $participant_id, true)));

			if(!$cpr->participant->relation->isComplete())
				throw new WEEEException("Not importing Incorrect Participant [%s]", $participant_id);

			$cpr->invoice_date = $invoice_date;
			$cpr->document = $this->makeDocument($document, $cpr->participant->relation, $dry_run);
			$cpr->reminder_level = $reminder_level;

			if(!$cpr->isUnique())
				throw new WEEEException("Already imported for invoice_date and reminder_level");

	        $this->db->begin();

			$dry_run || $cpr->insert();

			info(" - ".($dry_run? 'Dryrun ' : '')."Import successful for Participant [".$cpr->participant->relation->id."] and CRSO [".$cpr->crso->relation->id."]");

			// send a notification
			require_once(ELC_CLASSES.'Notification.php');
			$recipients = $map = array();
			$map['sender'] = $cpr->crso->relation;
			$map['type'] = NOTIFICATION_TYPE_INFO;
			$map['subject'] = IWRegistry::registry('langIf')->getText('NotificationTitle: New Invoice Reminder');
			$map['message'] = IWRegistry::registry('langIf')->getText('Notification: New Invoice Reminder Content');
			$map['product'] = DB_NULL;
			$recipients[] = array('recipient'=>$cpr->participant->relation,
						'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceReminderOverviewList.php');");
			$dry_run || Notification::send($this->db, $map, $recipients);

	        $this->db->commit();

			// send email
			$attachments = array();
			$attachments[] = array(
				'path'=>DOCUMENT_REPOSITORY.$cpr->document->getFileName(),
				'name'=>$cpr->document->name,
				'type'=>'application/pdf'
			);
			$this->sendEmail($cpr, NEW_REMINDER_LATE_GUARANTEE, $attachments, $dry_run);
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}

	}

	function sendGuaranteeInvoice($crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run)
	{
		$this->sendGuaranteeLatePaymentInvoice('CRSOParticipantGuarantee', $crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run);
	}

	function sendLatePaymentInvoice($crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run)
	{
		$this->sendGuaranteeLatePaymentInvoice('CRSOParticipantLatePayment', $crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run);
	}

	protected function sendGuaranteeLatePaymentInvoice($class, $crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run)
	{
		try
		{
			info(sprintf('SOAP: sendGuaranteeLatePaymentInvoice: %s, %s, %s, %s, %s, %s, %s, %s, %s, [%s]', $class, $crso_id, $participant_id, $type, $year, $invoice_id, $amount, $invoice_date, $document->fileName, md5($document->payload)));
			info(' - dry run '.$dry_run.': '.($dry_run ? 'enabled' : 'disabled'));

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');
			$this->fb->addValue('participant_id', $participant_id, 'required;integer');
			$this->fb->addValue('type', $type, 'required');
			$this->fb->addValue('year', $year, 'required;integer');
			$this->fb->addValue('invoice_id', $invoice_id, 'required;integer');
			$this->fb->addValue('amount', $amount, 'required;decimal');
			$this->fb->addValue('invoice_date', $invoice_date, 'required');

			if (!$this->fb->validate())
				throw new FormBehaviourException($this->fb->getFailures());

			$invoice_date = new IWDateTime($invoice_date);
			$invoice_date = $invoice_date->format("Y-m-d");

			$this->_checkAllowedCrso($crso_id, $participant_id);

			if ($type != 'credit' && $type != 'invoice')
				throw new WEEEException('invalid type: '.$type);

			$invoice_credit = ($type=='invoice') ? 'Invoice' : 'Credit';

			require_once(ELC_CLASSES.$class.'.php');
			$obj = new $class($this->db);
			$obj->crso = new CRSO($this->db,array('relation'=>new Relation($this->db, $crso_id)));
			$obj->participant = new Participant($this->db,array('relation'=>new Relation($this->db, $participant_id, true)));
			$obj->invoice_date = $invoice_date;

			$obj->amount = $amount;
			$obj->id = $invoice_id; // todo: should this validate against existing invoices?
			$obj->invoice_credit = $invoice_credit; ;
			$obj->year = $year;

			$obj->document = $this->makeDocument($document, $obj->participant->relation, $dry_run);
			if(!$obj->isUnique())
				throw new WEEEException(sprintf("Duplicate invoice"));

			$this->db->begin();

			$dry_run || $obj->insert();

			info(" - ".($dry_run? 'Dryrun ' : '')."Import successful for Participant [".$obj->participant->relation->id."] and CRSO [".$obj->crso->relation->id."]");

			$ntype = ($class == 'CRSOParticipantGuarantee') ? 'Guarantee' : 'LatePayment';

			require_once(ELC_CLASSES.'Notification.php');
			$recipients = $map = array();
			$map['sender'] = $obj->crso->relation;
			$map['type'] = NOTIFICATION_TYPE_INFO;
			$map['subject'] = IWRegistry::registry('langIf')->getText('NotificationTitle: New '.$invoice_credit.' '.$ntype);
			$map['message'] = IWRegistry::registry('langIf')->getText('Notification: New '.$invoice_credit.' '.$ntype.' Content');
			$map['product'] = DB_NULL;
			$recipients[] = array('recipient'=>$obj->participant->relation,
						'callback'=>"go('/".SITE_PATH."scripts/elc/participant/Interface".$ntype."OverviewList.php');");
			$dry_run || Notification::send($this->db, $map, $recipients);

			$this->db->commit();

			// send email
			$attachments = array();
			$attachments[] = array(
				'path'=>DOCUMENT_REPOSITORY.$obj->document->getFileName(),
				'name'=>$obj->document->name,
				'type'=>'application/pdf'
			);
			$this->sendEmail($obj, NEW_REMINDER_LATE_GUARANTEE, $attachments, $dry_run);
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}

	}

	function sendAgeing($crso_id, $lines, $dry_run)
	{
		try
		{
			info(sprintf('SOAP: sendAgeing: %s, %s', $crso_id, count($lines)));
			info(' - dry run '.$dry_run.': '.($dry_run ? 'enabled' : 'disabled'));

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');

			foreach ($lines->values as $i => $line) {

				$d = new DateTime($line->invoiceDate);
				$line->invoiceDate = $d->format('Y-m-d');

				$d = new DateTime($line->postingDate);
				$line->postingDate = $d->format('Y-m-d');

				$d = new DateTime($line->dueDate);
				$line->dueDate = $d->format('Y-m-d');

				$this->fb->addValue(sprintf('line-%s-participantId', $i), $line->participantId, 'required;integer');
				$this->fb->addValue(sprintf('line-%s-journalCode', $i), $line->journalCode, 'required');
				$this->fb->addValue(sprintf('line-%s-invoiceNr', $i), $line->invoiceNr, 'required');
				$this->fb->addValue(sprintf('line-%s-invoiceDate', $i), $line->invoiceDate, 'required;date(YYYY-MM-DD)');
				$this->fb->addValue(sprintf('line-%s-postingDate', $i), $line->postingDate, 'required;date(YYYY-MM-DD)');
				$this->fb->addValue(sprintf('line-%s-dueDate', $i), $line->dueDate, 'required;date(YYYY-MM-DD)');
				$this->fb->addValue(sprintf('line-%s-amount', $i), $line->amount, 'required;decimal');
				$this->fb->addValue(sprintf('line-%s-amountOutstanding', $i), $line->amountOutstanding, 'required;decimal');
			}

			if (!$this->fb->validate())
				throw new FormBehaviourException($this->fb->getFailures());

			$this->_checkAllowedCrso($crso_id);

	        $notified_participants = array();

	        $crso = new CRSO($this->db,array('relation'=>new Relation($this->db, $crso_id)));
	        $country = new Country($this->db,$crso->getCountry(),true);

	        $this->db->begin();

	        //flush the table
	        $flushSQL = "DELETE FROM TBL_AGEING WHERE CRSO_RELATION_ID = ?";
	        $flushQuery = $this->db->query($flushSQL);
	        $dry_run || $flushQuery->bind_values($crso_id);
	        $dry_run || $flushQuery->execute();

	        $currentDate = currentDate();

	        // prepare insert statement thats used for each line
	        $sql = "
	            INSERT INTO TBL_AGEING(
	                CRSO_RELATION_ID, PARTICIPANT_RELATION_ID, JOURNAL_CODE, DOCUMENT_NO,
	                DOCUMENT_DATE, POSTING_DATE, DUE_DATE, AMOUNT, AMOUNT_OUTSTANDING, NOT_DUE,
	                LATE_0_30, LATE_31_60, LATE_61_90, LATE_GT_90, IMPORT_DATE)
	            VALUES
	                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
	        $dry_run || $q = $this->db->query($sql);
	        $bindValues = array($crso_id, null, null, null, null, null, null, null,
	                            null, 'N', 'N' ,'N', 'N', 'N', $currentDate->format('Y-m-d'));

			foreach ($lines->values as $line) {

	            // check CRSOParticipant combo
	            require_once(ELC_CLASSES.'CRSOParticipant.php');
	            $cp = new CRSOParticipant($this->db);
	            if(!$cp->set($crso_id, $line->participantId))
	            	throw new WEEEException(sprintf('Participant [%s] is not active at CRSO [%s]', $line->participantId, $crso_id));

	            $values = array_values($bindValues);
	            $values[1] = $line->participantId;
	            $values[2] = $line->journalCode;
	            $values[3] = $line->invoiceNr;
	            $values[4] = $line->invoiceDate;
	            $values[5] = $line->postingDate;
	            $values[6] = $line->dueDate;
	            $values[7] = $line->amount;
	            $values[8] = $line->amountOutstanding;

	            $dueDate = new IWDateTime($line->dueDate);
	            if($currentDate->before($dueDate))
	            {
	                //not due yet
	                $values[9] = 'Y';
	            }
	            else
	            {
	                //now we need to work out how far late it is
	                $currentTS = $currentDate->format('U');
	                $dueTS = $dueDate->format('U');

	                $daysBehind = round(($currentTS - $dueTS) / 60 / 60 / 24); //mins, hours, days

	                if($daysBehind <= 30)
	                {
	                    $values[10] = 'Y';
	                }
	                else if($daysBehind <= 60)
	                {
	                    $values[11] = 'Y';
	                }
	                else if($daysBehind <= 90)
	                {
	                    $values[12] = 'Y';
	                }
	                else
	                {
	                    $values[13] = 'Y';
	                }
	            }

                $dry_run || $q->bind_values($values);
                $dry_run || $q->execute();
				info(" - ".($dry_run? 'Dryrun ' : '')."Import successful for ageing record of participant [".$line->participantId."] and CRSO [".$crso_id."]");

                // In case Country Settings HasParticipantAgeingAccess (#1990) throw new notification
                // Ageing information is not Product specific, use Lamps?. (!!!)
                if($country->has(HasParticipantAgeingAccess, 1) && !in_array($cp->participant->relation->id, $notified_participants))
                {
					require_once(ELC_CLASSES.'Notification.php');
					$recipients = $map = array();
					$map['sender'] = $crso->relation;
					$map['type'] = NOTIFICATION_TYPE_INFO;
					$map['subject'] = IWRegistry::registry('langIf')->getText('NotificationTitle: New Ageing Information');
					$map['message'] = IWRegistry::registry('langIf')->getText('Notification: New Ageing Information Content');
					$map['product'] = DB_NULL;
					$recipients[] = array('recipient'=>$cp->participant->relation,
								'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceAgeingBalanceList.php');");
					$dry_run || Notification::send($this->db, $map, $recipients);
					$dry_run || info(' - Notification sent to the Participant');
					// Make sure we only send them one notification.
					$notified_participants[] = $cp->participant->relation->id;
                }

			}

			// commit die shiff
			$this->db->commit();

		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}

	}

	function requestDeclarationInvoiceId($crso_id, $participant_id, $type, $year, $month) {

		return $this->_requestInvoiceId('Declaration', $crso_id, $participant_id, $type, $year, $month);
	}

	function requestRefundInvoiceId($crso_id, $participant_id, $type, $year, $month) {

		return $this->_requestInvoiceId('RefundExport', $crso_id, $participant_id, $type, $year, $month);
	}

	protected function _requestInvoiceId($class, $crso_id, $participant_id, $type, $year, $month) {

		try
		{
			info(sprintf('SOAP: _requestInvoiceId: %s, %s, %s, %s, %s, %s', $class, $crso_id, $participant_id, $type, $year, $month));

			$this->fb->addValue('crso_id', $crso_id, 'required;integer');
			$this->fb->addValue('participant_id', $participant_id, 'required;integer');
			$this->fb->addValue('year', $year, 'required;year');
			$this->fb->addValue('month', $month, 'required;month');

			if (!$this->fb->validate())
				throw new FormBehaviourException($this->fb->getFailures());

			$this->_checkAllowedCrso($crso_id, $participant_id);

			if ($type != 'credit' && $type != 'debit')
				throw new WEEEException('invalid type: '.$type);

			require_once(ELC_CLASSES.$class.'.php');
			$obj = new $class($this->db);
			$obj->crso = new CRSO($this->db,array('relation'=>new Relation($this->db, $crso_id)));

			require_once(ELC_CLASSES.'RequestedInvoice.php');
			$ri = new RequestedInvoice($this->db);
			$ri->setTbl($class == 'RefundExport' ? 'REFUND_EXPORT' : strtoupper($class));
			$ri->year = $year;
			$ri->month = $month;
			if (!$ri->addId($obj, $type, $participant_id))
				throw new Exception('Unable to add RequestedInvoice');

			$rv = (int) $ri->getInvoiceOrCreditId(strtoupper($type), $ri->id);
			info(' - returning new invoice_id: '.$rv);
			return $rv;
		}
		catch (WEEEAuthenticateException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', 'Please authenticate before using this Action');
		}
		catch (FormBehaviourException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (WEEEException $e)
		{
			throw new SoapFault('SOAP-ENV:Client', $e->getMessage());
		}
		catch (Exception $e)
		{
			trigger_error($e->getMessage(), E_USER_NOTICE);
			throw new SoapFault('SOAP-ENV:Server', 'There was an error processing your request. An error report has been sent.');
		}
	}

}


?>
