<?php
/**
 * @package
 * @subpackage
 * @author Nick Loeve <nick@basesoft.com>
 * @since 16/03/2007
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */

require_once(ELC_CLASSES.'RefundRequestController.php');

class ProofOfPaymentController extends RefundRequestController
{
	static $instance;

	static public function instance(CRSOParticipantProduct $cpp, ELCUser $user, FormBehaviour $fb = null)
	{
		$db = IWRegistry::registry('db');

		if(!isset(self::$instance))
			self::$instance = new ProofOfPaymentController($db, $cpp, $user, $fb);

		return self::$instance;
	}

	function registerPaymentFormBehaviours($popLines)
	{
		require_once(ELC_CLASSES.'CRSOVATBlock.php');

		$country_id = $this->cpp->crso->getCountry();
		$country = new Country($this->db, $country_id, true);

		// add fb plugins
  		$this->fb->addPlugin('elc_document_date', 'FormBehaviourPluginElcExporterDocumentdate');
        $this->fb->addPlugin('elc_not_future', 'FormBehaviourPluginElcNotFuture');
		$this->fb->addPlugin('elc_vat_block', 'FormBehaviourPluginElcVATBlock');

		// producer check
		$this->fb->register('producer_relation_vat_no', 'vat;required;elc_vat_block('.$this->cpp->crso->relation->id.','.CRSO_VAT_BLOCK_TYPE_POP_PRODUCER.')');
		if (!$country->has(HasRefundInternalVAT, $this->cpp->product->id))
			$this->fb->register('producer_relation_name', 'required;companyname');

		// Dynamic form behaviour for lines
   		foreach ($popLines as $key => $line)
   		{
			$this->fb->register('line['.$key.'][quantity]', 'required;integer(1)');
			$this->fb->register('line['.$key.'][invoice_id]', 'required;invoicenumber');
			$this->fb->register('line['.$key.'][payment_date]', 'required;date(YYYY-MM-DD);elc_not_future(YYYY-MM-DD);elc_document_date('.$this->cpp->crso->relation->id.',YYYY-MM-DD)');
			$this->fb->register('line['.$key.'][fee]', 'required;decimal');
   		}
	}

	/**
	 * new ProofOfPayment
	 *
	 * @return ProofOfPayment
	 */
	function proofOfPayment($popId=null, $producer_relation, $popLines, ELCDocument $document=null, $supplyChain=null)
	{
		require_once(ELC_CLASSES.'ProofOfPayment.php');

		$this->registerPaymentFormBehaviours($popLines);

		// Validate access, this will throw exceptions on error
		$this->checkAuthenticated();
		$this->checkClearance('ProofOfPayment', ClearanceProofOfPayment);
		$this->checkProducerExporter('Exporter', PARTICIPANT_EXPORTER);
		//$this->checkRefundSettings();

		// init country
		$country_id = $this->cpp->crso->getCountry();
		$country = new Country($this->db, $country_id, true);

		// set producer
		if ($producer_relation) {
			$this->fb->arrValues['producer_relation_vat_no'] = $producer_relation->vat_no;
			$this->fb->arrValues['producer_relation_name'] = $producer_relation->name;
		}

		// set all popLines
		foreach ($popLines as $index=>$popLine)
		{
			// check if the popLines array contains valid objects
			if (get_class($popLine) != 'ProofOfPaymentLine')
				trigger_error(sprintf('proofOfPaymentLine[%s] is not a ProofOfPaymentLine', $index), E_USER_ERROR);

			$this->fb->arrValues['line'][$index]['quantity'] = $popLine->quantity;
			$this->fb->arrValues['line'][$index]['invoice_id'] = $popLine->invoice_id;
			$this->fb->arrValues['line'][$index]['payment_date'] = $popLine->payment_date;
			$this->fb->arrValues['line'][$index]['fee'] = $popLine->fee;

			// check pop uniqueness

			$langAlreadySubmitted = IWRegistry::registry('langIf')->getText('A Proof of Payment has already been made for that Invoice/VAT_NO');

			if ($popId)
			{
				// Check if you are allowed to update this pop (if update).
				// We make the old PoP object and see if the cpp are the same
				$oldPop = new ProofOfPayment($this->db, $popId, true);
				$this->checkProofOfPaymentModify($oldPop);

				// notice that the unique check is ingored in multiline
				if (!$country->has(HasProofOfPaymentMultiline, $this->cpp->product->id) && !ProofOfPayment::isUnique($this->db, $popLine->invoice_id, $producer_relation->vat_no, $popId))
					throw new ProofOfPaymentException($langAlreadySubmitted);

			}
			else
			{
				// In case of a new PoP, check if it is unique without passing the $popId
				// notice that the unique check is ingored in multiline
				if (!$country->has(HasProofOfPaymentMultiline, $this->cpp->product->id) && !ProofOfPayment::isUnique($this->db, $popLine->invoice_id, $producer_relation->vat_no))
					throw new ProofOfPaymentException($langAlreadySubmitted);
			}
		}

		// Check if the supplyChain is supplied
		if ($country->has(HasPopSupplyChain, $this->cpp->product->id) && empty($supplyChain))
			throw new ProofOfPaymentException('You are obligated to supply a supplyChain');

		// Check supplyChain
		if ($country->has(HasPopSupplyChain, $this->cpp->product->id))
		{
			// add unique plugin, this one has to know what are the other values so lets register them in the fb
			$this->fb->addPlugin('pop_supply_chain_unique', 'FormBehaviourPluginElcPopSupplyChainUnique');

			// check class type
			$i = 0; foreach ($supplyChain as $sup)
			{
				if (get_class($sup) != 'ProofOfPaymentSupplyChain')
					throw new ProofOfPaymentException('Supply chain must be an array of ProofOfPaymentSupplyChain');

				// register all values to fb
				$values = array();
				$values['vat_no'] = $sup->vat_no;
				$values['company_name'] = $sup->company_name;
				$values['invoice_no'] = $sup->invoice_no;
				$this->fb->arrValues['supply_chain'][$i] = $values;

				// register checks
				$this->fb->register(sprintf('supply_chain[%s][%s]', $i, 'company_name'), 'required');
				$this->fb->register(sprintf('supply_chain[%s][%s]', $i, 'invoice_no'), 'required');
				$this->fb->register(sprintf('supply_chain[%s][%s]', $i, 'vat_no'),
					sprintf('required;vat;pop_supply_chain_unique(%s)', $sup->vat_no));

				$i++;
			}
		}

		// run fb validate
		$this->fb->validateWithExceptions();

		// All good, lets start building the PoP
		$pop = new ProofOfPayment($this->db);
		$pop->participant = $this->cpp->participant;
		$pop->crso = $this->cpp->crso;
		$pop->status = PROOF_OF_PAYMENT_PENDING;
		$pop->user = new ELCUser($this->db, $this->user->id);
		$pop->authorisation_by_type = $this->cpp->authorize_proof_of_payment;
		$pop->product = new Product($this->db, $this->cpp->product->id);

		// start transaction
		$this->db->begin();

		// check country setting HasRefundInternalVAT and if not create the participant if not exists
		$country = new Country($this->db, $this->cpp->participant->relation->getCountry(), true);

		// attach document if supplied
		if ($document) {
			require_once(ELC_CLASSES.'RelationDocument.php');
			$rd = new RelationDocument($this->db);
			$rd->relation = $this->cpp->participant->relation;
			$rd->document = $document;
			$rd->objectmap['inserttype'] = 'REPLACE';
			$rd->insert();

			$document->makePermanent();
			$pop->document = $document;
		}

		// producer
		if($producer_relation_id = Relation::getByVAT($producer_relation->vat_no, $this->db))
		{
			// producer already exists.. no need for update but just set it
			$producer_relation = new Relation($this->db, $producer_relation_id);
		}
		else if($country->has(HasRefundInternalVAT, $this->cpp->product->id))
		{
			$this->db->rollback();
			throw new ProofOfPaymentException('You are obligated to enter a valid producer. The producer specified does not exist');
		}
		else if(empty($producer_relation->name))
		{
			$this->db->rollback();
			throw new ProofOfPaymentException('You are obligated to enter a producer name.');
		}
		else
		{
			// if we come here the producer_relation doesnt exist yet so just insert it into the db
			$producer_relation->insert();
		}
		$pop->producer_relation = $producer_relation;

		if (!$popId)
			$pop->insert();
		else
		{
			$pop->id = $popId;
			$pop->update();
			$pop->deleteLines(); //need to clear them out before resaving
		}

		// insert the lines
		foreach ($popLines as $index => $popLine)
		{
			$popLine->line_index = $index+1; // line indexes start from 1
			$popLine->proof_of_payment = $pop;
			$popLine->insert();
		}


		// save supply chain
		$pop->deleteSupplyChain();
		$i = 1; foreach ($supplyChain as $supplier)
		{
			$supplier->supply_chain_index = $i;
			$supplier->pop = $pop;
			$supplier->insert();
			$i++;
		}

		$this->db->commit();

		return $pop;
	}

	/**
	 * This function can be extended to check for different groups too
	 */
	function checkProofOfPaymentModify(ProofOfPayment $pop)
	{
		if ($pop->crso->relation->id != $this->cpp->crso->relation->id ||
			$pop->participant->relation->id != $this->cpp->participant->relation->id ||
			$pop->product->id != $this->cpp->product->id )
		{
			throw new ProofOfPaymentException('You are not allowed to modify this ProofOfPayment');
		}
	}

	/**
	 * Check view rights, for now we just use modify
	 */
	function checkProofOfPaymentView(ProofOfPayment $pop)
	{
		return $this->checkProofOfPaymentModify($pop);
	}

	/**
	 * deleteProofOfPayment
	 */
	function deleteProofOfPayment(ProofOfPayment $pop)
	{
		$this->checkProofOfPaymentModify($pop);
		$pop->delete();
	}

	function getProofOfPayment($popId)
	{
		$pop = new ProofOfPayment(IWRegistry::registry('db'));
		if (!$pop->set($popId))
			throw new ProofOfPaymentException(sprintf('ProofOfPayment [%s] does not exist', $popId));
		$this->checkProofOfPaymentModify($pop);

		return $pop;
	}

}

/**
 * Exceptions
 */
class ProofOfPaymentException extends WEEEException {};
?>
