<?php

// server is needed to get classMap and include complex type classes
require_once(ELC_CLASSES.'ELCSoapServer.php');

require_once(IWARE.'IWXMLWriter.php');

/**
 * PwcExporter
 * @author martijn
 */
abstract class PwcExporter
{
	protected $hash_each_line_on_write = false;

	public function __construct($db, $path, $user, $pass, $langIf) {

		if (!substr($path, -1) == '/')
			throw new Exception('PwcExporter: path must end with slash');

		if (!is_dir($path))
			throw new Exception('PwcExporter: not a directory: '.$path);

		if (!is_writable($path))
			throw new Exception('PwcExporter: dir is not writable: '.$path);

		if (!$langIf || !is_object($langIf))
			throw new Exception('PwcExporter: missing langif or not an object');

		$this->_db = $db;
		$this->_path = $path;
		$this->_langIf = $langIf;

		// connect to soap server
		$this->_soapClient = new SoapClient(
			SOAP_WSDL_ACCOUNTANT_URL,
			array (
				'trace' => 1,
				'exceptions'=>1,
				'classmap'=>ELCSoapServer::getClassMap(),
				'features' => SOAP_SINGLE_ELEMENT_ARRAYS
			)
		);

		$this->_soapClient->authenticate($user, $pass);

		$this->writer = new IWXMLWriter(null, $this->hash_each_line_on_write);

		info('PwcExporter: created new pwc exporter');
	}

	public function export($year, $month, $day)
	{
		$crsoMasterData = $this->_soapClient->getCrsoMasterData()->values;

		foreach ($crsoMasterData as $crso) {

			$path = $this->_path.$crso->crsoId;
			if (!is_dir($path)) {
				info('PwcExporter: Creating directory '.$path);

				if (!mkdir($path, 0775)) {
					throw new Exception("Cannot create export directory for CRSO [".$r->crsoId."]: ".$this->_path);
				}
			}

			info('PwcExporter: export(): running export crso '.$crso->crsoId);
			$this->exportCrso($crso, $year, $month, $day);
		}

	}

	abstract function exportCrso(SoapCrsoMasterData $crso, $year, $month, $day);

	protected function writefile($dataset, $fieldlengths, $filename, $processors=array(), $hide_cols=array(), $sort_col=false)
	{
		if ( empty($dataset) ) {
			info('PwcExporter: writefile: Nothing to write');
			return;
		}

		$data = array();
		foreach($dataset as $tuple)
		{
			// remove hide_cols
			foreach ($hide_cols as $hide_col)
				unset($tuple[$hide_col]);

			//first process
			if ( !empty($processors) ) {
				foreach ( $processors as $attr=>$method )
					$tuple[$attr] = $this->$method($tuple[$attr]);
			}

			//then fix length..
			array_walk($tuple, array($this, 'fixAttrLength'), $fieldlengths);
			$data[] = $tuple;
		}


		if(is_string($sort_col))
		{
			foreach($data as $key => $row)
				$tmp[$key] = $row[$sort_col];

			array_multisort($tmp, SORT_ASC, $data);
		}
		$this->writer->setData($data);

		$filename = $this->_path.$filename;
		info('PwcExporter::writefile() - writing to file '.$filename);
		$this->writer->write($filename);
	}

	public function fixAttrLength(&$value, $index, $lengthmap)
	{
		if ( isset($lengthmap[$index]) )
		{
			list($len, $start) = array_pad((array)$lengthmap[$index],2 ,0);
			$value = mb_substr($value, $start, $len, 'UTF-8');
		}
	}

	function getLanguageIdFromCode($langCode) {

		$sql = "SELECT LANGUAGE_ID FROM TBL_LANGUAGE WHERE LANGUAGE_CODE=?;";
		$q = $this->_db->query($sql);
		$q->bind_values($langCode);
		$q->execute();

		if (!($row = $q->fetch()))
			throw new Exception('PwcExporter: langCode not found: '.$langCode);

		return $row->LANGUAGE_ID;
	}

	function getNavisionLanguageCode($langCode) {
		$sql = "SELECT NAVISION_CODE FROM TBL_LANGUAGE WHERE LANGUAGE_CODE=?;";
		$q = $this->_db->query($sql);
		$q->bind_values($langCode);
		$q->execute();

		if (!($row = $q->fetch()))
			throw new Exception('PwcExporter: langCode not found: '.$langCode);

		if (!$row->NAVISION_CODE) {
			alert('PwcExporter: no navision code for langCode: '.$langCode);
			return '';
		}

		return $row->NAVISION_CODE;
	}

	function getNavisionCountryCode($countryCode) {
		$sql = "SELECT NAVISION_CODE FROM TBL_COUNTRY WHERE CODE=?;";
		$q = $this->_db->query($sql);
		$q->bind_values($countryCode);
		$q->execute();

		if (!($row = $q->fetch()))
			throw new Exception('PwcExporter: countryCode not found: '.$countryCode);

		if (!$row->NAVISION_CODE)
			throw new Exception('PwcExporter: no navision code for countryCode: '.$countryCode);

		return $row->NAVISION_CODE;
	}

	function getProductIdFromName($name) {
		$sql = "SELECT PRODUCT_ID FROM TBL_PRODUCT WHERE NAME=?;";
		$q = $this->_db->query($sql);
		$q->bind_values($name);
		$q->execute();

		if (!($row = $q->fetch()))
			throw new Exception('PwcExporter: product not found: '.$name);

		return $row->PRODUCT_ID;
	}

	function buildInvoiceForExport($crso_id, $year, $invoice_id, $is_credit, $is_refund) {

		$sql = "SELECT f_elcBuildInvoiceNoForExport(?, ?, ?, ?, ?) as RESULT;";
		$q = $this->_db->query($sql);
		$q->bind_values($crso_id, $year, $invoice_id, (int) $is_credit, (int) $is_refund);
		$q->execute();
		$r = $q->fetch();
		return $r->RESULT;
	}

	function buildInvoiceNoForInterfaces($crso_id, $year, $invoice_id, $is_credit, $is_refund) {

		$sql = "SELECT f_elcBuildInvoiceNoForInterfaces(?, ?, ?, ?, ?) as RESULT;";
		$q = $this->_db->query($sql);
		$q->bind_values($crso_id, $year, $invoice_id, (int) $is_credit, (int) $is_refund);
		$q->execute();
		$r = $q->fetch();
		return $r->RESULT;
	}


}


?>
