<?php
/**
 * ELCImporter handles various importing actions like invoices

 * @package PwcImporter
 * @since 7-sep-2006
 * @author Marten Koedam <marten@dalines.net>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V
 */

// server is only needed to get classMap and include complex type classes
require_once(ELC_CLASSES.'ELCSoapServer.php');

class PwcImporter
{
	protected $log = array();
	protected $lock = false;
	protected $failure = array();

	var $db;
	var $processor;
	var $importPath = false;

	function PwcImporter($soapUser, $soapPass, $db=null)
	{
		if (!$soapUser || !$soapPass)
			throw new Exception('PwcImporter: constructor: missing user/pass');

		// prepare a soap client
		$this->_soapClient = new SoapClient(
			SOAP_WSDL_ACCOUNTANT_URL,
			array (
					'trace' => 1,
					'exceptions'=>1,
					'classmap'=>ELCSoapServer::getClassMap(),
					'features' => SOAP_SINGLE_ELEMENT_ARRAYS
			)
		);
		$this->_soapClient->authenticate($soapUser, $soapPass);

		$this->_db = $db;

	}

	function getLog()
	{
		return $this->log;
	}
	/**
	 * @param string $processor
	 */
	function setFileProcessor($processor)
	{
		$this->processor = $processor;
	}

	function run($dry_run=true)
	{
		if(!($pattern = $this->getRegExpPattern()))
			return false;

		if(!$this->importPath) {
			throw new Exception('No importPath found');
		}

		$date = currentDate();
		$subdir = $date->getYear().'-'.$date->getMonth().'-'.$date->getDay();

		if ($handle = opendir($this->importPath))
		{
			info('Starting import from directory '.$this->importPath);
			while (false !== ($file = readdir($handle)))
			{
				if ($file == '.' || $file == '..' || is_dir($this->importPath.$file) )
					continue;
				if (preg_match($pattern, $file, $matches))
				{
					if(!is_dir($this->importPath.$subdir))
					{
						info('Creating archive dir: '.$subdir);
						if(!mkdir($this->importPath.$subdir))
						{
							$subdir = false;
							alert('WARNING - Cannot create archive dir: '.$subdir.'  - not archiving');
						}
					}

					info('PwcImporter: processing file '.$file);
					info('PwcImporter: using processor '.$this->processor);

					$this->log[$file] = array();
					if($this->{$this->processor}($file, $matches, $dry_run))
					{
						if($subdir && !$dry_run)
						{
							if(rename($this->importPath.$file, $this->importPath.$subdir.'/'.$file))
								info('Archived file to: '.$subdir.'/'.$file);
							else
								alert('WARNING - Cannot archive file to: '.$subdir.'/'.$file);
						}
					}
				}
			}

		}
		closedir($handle);
		return true;
	}
	/**
	 * @abstract
	 */
	function getRegExpPattern()
	{
		return false;
	}


	public function importSuccessful()
	{
		return empty($this->failure);
	}

	public function getFailures()
	{
		return $this->failure;
	}

	public function import($dry_run=true)
    {
	$this->_validateLocks();

	if($this->lock)
		return false;
	$this->_lockDirectory();

	$this->run($dry_run);

	$this->_unlockDirectory();
	return empty($this->failure);
    }

    private function _lockDirectory()
    {
	// before we start, lets lock the directory
	$path = $this->importPath;
	$fp = fopen($path.'.lock','w+');
	fwrite($fp,date('U'));
	fclose($fp);
    }

    private function _unlockDirectory()
    {
	// we are done with the import process, unlock the directory
	$path = $this->importPath;
	unlink($path.'.lock');
    }

    private function _validateLocks()
    {
	$path = $this->importPath;
	if(!file_exists($path.'.lock')) // no lock exists, safe to continue
		return;

	// The lock expires eventually
	$fp = fopen($path.'.lock','r');
	$dt = fread($fp,1024);
	if($dt<(date('U')-IMPORT_LOCK_EXPIRY))
		return;

	$this->lock = true;
    }
}


/**
 * Handles the import of RefundExport Invoices and Declaration Invoices
 *
 * @package PwcImporter
 * @since 7-sep-2006
 * @author Marten Koedam <marten@dalines.net>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class PwcImporterDeclarationRefund extends PwcImporter
{
	var $importPath = ELC_INVOICES_IMPORT_PATH;
	/**
	 * @var string $processor
	 */
	var $processor = 'processInvoices';
	/**
	 * @var array $messageType
	 */
	/**
	 * @param string $file
	 * @param array $matches
	 */
	function processInvoices($file, $matches, $dry_run=true)
	{
		info("PwcImporterDeclarationRefund: processing '$file'");

		$method = ($matches[2]{0} == 'R') ? 'sendRefundInvoice':'sendDeclarationInvoice';
		$crso_id = (int) $matches[5];
		$participant_id = (int) $matches[6];

		$invoice_type = ($matches[1] == 'INV') ? 'debit' : 'credit';
		$crsoYearInvoiceIdStupidRedundantString = $matches[3];
		$yearInvoiceIdStupidString = substr($crsoYearInvoiceIdStupidRedundantString,strlen((int)$matches[5]));
		$year = ('20'.substr($yearInvoiceIdStupidString,0,2));
		$invoice_id = $lookup_id = (int) substr($yearInvoiceIdStupidString,2);

		$document = new SoapDocument($file, file_get_contents(ELC_INVOICES_IMPORT_PATH.$file));

		try {

			$this->_soapClient->{$method}($crso_id, $participant_id, $invoice_type, $year, $invoice_id, $document, $dry_run);
			return true;

		} catch (Exception $e) {

			// this is used in the interface to give feedback of the imported files
			alert($e->getMessage());
			$this->log[$file][] = $e->getMessage();
			$this->failure[] = $file;

			return false;
		}
	}

	/**
	 * Examples:
	 *
	 * Invoices						I2_INVI2070000000134_DEU0000000200000005_1124,42.signed.pdf
     * Credit notes 				C2_CRC2070000000044_DEU0000000200000023_-399,06.signed.pdf
     * Refund request invoices 		I2_INVRRI2070000000028_DEU0000000200000125_65,47.signed.pdf
	 * Refund request credit note 	C2_CRRRC2070000000044_DEU0000000200000172_-1740,18.signed.pdf
	 */
	function getRegExpPattern()
	{
		return '/^(?:I|RI|C|RC)\d+_(INV|CR)[R]{0,1}(RI|RC|I|C)([\d]+)_(\w{3})(\d{8})(\d{8})_([\-\d,.]*)\.signed\.pdf$/';
		// old pattern
		// return '/^(RI|RC|I|C)([0-9]+)_(\w{3})(\d{8})(\d{8})_([\w\d ]*)_(INV|CR)[R]{0,1}(RI|RC|I|C)([\d]+)_([\d\.]*)\.pdf/';
	}
}
/**
 * @package PwcImporter
 * @author Marten Koedam <marten@dalines.net>
 * @since 24-apr-2007
 * @copyright 2007 BaseSoft B.V.
 */
class PwcImporterReminder extends PwcImporter
{
	var $importPath = ELC_REMINDERS_IMPORT_PATH;
	/**
	 * @var string $processor
	 */
	var $processor = 'processReminders';
	/**
	 * Example: SREM3_DNK0000000300000079_10080479_191206_1.pdf
	 */
	function getRegExpPattern()
	{
		return '/^SREM([0-9]+)_(\w{3})(\d{8})(\d{8})_([\w\d ]*)_(\d{6})_([0-9]+)\.pdf$/';
	}
	/**
	 * @param $file
	 * @param array $matches
	 * @param bool $dry_run
	 */
	function processReminders($file, $matches, $dry_run=true)
	{

		try {

			$crso_id = $matches[1];
			$participant_id = $matches[4];
			$reminder_level = $matches[7];
			$d = $matches[6];
			$invoice_date = '20'.substr($d,4,2).'-'.substr($d,2,2).'-'.substr($d,0,2);
			$document = new SoapDocument($file, file_get_contents(ELC_REMINDERS_IMPORT_PATH.$file));

			// pwc specific validate matching ParticipantID and VAT NO
			$p = new Participant($this->_db,array('relation'=>new Relation($this->_db, $participant_id, true)));
			$pvat = str_replace(' ', '', $p->relation->vat_no);
			$importvat = str_replace(' ', '', $matches[5]);
			if($pvat != $importvat)
				throw new Exception(sprintf("VAT mismatch [%s] != [%s]", $pvat, $importvat));

			$this->_soapClient->sendReminderDocument($crso_id, $participant_id,
				$reminder_level, $invoice_date, $document, $dry_run
			);

			return true;

		} catch (Exception $e) {

			// this is used in the interface to give feedback of the imported files
			alert($e->getMessage());
			$this->log[$file][] = $e->getMessage();
			$this->failure[] = $file;

			return false;
		}

	}
}
/**
 * Trac Tickets #1527 and #1523
 * @package PwcImporter
 * @author Marten Koedam <marten@dalines.net>
 * @since 25th september 2007
 * @copyright 2007, BaseSoft Solutions B.V.
 */
class PwcImporterGuaranteeAndLate extends PwcImporter
{
	public $importPath = ELC_GUARANTEE_LATE_IMPORT_PATH;
	/**
	 * @var string $processor
	 */
	var $processor = 'processFiles';
	/**
	 * GI2_200207_GI2070000000001_DEU0000000200000004_439.546,42.signed.pdf
	 * GC2_200207_GC2070000000001_DEU0000000200000004_439.546,42.signed.pdf
	 *
	 * LI2_200207_LI2070000000001_DEU0000000200000004_439.546,42.signed.pdf
	 * LC2_200207_LC2070000000001_DEU0000000200000004_439.546,42.signed.pdf
	 */
	function getRegExpPattern()
	{
		return '/^(G|L)(I|C)(\d+)_(\d{6})_(?:G|L)(?:I|C)(\d+)_(\w{3})(\d{8})(\d{8})_([\d\.,]+)\.signed\.pdf$/';
	}
	/**
	 *
	 */
	function processFiles($file, $matches, $dry_run=true)
	{
		try
		{
			$method = ($matches[1]=='G') ? 'sendGuaranteeInvoice' : 'sendLatePaymentInvoice';

			$crso_id = $matches[3];
			$participant_id = $matches[8];
			$amount = str_replace(',','.',str_replace('.','',$matches[9]));
			$d = $matches[4];
			$invoice_date = '20'.substr($d,4,2).'-'.substr($d,2,2).'-'.substr($d,0,2);
			$document = new SoapDocument($file, file_get_contents(ELC_GUARANTEE_LATE_IMPORT_PATH.$file));

			$type = ($matches[2]=='I')?'invoice':'credit';
			$invoice = $matches[5];
			$invoice_year_id = substr($invoice,strlen((int)$matches[3]));
			$year = ('20'.substr($invoice_year_id,0,2));
			$invoice_id = (int) substr($invoice_year_id,2);

			$this->_soapClient->{$method}($crso_id, $participant_id,
				$type, $year, $invoice_id, $amount, $invoice_date, $document, $dry_run
			);

		} catch (Exception $e) {

			// this is used in the interface to give feedback of the imported files
			alert($e->getMessage());
			$this->log[$file][] = $e->getMessage();
			$this->failure[] = $file;

			return false;
		}
	}
}

/**
 * Trac Tickets #263
 * @package PwcImporter
 * @author Nick Loeve <nick@basesoft.com>
 * @since 18th Jan  2008
 * @copyright 2008, BaseSoft Solutions B.V.
 */
class PwcImporterAgeing extends PwcImporter
{

	public $importPath = ELC_AGEING_IMPORT_PATH;
	/**
	 * @var string $processor
	 */
	var $processor = 'processFiles';
	/**
	 * 20080118_2.xml
         * 20080218_163.xml
	 */
	function getRegExpPattern()
	{
		return '/^(\d{4})(\d{2})(\d{2})_(\d+)\.xml$/';
	}
	/**
	 *
	 */
	function processFiles($file, $matches, $dry_run=true)
	{
		$crso_id = $matches[4];
		$lines = array();

        // we set this so when PwC send bad XML we don't flunky all over the place
        libxml_use_internal_errors(true);
        $sxml = simplexml_load_file($this->importPath.$file);
        if(!$sxml)
        {
            $errors = libxml_get_errors();
            $this->failure[$file] = $file;
            $this->log[$file][] = 'WARNING - XML file is not valid!';
            foreach($errors as $error)
            {
                $this->log[$file][] = 'WARNING - '.trim($error->message)." - Line: ".$error->line;
            }
            libxml_clear_errors();
            return false;
        }

		$lines = array();
        foreach($sxml->DataItem as $item)
        {
            //now we do the checks for each ledger
            //for some reason PwC saw fit to include a whole bunch of
            //unrelated data that we have to sift through...

            //make sure we only do ledgers
            if($item['Name'] != 'Customer Ledger Entry')
                continue;

            //is this still outstanding?
            if(!isset($item->Fields->{'FullySettled'}) || trim((string)$item->Fields->{'FullySettled'}) == 'Yes')
                continue;

            //make sure customer number is correct format
            $customerMatches = null;
            if(!preg_match('/^[A-Z]{3}(\d{8})(\d{8})$/', (string)$item->Fields->{'CustomerNo.'}, $customerMatches))
            {
                $this->failure[$file] = $file;
                $this->log[$file][] = 'WARNING - '.((string)$item->Fields->{'CustomerNo.'}).' is not a valid cutomer number!';
                continue;
            }

            if((int)$customerMatches[1] != $crso_id)
            {
                $this->failure[$file] = $file;
                $this->log[$file][] = 'WARNING - Customer Number '.((string)$item->Fields->{'CustomerNo.'}).' does not match CRSO of import file!!';
                continue;
            }

            //now we work out the date due compared to the current date
            //check they supplied it (it is PwC you know...)
            if(!isset($item->Fields->{'DueDate'}) || trim((string)$item->Fields->{'DueDate'}) == '')
            {
                $this->failure[$file] = $file;
                $this->log[$file][] = 'WARNING - DueDate must be set for unsettled ledgers!';
                continue;
            }

            //now we need to check that they supplied the amounts
            //and if they did convert them to MySQL number formats
            if(!isset($item->Fields->{'Amount'}) || trim((string)$item->Fields->{'Amount'}) == '')
            {
                $this->failure[$file] = $file;
                $this->log[$file][] = 'WARNING - Amount must be set for unsettled ledgers!';
                continue;
            }
            if(!isset($item->Fields->{'OutstandingAmount'}) || trim((string)$item->Fields->{'OutstandingAmount'}) == '')
            {
                $this->failure[$file] = $file;
                $this->log[$file][] = 'WARNING - OutstandingAmount must be set for unsettled ledgers!';
                continue;
            }

			$line = new SoapAgeingLine();
            $line->participantId = $customerMatches[2];
            $line->journalCode = isset($item->Fields->{'JournalCode'}) && trim((string)$item->Fields->{'JournalCode'}) != '' ? trim((string)$item->Fields->{'JournalCode'}) : null;
            $line->invoiceNr = isset($item->Fields->{'DocumentNo.'}) && trim((string)$item->Fields->{'DocumentNo.'}) != '' ? trim((string)$item->Fields->{'DocumentNo.'}) : null;
            $line->invoiceDate = trim((string)$item->Fields->{'DocumentDate'}) != '' ? $this->scrobbleDate(trim((string)$item->Fields->{'DocumentDate'})) : null;
            $line->postingDate = isset($item->Fields->{'PostingDate'}) && trim((string)$item->Fields->{'PostingDate'}) != '' ? $this->scrobbleDate(trim((string)$item->Fields->{'PostingDate'})) : null;
            $line->dueDate = $this->scrobbleDate(trim((string)$item->Fields->{'DueDate'}));
            $line->amount = str_replace(',', '.', str_replace('.', '', trim((string)$item->Fields->{'Amount'})));
            $line->amountOutstanding = str_replace(',', '.', str_replace('.', '', trim((string)$item->Fields->{'OutstandingAmount'})));

            $lines[] = $line;
        }

        try {
			$this->_soapClient->sendAgeing($crso_id, $lines, $dry_run);
		} catch (Exception $e) {
			// this is used in the interface to give feedback of the imported files
			alert($e->getMessage());
			$this->log[$file][] = $e->getMessage();
			$this->failure[] = $file;

			return false;
		}
		return true;
	}

    function scrobbleDate($date)
    {
        $bits = explode('/', $date);
        $bits[2] = '20'.$bits[2];
        $rv = join('-', array_reverse($bits));
        return $rv;
    }
}


define('IMPORT_LOCK_EXPIRY',7200);
?>
