<?php
require_once(IWARE.'AbstractObject.php');
/**
 * A way to give the Operator credit/debit ids for certain objects (Declaration/RefundExport)
 *
 * This way they can correct mistakes that have been invoiced/credited
 * Related to Trac tickets: #326, #349, #385,
 *
 * If at any time we want to add more credit/debits make sure that the new Object is compatible with
 *  standard function calls used in this class (insertRequestedInvoice)
 *
 * @package ELC
 * @subpackage Invoice
 * @since 31-aug-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class RequestedInvoice extends AbstractObject
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @var Participant $participant
	 */
	var $participant;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Document $document
	 */
	var $document;
	/**
	 * @var date $request_datetime
	 */
	var $request_datetime;
	/**
	 * @var int $month (1-12)
	 */
	var $month;
	/**
	 * @var int $year (4 digits)
	 */
	var $year;
	/**
	 * The table in which the class should search for the id
	 * @var string $tbl
	 */
	var $tbl;
	/**
	 * The prefix of the credit/debit id (for declaration this is the same as $tbl)
	 * @var string $field
	 */
	var $field;
	/**
	 * Sets a RequestedInvoice by it's ID, returns false if it cannot be found
	 * @param int $id
	 * @return bool
	 */
	function set($id=null)
	{
		$id || $id = $this->id;
		$sql = "SELECT `PARTICIPANT_RELATION_ID`, `CRSO_RELATION_ID`,
				`DOCUMENT_ID`, `REQUEST_DATETIME`, `MONTH`, `YEAR`
			FROM TBL_REQUESTED_INVOICE
			WHERE REQUESTED_INVOICE_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		;
		$this->id = $id;
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Document.php');

		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->document = $r->DOCUMENT_ID?new ELCDocument($this->db,$r->DOCUMENT_ID):null;
		$this->request_datetime = $r->REQUEST_DATETIME;
		$this->month = $r->MONTH;
		$this->year = $r->YEAR;
		return $this->complete = true;
	}
	/**
	 * Set the table where this class should be looking for it's ID
	 * @param string $tbl
	 */
	function setTbl($tbl)
	{
		$this->tbl = $tbl;
		$this->field = ($tbl=='REFUND_EXPORT')?'REFUND_REQUEST':$tbl;
	}
	/**
	 * Gets the RequestedInvoiceId by DebitId (on $tbl)
	 * @param int $debit_id
	 * @param int $crso_id
	 * @return int|bool
	 */
	function getByDebitID($debit_id,$crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($debit_id,$crso_id,'DEBIT',$year);
	}
	/**
	 * Gets the RequestedInvoiceId by CreditId (on $tbl)
	 * @param int $credit_id
	 * @param int $crso_id
	 * @return int|bool
	 */
	function getByCreditID($credit_id,$crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($credit_id,$crso_id,'CREDIT',$year);
	}
	/**
	 * Gets the RequestedInvoiceId by $typeId (on $tbl)
	 * @param int $id
	 * @param int $crso_id
	 * @param string $type
	 * @return int|bool
	 */
	function getByCreditOrDebitID($id,$crso_id,$type,$year)
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		if (!is_numeric($crso_id))
			trigger_error('RequestedInvoice::getByDebitID() - CRSO is not numeric', E_USER_ERROR);

		$sql = "SELECT REQUESTED_INVOICE_ID
			FROM TBL_".$this->tbl."_".$type."
			WHERE ".$this->field."_".$type."_ID = ?
				AND CRSO_RELATION_ID=?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id, $crso_id,$year);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$rv = array();
		foreach($rs as $r)
			$rv[] = $r->REQUESTED_INVOICE_ID;
		return $rv;
	}

	function getInvoiceOrCreditId($type, $id=null)
	{
		if(is_null($id))
			$id = $this->id;

		$sql = "SELECT ".$this->field."_".$type."_ID
			FROM TBL_".$this->tbl."_".$type."
			WHERE REQUESTED_INVOICE_ID  = ?";


		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		if(!$r = $q->fetch())
		     throw new Exception('getInvoiceOrCreditId: no invoice found');

		return $r->{$this->field."_".$type."_ID"};
	}

	/**
	 * Add a new RequestedInvoice to the database.
	 *
	 * Link it to $obj by calling a default function insertRequestedInvoice
	 * @param mixed &$obj
	 * @param string $type
	 * @param int $participant_id
	 * @return bool
	 */
	function addId($obj,$type,$participant_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;

		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$participant_id)));
		$this->crso =& $obj->crso;
		$this->request_datetime = date('Y-m-d H:i:s');
		if(!$this->insert())
			return false;

		// ALWAYS USE THE CURRENT YEAR!!!
		$d = currentDate();
		$year = $d->getYear();

		if(!$obj->insertRequestedInvoice(strtoupper($type),$this->id,$year))
			return false;

		return true;
	}
	/**
	 * Gets the overview of all the RequestedInvoices of Declarations for $type
	 * @param string $type
	 * @param int $crso_id
	 * @return ResultSet
	 */
	function getDeclarationOverview($type,$crso_id)
	{
		$this->setTbl('DECLARATION');
		return $this->_getOverview($type,$crso_id);
	}
	/**
	 * Gets the overview of all the RequestedInvoices of RefundExports for $type
	 * @param string $type
	 * @param int $crso_id
 	 * @return ResultSet
	 */
	function getRefundexportOverview($type,$crso_id)
	{
		$this->setTbl('REFUND_EXPORT');
		return $this->_getOverview($type,$crso_id);
	}
	/**#@+
	 * @access protected
	 */
	/**
	 * @param string $type
	 * @param int $crso_id
	 * @param string $idfield
	 * @return ResultSet $q
	 */
	function _getOverview($type,$crso_id,$idfield=null)
	{
		$pre = ($this->field=='DECLARATION')?($type=='DEBIT'?'INV':'CR'):($type=='DEBIT'?'INVR':'CRR');
		$sql = "SELECT IR.REQUESTED_INVOICE_ID,
				RTRIM(CONCAT(
					'".$pre.(($this->field=='DECLARATION')?'':'R')."','".($type=='DEBIT'?'I':'C')."',T.CRSO_RELATION_ID,
					SUBSTRING(T.`YEAR`, -2),
					T.".$this->field.'_'.$type."_ID)) as '".$this->field.'_'.$type."_ID',
			R.`NAME` as PARTICIPANT_NAME, REQUEST_DATETIME, `MONTH`, IR.`YEAR`
			FROM TBL_".$this->tbl."_".$type." as T
			INNER JOIN TBL_REQUESTED_INVOICE as IR
				ON T.REQUESTED_INVOICE_ID = IR.REQUESTED_INVOICE_ID
			INNER JOIN TBL_RELATION as R
				ON IR.PARTICIPANT_RELATION_ID = R.RELATION_ID
			WHERE T.CRSO_RELATION_ID = ?
			ORDER BY IR.REQUESTED_INVOICE_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
	}
	/**
	 * Initializes the objectmaps for AbstractObject
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_REQUESTED_INVOICE';
		$this->objectmap['primary']['id'] = 'REQUESTED_INVOICE_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['request_datetime'] = 'REQUEST_DATETIME';
		$this->objectmap['properties']['month'] = 'MONTH';
		$this->objectmap['properties']['year'] = 'YEAR';
	}
	/**#@-*/
}
?>
