<?php
require_once(IWARE.'AbstractObject.php');
require_once(IWARE.'xmlrpc.php');
/**
 * Trac User abstract object.
 * @author marten
 * Copyright 2010, BaseSoft B.V.
 */
class TracUser extends AbstractObject {
	var $id;
	var $user_name;
	var $password;
	var $created;
	/**
	 * Set the TracUser.
	 * @param int $id
	 */
	function set($id=null) {
		$id || $id = $this->id;

		$sql = "SELECT TRAC_USER_NAME, TRAC_USER_PASSWORD, CREATED
			FROM TBL_TRAC_USER
			WHERE TRAC_USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		$this->user_name = $r->TRAC_USER_NAME;
		$this->password = $r->TRAC_USER_PASSWORD;
		$this->created = $r->CREATED;

		return $this->complete = true;
	}
	/**
	 * @see AbstractObject
	 */
	function _initObjectMaps() {
		$this->objectmap['dbtable'] = 'TBL_TRAC_USER';
		$this->objectmap['primary']['id'] = 'TRAC_USER_ID';
		$this->objectmap['properties']['user_name'] = 'TRAC_USER_NAME';
		$this->objectmap['properties']['password'] = 'TRAC_USER_PASSWORD';
		$this->objectmap['properties']['created'] = 'CREATED';
	}
}

/**
 * XML RPC client for trac.
 *
 * Copyright 2010, BaseSoft B.V.
 *
 *
 * $test = new TracClient('trac.office.basesoft.com', 'blackboxrpc', 'WruS4eSt');
 * print $test->createTicket('SUmmary from PHP', 'Description from PHP alsop');
 * print_r($test->getTicketOverview());
 * print_r($test->getTicketDetails(31));
 */
class TracClient {
	/**
	 * @var xmlrpc_client $client
	 */
	var $client;
	/**
	 * @var array $hooks
	 */
	var $hooks = array(
		'totalhours'=>'convertHours'
	);
	/**
	 * @var array $ticket_fields
	 */
	var $ticket_fields = array(
		'reporter' => 'Reported by',
		'owner' => 'Owner',
		'priority' => 'Priority',
		'status' => 'Status',
		'type' => 'Type',
		'totalhours' => 'Total hours',
		'resolution' => 'Resolution'
	);
	/**
	 * Contructor, setup the connection to the server.
	 *
	 * @param string $host
	 * @param string $user
	 * @param string $password
	 */
	function TracClient($host, $user, $password, $person=null, $port=80) {
		$this->user = $user;
		$this->person = $person;
		$this->connect($host, $user, $password, $port);
	}
	/**
	 *
	 * @param string $host
	 * @param string $user
	 * @param string $password
	 * @param int $port
	 */
	protected function connect($host, $user, $password, $port=80) {
		$this->client = new xmlrpc_client(
			sprintf('%s/login/xmlrpc', TRAC_INSTANCE),
			$host,
			$port,
			'http'
		);
		$this->client->setCredentials($user, $password);
		$this->client->return_type = 'phpvals';
		$this->client->request_charset_encoding = 'UTF-8';
	}
	/**
	 * Send the message to the xmlrpc server and throws an exception if an error happened.
	 * @param xmlrpcmsg $message
	 * @return mixed
	 */
	protected function send($message) {
		$resp = $this->client->send($message);
		if($resp->faultCode())
			throw new Exception($resp->faultString());
		return $resp->value();
	}
	/**
	 * Get all the priorities as they are defined in the trac Instance
	 */
	public function getPriorities() {
		$message = new xmlrpcmsg('ticket.priority.getAll');
		return $this->send($message);
	}
	/**
	 * Get all the ticket types as they are defined in the trac Instance
	 */
	public function getTypes() {
		$message = new xmlrpcmsg('ticket.type.getAll');
		return $this->send($message);
	}
	/**
	 * Create a new ticket in the trac system with summary and description
	 * @param string $summary
	 * @param string $description
	 * @return int $ticket_id
	 */
	public function createTicket($summary, $description, $priority=null, $type=null) {

		$values = array(
			new xmlrpcval($summary, "string"),
			new xmlrpcval($description, "string")
		);
		$attributes = array('escalate' => new xmlrpcval(false, "boolean"));
		if($this->person) {
			$this->person->set();
			$attributes['cc'] = new xmlrpcval($this->person->email, "string");
		}

		if($priority)
			$attributes['priority'] = new xmlrpcval($priority, "string");
		if($type)
			$attributes['type'] = new xmlrpcval($type, "string");

		$values[] = new xmlrpcval($attributes, "struct");
		$values[] = new xmlrpcval(true, "boolean"); // Notify is on

		$message = new xmlrpcmsg("ticket.create", $values);
		return $this->send($message);
	}
	/**
	 * Add a comment to the ticket
	 * @param int $id
	 * @param string $comment
	 */
	public function addComment($id, $comment) {
		$message = new xmlrpcmsg("ticket.update", array(
			new xmlrpcval($id, "int"),
			new xmlrpcval($comment, "string"),
			new xmlrpcval(array('action'=> new xmlrpcval('leave')), "struct"),
			new xmlrpcval(true, "boolean")
		));
		return $this->send($message);
	}
	/**
	 * Reopens a previously closed ticket
	 * @param int $id
	 * @param string $comment
	 */
	public function reOpen($id, $comment) {
		$message = new xmlrpcmsg("ticket.update", array(
			new xmlrpcval($id, "int"),
			new xmlrpcval($comment, "string"),
			new xmlrpcval(array('action'=> new xmlrpcval('reopen')), "struct")
		));
		return $this->send($message);
	}
	/**
	 * Checks if the ticket with the supplied ID is reported by the current user.
	 * @param int $ticket_id
	 */
	public function isReporter($ticket_id) {
		$message = new xmlrpcmsg('ticket.query', array(
			new xmlrpcval(sprintf('reporter=%s&id=%s', $this->user, $ticket_id), "string")
		));
		$rv = $this->send($message);
		return count($rv)===1;
	}

	/**
	 * This gets all the tickets owned by the current user.
	 * @param int $offset from which index do we start (for paging purposes)
	 * @param int $limit or null. What is the number of items per page?
	 * @return array
	 */
	public function getTicketOverview($offset=0, $limit=null) {

		$message = new xmlrpcmsg('ticket.query', array(
			new xmlrpcval(sprintf('reporter=%s&order=changetime&desc=1', $this->user), "string")
		));
		$tickets = $this->send($message);
		$messages = array();
		foreach($tickets as $index => $ticket_id) {

			if($index < $offset)
				continue;
			if($limit && ($index >= $offset+$limit))
				break;

			$messages[] = new xmlrpcmsg('ticket.get', array(
				new xmlrpcval($ticket_id, "int")
			));

		}

		$contents = $this->client->multicall($messages);
		$rv = array();
		foreach($contents as $xmlresp) {
			$ticket = $xmlresp->value();
			$rv[] = array(
				'ID'=> $ticket[0],
				'Ticket Summary'=> $ticket[3]['summary'],
				'Ticket Type'=> $ticket[3]['type'],
				'Ticket Status'=> $ticket[3]['status'],
				'Ticket Priority'=> $ticket[3]['priority'],
				'Ticket Created'=> $this->convertDateTime($ticket[1]),
				'Ticket Last Updated'=> $this->convertDateTime($ticket[2])
			);
		}
		return array($rv, count($tickets));
	}
	/**
	 * Get all the changes on the ticket.
	 * @param int $id
	 * @return array
	 */
	public function getTicketDetails($id) {

		$messages = array(
			// Get the latest ticket attributes
			new xmlrpcmsg('ticket.get', array(
				new xmlrpcval($id, "int")
			)),
			// Get the changelog, this includes the comments and stuff.
			new xmlrpcmsg('ticket.changeLog', array(
				new xmlrpcval($id, "int")
			))
		);

		$result = $this->client->multicall($messages);
		$ticket = $result[0]->value();

		$rv = array();
		$rv['ticket'] = array(
			'Created' => $this->convertDateTime($ticket[1]),
			'Last Updated' => $this->convertDateTime($ticket[2]),
		);
		foreach($this->ticket_fields as $field => $handle) {
			$rv['ticket'][$handle] = isset($ticket[3][$field]) ? $this->executeHook($field, $ticket[3][$field]) : '';
		}
		$rv['Summary'] = $ticket[3]['summary'];
		$rv['Description'] = $ticket[3]['description'];

		$changelog = $result[1]->value();
		$cl = array();
		foreach($changelog as $values) {
			if(!array_key_exists($values[0], $cl)) {
				$cl[$values[0]] = array();
				$cl[$values[0]]['reporter'] = $values[1];
				$cl[$values[0]]['changes'] = array();
			}
			if($values[2]=='comment') {
				$cl[$values[0]]['comment'] = $values[4];
				continue;
			}
			if(!array_key_exists($values[2], $this->ticket_fields)) {
				continue;
			}
			$cl[$values[0]]['changes'][$this->ticket_fields[$values[2]]] = array(
				'old'=>$this->executeHook($values[2], $values[3]),
				'new'=>$this->executeHook($values[2], $values[4])
			);
		}
		$rv['changelog'] = $cl;

		return $rv;
	}
	/**
	 * Convert the totalhours field from a floating point to human readable hours vs minutes.
	 * @param float $hours
	 * @return string $rv
	 */
	protected function convertHours($hours) {
		if(!$hours)
			return '';
		$rv = '';
		$h = (int) $hours;
		$m = (int) (($hours - $h)*60);
		$rv .= $h.'h';
		if($m) {
			$rv .= ' '.$m.'m';
		}
		return $rv;
	}
	/**
	 * Convert the trac timestamp to a more user friendly format.
	 * @param string $datetime
	 * @return string
	 */
	public function convertDateTime($datetime) {
		$dt = new IWDateTime($datetime);
		// EEP, crappy PHP Date objects, haxx0rz
		$dt->setTimezone(new DateTimeZone('Europe/Amsterdam'));
		$u = ((int)$dt->format('U'))+$dt->getOffset();
    	return date('Y-m-d H:i', $u);
	}
	/**
	 * Execute a hook that was set for a certain trac field
	 * @param string $field
	 * @param mixed $value
	 * @return mixed
	 */
	protected function executeHook($field, $value) {
		if(!array_key_exists($field, $this->hooks))
			return $value;
		return $this->{$this->hooks[$field]}($value);
	}
	/**
	 * Formats a string in wiki format to the HTML version
	 * @param string $txt
	 * @return string
	 */
	public function format($txt) {
		// Disabled due to UTF8 conversion errors
		return nl2br($txt);
		/*$message = new xmlrpcmsg('wiki.wikiToHtml', array(
			new xmlrpcval($txt, "string")
		));
		return $this->send($message);*/
	}
}

?>
