<?php
/**
 * Abstract IWare Class
 * Use for DB related Objects.
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @copyright Copyright 2004, BaseSoft b.v.
 * @package I-Ware
 * @version $Id$
 * @abstract
 */
class AbstractObject
{
	/**
	 * @var Query last (failed) Query executed. Inspect $lastQuery->exception and $lastQuery->tmp_sql for details.
	 */
	var $lastQuery = null;
	var $lastError = null;

	public function __toString()
	{
		return get_class($this).(isset($this->id) ? '['.$this->id.']' : '');
	}

	/**
	 * Constructor
	 * @param Database $db
	 * @param mixed $preset int (id) or array ('property'=> value);
	 * @param boolean $set call {@link set()}?
	 */
	function AbstractObject(IWDatabase $db, $preset=null, $set=false)
	{
		$this->db = $db;
		$this->complete = false;
		if ( is_numeric($preset) )
			$this->id = (int) $preset;
		elseif ( is_array($preset) )
			$this->_preset($preset);
		$this->_initObjectMaps();
		if ( $set )
			$this->set();
	}
	/**
	 * Set the object from DB
	 * @param mixed $key int (id) or array ('property'=> value);
	 */
	function set($key=null)
	{
		if ( is_numeric($key) )
			$this->id = (int) $key;
		elseif ( is_array($key) )
			$this->_preset($key);

		$values = array();
		$sql = "SELECT * FROM `".$this->objectmap['dbtable']."`";
		$this->_appendWhereAllPrimaries($sql, $values);
		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		if ( $r = $q->fetch() )
		{
			foreach ( array('primary', 'properties') as $proptype )
				foreach ( $this->objectmap[$proptype] as $prop=>$field )
				{
					$this->$prop = $r->$field;
					if ( isset($this->typemap[$prop]) && !is_null($this->$prop) )
						settype($this->$prop, $this->typemap[$prop]);
				}
			return $this->complete = true;
		}
		return false;
	}
	/**
	 * Alter the object properties with stuff from the array/object
	 *
	 * You can't unset properties this way, but you can set them to null or '';
	 * @param mixed array('property'=>value) or object
	 */
	function alter($settings)
	{
		$this->_preset($settings);
	}
	/**
	 * Insert the object in the DB as a new
	 * @param bool $graceful Don't die with a Fatal, but return false and set {@link $this->lastQuery}
	 * @return bool
	 */
	function insert($graceful=false)
	{
		do
		{
			isset($this->objectmap['inserttype']) || $unsetInsertType = $this->objectmap['inserttype'] = 'INSERT'; //could be set to REPLACE

			if ( !(@list($q, $id) = $this->_buildIntoSetQuery($this->objectmap['inserttype'])) )
				continue; //CONSTRAINT broken _nextTable

			if ( $q->execute() )
			{
				if ( $id && empty($this->$id) && $q->lastid() )
					$this->$id = $q->lastid();
				$this->complete = true;
			}
			else
			{
				$this->lastQuery = &$q;
				$this->setLastError($q->exception);
				return false; //if _nextTable.. DB is in a filthy state!! But hey, isn't that the fun of MySQL
			}
		} while ( $this->_nextTable() );
		//reset state to original. so Unittests get what they expect...
		if ( isset($unsetInsertType) )
			unset($this->objectmap['inserttype']);
		unset($this->objectmap0);
		return $this->complete;
	}
	/**
	 * Updates the database
	 */
	function update()
	{
		$rv = true;
		do
		{
			$values = array();
			if ( isset($this->objectmap['updatetype']) && $this->objectmap['updatetype'] == 'REPLACE' )
			{
				//TRY BUILD REPLACE QUERY
				if ( !(@list($q, $id) = $this->_buildIntoSetQuery('REPLACE')) )
					continue; //CONSTRAINT broken _nextTable
			}
			else
			{ // BUILD UPDATE QUERY
				$sql = 'UPDATE `'.$this->objectmap['dbtable'].'`';
				$values = array();
				if ( !$this->_appendSet($sql, $values) )
					continue; //constraint broken or no values set
				$this->_appendWhereAllPrimaries($sql, $values);

				$q = $this->db->query($sql);
				$q->bind_values($values);
			}
			// EXECUTE
			$this->lastQuery =& $q;
			$rv &= $q->execute();
		} while ( $this->_nextTable() );
		unset($this->objectmap0); //return state to same before call. for Unittests
		return $rv;
	}
	/**
	 * Deletes element from Database
	 * @return boolean
	 */
	function delete()
	{
		// COmmented out because of PHP 5.2, can only be used if the __toString() function is added to the objects.
		debug('Deleting '.$this.' from database');

		$refs = array();

		//if ( !empty($this->referencemap) )
			//$refs = $this->_checkReferences();

		//if ( $refs === false )
			//return false;

		//DELETE direct object stuff
		do
		{
			$values = array();
			$sql = 'DELETE FROM `'.$this->objectmap['dbtable'].'`';
			$this->_appendWhereAllPrimaries($sql, $values);
			$q = $this->db->query($sql);
			$this->lastQuery =& $q;
			$q->bind_values($values);
			debug('Executing '.$q->tmp_sql);
			$rv = $q->execute();
		} while ( $this->_nextTable() );

		// delete ondelete cascade references
		// disabled because now we use innodb
		#foreach ( $refs as $sql )
		#{
		#	$q = $this->db->query($sql);
		#	$q->execute();
		#}

		return $rv;
	}
	/**
	 * Fills object with children according to familymap
	 * @param bool $cascade Fill children too?
	 * @param bool $doublelink Double link? (set parentprop on children to a ref to $this)
	 * @param progress_bar $progressbar
	 */
	function fill($cascade=false, $doublelink=true, $barname=null)
	{
		//NASTY... but copy(&$progressbar=null) is illegal, and we need a reference.
		$progressbar = null;
		$barname && $progressbar = $GLOBALS[$barname];
		if ( !isset($this->familymap) || empty($this->familymap['children']) )
			return;
		foreach ( $this->familymap['children'] as $class => $props )
		{
			require_once($props['classfile']);
			$sql = 'SELECT * FROM `'.$props['dbtable'].'` WHERE ';
			foreach ( $props['key'] as $property => $field )
			{
				$sql .= '`'.$field.'` = ? AND ';
				eval('$values[] = $this->'.$property.';');
			}
			$sql = substr($sql, 0, -5);
			$q = $this->db->query($sql);
			$q->bind_values($values);
			if ( $q->execute() )
			{
				while ( $r = $q->fetch() )
				{
					foreach ( $props['properties'] as $childprop=>$attr)
					{
						#if (!class_exists($attr['class']))
						#	trigger_error('class '.$attr['class'].' does not exist');

						if(is_array($attr) && (sizeof($attr)==2) && class_exists($attr['class']))
							$preset[$childprop] = new $attr['class']($this->db,$r->$attr['val']);
						else {
							$preset[$childprop] = $r->$attr;
						}
					}

					$child = new $class($this->db, $preset);
					$progressbar && $progressbar->step($class.$child->id);
					$child->set();

					if ( $doublelink )
					{
						$parentprop = $child->getParentProperty($this);

						if ($parentprop === false)
							trigger_error('parentprop is false', E_USER_ERROR);

						$child->$parentprop =& $this;
					}

					$this->{$props['property']}[] = $child;
				}
			}
			if ( $cascade )
				for ( $i = 0 ; $i < count($this->{$props['property']}) ; $i++ )
				{
					$child =& $this->{$props['property']}[$i];
					$child->fill($cascade, $doublelink, $barname);
				}
		}
		$this->filled = true;
	}
	/**
	 * Create copy of the object..
	 *
	 * The steps should be
	 * <code>
	 * $object = new AbstractObject();
	 * $object->set($origin_id);
	 * $object->fill(true); //or false if you don't want to "deep-copy";
	 * //Change things, like name or parent;
	 * $object->name = 'Copy of '.$object->name;
	 * $object->parent = $current_building_or_something;
	 * //then copy
	 * $object->copy();
	 * </code>
	 *
	 * Progressbar looks really nasty... Needs a rehaul if requested.
	 * @see hotcopy()
	 * @obsolete
	 * @param string $progressbar The name of the {@link progress_bar} var in the global scope
	 */
	function copy($progressbar=null)
	{
		//NASTY... but copy(&$progressbar=null) is illegal, and we need a reference.
		$progressbar && $progressbar = $GLOBALS[$progressbar];
		if ( !isset($this->familymap) )
			return;
		if ( !$this->complete )
			die ('Fill me first!');
		foreach ( $this->objectmap['primary'] as $prop=>$attr )
			$this->$prop = null;
		$this->insert();
		if ( !empty($this->familymap['children']) )
			foreach ( $this->familymap['children'] as $class => $props )
			{
				if ( !empty($this->{$props['property']}) )
					for ( $i = 0 ; $i < count($this->{$props['property']}) ; $i++ )
					{
						$child =& $this->{$props['property']}[$i];
						$child->copy();
						$progressbar && $progressbar->step();
					}
			}
	}
	/**
	 * Create copy of the object..
	 * But with lower memory footprint. Especially for large objecttrees.
	 * Obsoletes the need to "fill" the object
	 *
	 * Not sure if this works with familymap properties of the form "someproperty->something"
	 * But I guess these seldomly occur in primary keys..
	 * Anyway.. the fill, copy manouvre still works in those cases.
	 *
	 * The steps should be
	 * <code>
	 * $object = new AbstractObject();
	 * $object->set($origin_id);
	 * //Change things, like name or parent;
	 * $object->name = 'Copy of '.$object->name;
	 * $object->parent = $current_building_or_something;
	 * //then copy
	 * $object->hotcopy();
	 * </code>
	 */
	function hotcopy()
	{
		if ( !isset($this->familymap) )
			return;
		if ( !$this->complete )
			$this->set();

		//rememeber original primary keys
		foreach ( $this->objectmap['primary'] as $prop=>$attr )
		{
			$this->{'hotcopy_orig'.$prop} = $this->$prop;
			$this->$prop = null;
		}
		//insert new object
		$this->insert();

		if ( !empty($this->familymap['children']) )
			foreach ( $this->familymap['children'] as $class => $props )
			{
				require_once($props['classfile']);
				$sql = 'SELECT * FROM `'.$props['dbtable'].'` WHERE ';
				foreach ( $props['key'] as $property => $field )
				{
					$sql .= '`'.$field.'` = ? AND ';
					//use original primary;
					eval('$values[] = $this->hotcopy_orig'.$property.';');
				}
				$sql = substr($sql, 0, -5);
				$q = $this->db->query($sql);
				$q->bind_values($values);
				if ( $q->execute() )
					while ( $r = $q->fetch() )
					{
						foreach ( $props['properties'] as $childprop=>$attr)
							$preset[$childprop] = $r->$attr;
						$child = new $class($this->db, $preset);
						$child->set();
						//set original parent to this.
						$parentprop = $child->getParentProperty($this);
						$child->$parentprop =& $this;
						//and do again.
						$child->hotcopy();
					}
			}
	}
	/**
	 * @param object
	 * @return string Property which contains the parent represented by $parent
	 */
	function getParentProperty($parent)
	{
		if ( isset($this->familymap['parent'][get_class($parent)]) )
			return $this->familymap['parent'][get_class($parent)];

		return false;
	}
	/**
	 * @return bool
	 */
	function isComplete()
	{
		return $this->complete;
	}
	function setComplete()
	{
		$this->complete = true;
	}
	/**
	 * Set properties from array
	 * @param array $props ('property' => value)
	 */
	function setProperties($props)
	{
		return $this->_preset($props);
	}
	/**
	 * @access protected
	 */
	function getLastError()
	{
		return $this->lastError;
	}
	/**
	 * @access protected
	 */
	function setLastError($sErr)
	{
		$this->lastError = $sErr;
	}
	/**#@+
	 * @access private
	 */
	/**
	 * Set properties from array
	 * @param array $props ('property' => value)
	 */
	function _preset($props)
	{
		foreach ( $props as $prop=>$val )
				$this->$prop = $val;
	}
	/**
	 * Set map to next table if appliccable
	 *
	 * You should always cycle to call this function untill it returns false (and sets the first map again).
	 * Call this with reset true.
	 * Or it will render the map in an unpredictable state. For possible future calls.
	 * It uses seperate static counters per maptype, so mixed calling can take place.
	 *
	 * @staticvar int $objectmapi Next objectmap postfix
	 * @staticvar int $referencemapi Next referencemap postfix
	 * @param string $maptype 'objectmap'|'referencemap'
	 * @param bool $reset reset d
	 * @return bool true if a new map has been set, false if previous map was the last (and the first is set again)
	 */
	function _nextTable($maptype='objectmap', $reset=false)
	{
		//static $objectmapi = 1; //seems statics can't be variable variables
		//static $referencemapi = 1;
		if ( !$reset )
		{
			if ( !in_array($maptype, array('objectmap','referencemap')) )
				trigger_error('AbstractObject::_nextTable() : Unknown maptype.', USER_ERROR); //Fatal
			if ( self::${$maptype.'i'} == 1 )
			{
				// no idea what this line does, it used to do this:
				// $this->{$maptype.'0'} = self::$this->$maptype;
				$this->{$maptype.'0'} = $this->$maptype;

			}
			if ( isset($this->{$maptype.self::${$maptype.'i'}}) )
			{
				$this->$maptype = $this->{$maptype.self::${$maptype.'i'}};
				self::${$maptype.'i'}++;
				return true;
			}
		}
		if ( self::${$maptype.'i'} != 1 )
		{
			self::${$maptype.'i'} = 1;
			$this->$maptype = $this->{$maptype.'0'};
		}
		return false;
	}

	public static $objectmapi = 1;
	public static $referencemapi = 1;

	/**
	 * @param String $command INSERT|REPLACE
	 * @return Array ((Query) $q, (string) IdProperty)
	 */
	function _buildIntoSetQuery($command)
	{
		$lastid = '';
		$values = array();
		$sql = $command.' INTO `'.$this->objectmap['dbtable'].'`';

		$appended = $this->_appendSet($sql, $values);
	   	if ( $appended === false )
			return false;
		if ( $appended )
			$sql .= ',';

		foreach ( $this->objectmap['primary'] as $property => $field )
		{
			if ( eval('return isset($this->'.$property.');') )
			{
				$sql .= '`'.$field.'` = ?,';
				eval('$values[] = $this->'.$property.';');
			}
			else
				$lastid = $property;
		}
		$sql = substr($sql, 0, -1);
		$q = $this->db->query($sql);
		$q->bind_values($values);
		return array($q, $lastid);
	}
	/**
	 * Append SET ...=... to sql statement
	 * @return mixed false iff contraint broken or count(values)
	 */
	function _appendSet(&$sql, &$values)
	{
		$sql .= ' SET ';
		foreach ( $this->objectmap['properties'] as $property => $field )
		{
			//use eval so maps like district->id work;
			if ( ($bitbucket = eval('return preg_match("/^\\w+\\(.*\\)$/", $property)?1:0;')) //functioncall
					||	($bitbucket = eval('return @isset($this->'.$property.')?1:0;')) ) // || property isset
			{

				$sql .= '`'.$field.'` = ?,';
				eval('
					if ( is_bool($this->'.$property.') )
						$values[] = (int) $this->'.$property.';
					elseif ($this->'.$property.' === DB_NULL)
						$values[] = null;
					else
						$values[] = $this->'.$property.';');
			}
			//DBMS doesn't implement restraints... so hack it this way.
			//Currently only ['properties']['langname'] needs such a constraint.
			//If new constraints popup,
			//I can imagine a objectmap['constraints'][$property][$field]['eval code which should return true if constraint is met'];
			//For the time being this hack will do.
			if ( $property == 'langname' && empty($this->langname) )
				return false;
		}
		$sql = substr($sql, 0, -1);
		return count($values);
	}
	/**
	 * Append the where statement based on primary part of objectmap
	 * and fill values
	 */
	function _appendWhereAllPrimaries(&$sql, &$values)
	{
		$sql .= ' WHERE ';
		foreach ( $this->objectmap['primary'] as $property => $field )
		{
			//debug($property);

			# FIXME: is_null gives a notice when accesing a property of a property that borth are null
			if ( eval('return @is_null($this->'.$property.');') )
				$sql .= 'ISNULL(`'.$field.'`) AND ';
			else
			{
				$sql .= '`'.$field.'` = ? AND ';
				eval('$values[] = $this->'.$property.';');
			}
		}
		$sql = substr($sql, 0, -5);
	}
	/**
	 * Checks references defined in the {@link referencemap}(s)
	 *
	 * This is madness ofcourse, and should be handled by the DBMS
	 * AARGH this can in no way be made complete..... It only works one "FOREIGN KEY" deep....
	 * You could endlessly define referencemaps ofcourse..... ;(( To get the same effect...
	 * @returns array of string (each string is a delete query which should be executed if no RESTRICT is violated, in which case it returns false)
	 */
	function _checkReferences()
	{
		$rv = array();
		do
		{
			$values = array();
			$sql = 'SELECT COUNT(*) as ROWS FROM '.$this->referencemap['dbtable'].' WHERE '; //NO backticks so JOINS work
			foreach ( $this->referencemap['fields'] as $property => $field )
			{
				$sql .= '`'.$field.'` = ? AND ';
				eval('$values[] = $this->'.$property.';');
			}
			$sql = substr($sql, 0, -5);
			$q = $this->db->query($sql); //could use some tuning, but this works
			$q->bind_values($values);
			$q->execute();
			$r = $q->fetch();
			if ( $r->{'ROWS'} )
			{
				if ( !$this->referencemap['ondelete'] ) //IC_REF_RESTRICT === false
				{
					$this->_nextTable('referencemap', true); // reset map to first
					return false;
				}
				else //IC_REF_CASCADE
					$rv[] = preg_replace('/(^[^F]*)/', 'DELETE ', $q->tmp_sql);
			}
		} while ( $this->_nextTable('referencemap') );
		return $rv;
	}
	/**
	 * Initialize objectmap(s) and familymap
	 *
	 * Called in the Constructor children must either override the Constructor or this method.
	 */
	function _initObjectMaps()
	{
	}
	/**
	 * @var array Objectmap multidimensional 'dbtable' => tablename, 'properties' => property => tablefieldname, 'primary' => property => tablefieldname
	 */
	var $objectmap;
	/**
	 * Used for typecasting in {@link set()}.
	 * MySQLDriver returns NULL or String, this will enable you to cast it to bool or int etc.
	 * @var array typemap ('property'=>'type')
	 */
	var $typemap;
	/**
	 * @var array multidimensional ('parent'=>parentclassname=>property, 'children'=>ChildClass=>array('property'=>property, 'classfile'=>classpath, 'dbtable'=>linktablename, 'key'=>array(property=>linktablefieldname), 'properties'=>array(childproperty=>linktablefieldname)))
	 * <code>
	 * //From {@link IntercleanFloor::_initObjectMaps()}
 	 * $this->familymap['children']['IntercleanSpace']['property'] = 'spaces';
 	 * $this->familymap['children']['IntercleanSpace']['classfile'] = INTERCLEAN.'interclean_space.php';
 	 * $this->familymap['children']['IntercleanSpace']['dbtable'] = 'TBL_SPACE';
 	 * $this->familymap['children']['IntercleanSpace']['key']['id'] = 'FLOOR_ID';
 	 * $this->familymap['children']['IntercleanSpace']['properties']['id'] = 'SPACE_ID';
 	 * $this->familymap['parent']['intercleanbuilding'] = 'building';
	 * </code>
	 */
	var $familymap;
	/**
	 * @var array Referencemap (1 foreach table referencing this) multidimensional ('dbtable' => tablename, 'field' => property => tablefieldname)
	 */
	var $referencemap;
	/**
	 * @var bool Flag if Object isset;
	 */
	var $complete = false;
	/**
	 * @var bool Flag if Object is filled;
	 */
	var $filled = false;
	/**
	 * @var Database
	 */
	var $db;
	/**#@-*/
}

define('DB_NULL', '___NULL___');

?>
