<?php
/**
 * @package
 * @subpackage
 * @author Nick Loeve <nick@basesoft.com>
 * @since 12/03/2007
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */

/**
 * We want to use PDO here, but we also need to maintain some backward compat with the old
 * i-ware classes.
 */
class IWDatabase extends PDO
{

	public $host;

	public function __construct($dsn, $user, $pass, $persistent=0)
	{
		//do options etc
		//we want exceptions
		$driver_options = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION);

		//we cannot have persistent connections and subclasses for PDOStatements
		/*if($persistent)
			if(!is_string($persistent))
				$driver_options[PDO::ATTR_PERSISTENT] = true;
			else
				$driver_options[PDO::ATTR_PERSISTENT] = $persistent; //this allows you to specify persistent cache name
		*/
		parent::__construct($dsn, $user, $pass, $driver_options);
		$this->setAttribute(PDO::ATTR_STATEMENT_CLASS, array('IWQuery', array($this, array(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY=>true))));
		$this->setAttribute(PDO::ATTR_EMULATE_PREPARES, true);
		//turn this off for a nicer speed up
		$this->placeholderBackwardCompat= true;
		$this->host = $dsn;
		$this->exec("SET NAMES utf8");
		$this->exec("SET SESSION sql_mode=''");
	}


	//compat function
	public function begin()
	{
		$this->beginTransaction();
	}

	public function rollbackOnError()
	{
		try
		{
			$this->rollback();
		}
		catch(Exception $e)
		{
			//pass
			//we don't want errors that don't have active
			//transactions to make loud unnecessary errors
		}
	}

	//compat function
	public function query($sql = '', $driver_options=array(), $useNativeQuery=false)
	{
		if($sql == '')
			throw new IWDatabaseException('You cannot create an empty query object');
		if(!$useNativeQuery)
		{
			$this->placeholderBackwardCompat && ($sql = $this->cleanOutOldPlaceholders($sql));
			$stmt = $this->prepare($sql, $driver_options);
			$stmt->tmp_sql = $sql; //pretty useless now though
			return $stmt;
		}
		return parent::query($sql);
	}

	private function cleanOutOldPlaceholders($sql)
	{
		$new_sql = str_replace('??', '?', $sql);
		if($new_sql !== $sql)
			trigger_error('Should fix data placeholders on query: '."\n".$sql, E_USER_NOTICE);
		return $new_sql;
	}
}

class IWQuery extends PDOStatement
{
	public $tmp_sql; //backwards compat
	protected $dbh;

	protected function __construct($dbh)
	{
		$this->dbh = $dbh;
		$this->setFetchMode(PDO::FETCH_OBJ);
	}

	//compat function
	public function bind_values()
	{
		$i = 1;
		$args = func_get_args();
		if(is_array($args[0]))
			$args = $args[0];
		foreach($args as $arg)
		{
			$this->bindValue($i, $arg);
			$i++;
		}
	}

	//compat function
	public function field($num=0)
	{
		return $this->getColumnMeta($num);
	}

	//compat function
	//may not be guaranteed to be of use for SELECT statements
	public function rows()
	{
		throw new Exception('IWDatabase: rows() can no longer be used.');
	}

	//compat function
	public function cols()
	{
		return $this->columnCount();
	}

	//compat function
	public function lastid()
	{
		return $this->dbh->lastInsertId();
	}

	//compat function
	public function fetch($style=PDO::FETCH_OBJ, $orientation = NULL, $offset = NULL)
	{
		return parent::fetch($style);
	}

	//compat function
	public function fetchAssoc()
	{
		return $this->fetch(PDO::FETCH_ASSOC);
	}

	//compat function
	public function affected_rows()
	{
		return $this->rowCount();
	}

	//compat function
	public function fetch_array()
	{
		return $this->fetch(PDO::FETCH_NUM);
	}

	//compat function
	public function execute($throwError=false)
	{
		try
		{
			return parent::execute();
		}
		catch (PDOException $e)
		{
			throw new IWQueryException($e->getMessage() . ': '.$this->tmp_sql);
		}
	}
}

# duno what to do about exceptions yet
# Exceptions which can be throwed in this class
abstract class MysqlException extends Exception {}
class MysqlForeignKeyException extends MysqlException {}
class MysqlUnknownException extends MysqlException {}
class IWDatabaseException extends Exception {}
class IWQueryException extends Exception {}


?>