<?php
/**
 * @package Test
 * @subpackage SeleniumRC
 * @author Nick Loeve <nick@basesoft.com>
 * @since 10/10/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
class SeleniumRC
{
    /**
     * @var SeleniumRCServerProxy
     */
    private $proxy;

    /**
     * @var string
     */
    private $browser;

    /**
     * @var string
     */
    private $browserUrl;

    /**
     * @var string
     */
    private $sessionId;

    /**
     * @var string
     */
    private $timeout;

    /**
     * Constructor
     *
     * @param SeleniumRCServerProxy $proxy
     * @param string $browser
     * @param string $browserUrl
     * @param int $timeout
     * @throws SeleniumRCException
     */
    function __construct(SeleniumRCServerProxy $proxy, $browser, $browserUrl, $timeout = 60000)
    {
        $this->proxy = $proxy;
        $this->browser = $browser;
        $this->browserUrl = $browserUrl;
        $this->timeout = $timeout;
    }

    /**
     * Run the browser and set session id.
     *
     * @return string session id on success
     * @throws SeleniumRCException
     */
    public function start()
    {
        $this->sessionId = $this->getString('getNewBrowserSession', array($this->browser, $this->browserUrl));
        $this->doCommand('setTimeout', array($this->timeout));
        return $this->sessionId;
    }

    /**
     * Close the browser and set session id null
     *
     * @throws SeleniumRCException
     */
    public function stop()
    {
        $result = $this->doCommand('testComplete');
        $this->sessionId = null;
    }

    /**
     * __call - runtime methods
     */
    public function __call($func, $args)
    {
    	if(substr($func, 0, 2) == 'is')
    	{
    		return $this->getBoolean($func, $args);
    	}
    	else if(substr($func, 0, 3) == 'get')
    	{
    		return $this->getString($func, $args);
    	}
    	return $this->doCommand($func, $args);
    }

    /**
     * Send the specified Selenese command to the browser to be performed
     *
     * @param string $verb
     * @param array $args
     * @return string on success
     * @throws SeleniumRCException
     */
    protected function doCommand($verb, $args = array())
    {
        $params = sprintf('cmd=%s', urlencode($verb));
        for($i = 0; $i < count($args); $i++)
        {
            $argNum = strval($i + 1);
            $params .= sprintf('&%s=%s', $argNum, urlencode($args[$i]));
        }

        if (isset($this->sessionId))
            $params .= sprintf('&%s=%s', 'sessionId', $this->sessionId);

        $response = $this->proxy->request($params);
        // added in selenium 1.0.3
        if(preg_match('/^XHR ERROR/', $response))
            throw new SeleniumRCXHRException('Response error: ' . $response);
        if(!preg_match('/^OK/', $response))
            throw new SeleniumRCException('The Response of the Selenium RC is invalid: ' . $response);
        return $response;
    }


    /**
     * Get the string result of the Selenese Command
     *
     * @param string $verb
     * @param array $arg
     * @return string on success
     * @throws SeleniumRCException
     */
    protected function getString($verb, $args = array())
    {
        $result = substr($this->doCommand($verb, $args), 3);
        $parts = preg_split('/(?<!\\\\),/', $result, -1, PREG_SPLIT_NO_EMPTY);
        if(count($parts) < 1)
        	throw new SeleniumRCException("String result cannot be parsed");
        if(count($parts) == 1)
        	return str_replace(array('\\,', '\\\\'), array(',', '\\'), $parts[0]);
        $arrayResult = array();
        foreach($parts as $part)
        {
        	$arrayResult[] = str_replace(array('\\,', '\\\\'), array(',', '\\'), $part);
        }
        return $arrayResult;
    }

    /**
     * Get the boolean result of the Selenese Command
     *
     * @param string $verb
     * @param array $args
     * @return boolean on success
     * @throws SeleniumRCException
     */
    protected function getBoolean($verb, $args = array())
    {
        $result = $this->getString($verb, $args);

        switch($result)
        {
	        case 'true':
	            return true;
	        case 'false':
	            return false;
	        default:
	            throw new SeleniumRCException('result is neither "true" or "false": ' . $result);
        }
    }

    public function isIE()
    {
    	return ($this->browser == '*iexplore');
    }
}

/**
 * A class to proxy out the request to the selenium server
 */
class SeleniumRCServerProxy
{
	/**
     * @var string
     */
    private $host;

    /**
     * @var int
     */
    private $port;

	/**
	 * Constructor
	 *
	 * @param string $host
     * @param int $port
	 */
	public function __construct($host = 'localhost', $port = 4444)
	{
		$this->host = $host;
        $this->port = $port;
	}

	/**
	 * @param string $params
	 * @return string
	 * @throws SeleniumRCException
	 */
	public function request($params)
	{
		$url = sprintf('http://%s:%s/selenium-server/driver/?', $this->host, $this->port).$params;

		// try to open the url and just die on error
		$handle = fopen($url, 'r');
		if (!$handle)
			throw new SeleniumRCException('Cannot connected to Selenium RC Server: '.$this->host.':'.$this->port);

        // no socket block
        stream_set_blocking($handle, false);
        $response = stream_get_contents($handle);
        fclose($handle);

        return $response;
	}
}

/**
 * SeleniumRC Exception
 */
class SeleniumRCException extends Exception {}
class SeleniumRCXHRException extends SeleniumRCException {}
?>