<?php
/**
 * VATChecker
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @package I-Ware
 */

define('VAT_CHECKER_WSDL', 'http://ec.europa.eu/taxation_customs/vies/services/checkVatService.wsdl');
define('VAT_REGEXP', '/^([A-Z]{2})([0-9A-Za-z\+\*\.]{2,12})$/');

/**
 * This will check if a VAT number is in fact a valid VAT
 * We have inbuilt stream handling and fail-over if service is down
 */
class VATChecker
{
	private $langue;
	private $data;
	private $last_error;
	private $urlVies;
	private $errors;
	private $server_ok; // true if the soap client was successfully created, false otherwise

	public function __construct($urlVies=null)
	{
		// skip the vat checker for selenium tests.
		// we just want to see if they match against the regexp
		if(defined('TEST_SITE') && TEST_SITE && !defined('I_AM_PHPUNIT')) {
			$this->server_ok = true;
			return;
		}

		$this->urlVies = is_null($urlVies) ? VAT_CHECKER_WSDL : $urlVies;
		$this->last_error = null;
		$this->errors = array(
				'VAT number structure is invalid',
				'Unable to connect to the VAT number validation site',
				'VAT number is invalid',
				'Internal error'
		);
		if (!@fopen($this->urlVies, "r"))
		{
			$this->server_ok = false;
			$this->setLastError(1); // unable to connect...
		}
		else
		{
			try
			{
				$this->soap_client = new SoapClient($this->urlVies);
				$this->server_ok = true;
			}
			catch(Exception $e)
			{
				$this->server_ok = false;
				$this->setLastError(1); // unable to connect...
			}
		}
	}

	public function getErrors()
	{
		return $this->errors;
	}

	public function getError($i)
	{
		return $this->errors[$i];
	}

	private function setLastError($num)
	{
		if(array_key_exists($num, $this->getErrors()))
			$this->last_error = $num;
		else
			$this->last_error = 3;

		return false;
	}

	public function getLastError()
	{
		if (is_null($this->last_error))
			return false;

		return $this->errors[$this->last_error];
	}


	/* Checks if vat number exists
	* $vatno : VAT number
	*/
	public function check($vatno)
	{
		if (!$this->server_ok)
			return false;

		$vatno = VATChecker::normalizeVAT($vatno);
		return $this->checkMSVAT($vatno);
	}

	/* Send & request to VIES site and interprets results
	* $vatno : VAT number
	*/
	private function checkMSVAT($vatno)
	{
		if(defined('TEST_SITE') && TEST_SITE && !defined('I_AM_PHPUNIT'))
			return true;
		// UNCOMMENTED code above, so all the checks for bad VATNo's won't take ages against the site (if it's phpunit, it won't skip). we just want to see if they match against the regexp

		$ViesOk		= 'YES, VALID VAT NUMBER';
		$ViesINVALID[] = 'NO, INVALID VAT NUMBER';
		$ViesINVALID[] = 'ERROR: INCOMPLETE (VAT + MS REQUIRED) OR CORRUPTED DATA INPUT';

		$parse_result = preg_match(VAT_REGEXP, $vatno, $matches);
		if ($parse_result)
		{
			// format is ok, now let's check it
			$country_code = $matches[1];
			$stripped_vat = $matches[2];
			try
			{
				$soap_response = $this->soap_client->checkVat(array('countryCode' => $country_code, 'vatNumber' => $stripped_vat));
				if ($soap_response->valid)
				{
					// it's valid
					return true;
				}
				else
				{
					// it's invalid
					return $this->setLastError(2);
				}
			}
			catch(Exception $e)
			{
				notice(sprintf('VATChecker: unexpected response for [%s %s]', $country_code, $stripped_vat)); //log, so we can improve expectance
				return $this->setLastError(3); //internal error
			}
		}
		else
		{
			// invalid vat number. after normalization it doesn't match the regular expression
			return $this->setLastError(2);
		}
	}

	private function checkForString($sHaystack, $aNeedles)
	{
		foreach ( $aNeedles as $needle )
			if ( strpos($sHaystack, $needle) !== false )
				return true;
		return false;
	}

	public static function normalizeVAT($vatno)
	{
		return str_replace(" ", "", strtoupper($vatno));
	}
}
?>
