<?php
/**
 *	Database is a database abstraction class; a unified means
 *	of accessing different relational database systems.  It is accompanied
 *	by a second class called Query, and relies on driver classes to provide
 *	all the database specific functionality.
 *
 *
 *	@package I-Ware
 *	@author John Luxford <lux@simian.ca>
 *	@since 2001-05-16
 *	@version 1.0
 *	@copyright 2001, Simian Systems
 *	@license http://www.simian.ca/license.php
 */

class Database
{
	/**
	 *	@var Resource $connection
	 */
	var $connection;
	/**
	 *	@var integer $transactions
	 */
	var $transactions = 0;
	/**
	 *	@var string $driver
	 */
	var $driver;
	/**
	 *	@var bool $noinc
	 */
	var $noinc = false;
	/**
	 *	@var string $host
	 */
	var $host;
	/**
	 *	@var string $name
	 */
	var $name;
	/**
	 *	@var string $user
	 */
	var $user;
	/**
	 *	@var string $pass
	 */
	var $pass;
	/**
	 *	@var MySQL_Driver $dbd
	 */
	var $dbd;
	var $noCommitMode = false;
	// This file can output queries when not empty. Using mode = A+. Use this for testing or logging
	var $outputFile = "";

	var $maxtries = 5;
	var $unavailableDbs = array();

	/**
	 *	Constructor method.  Establishes a connection to the specified database system.
	 *
	 *	@param string $connstr
	 *	@param string $user
	 *	@param string $pass
	 *	@param integer $persistent
	 *	@param bool $noinc
	 */
	function Database ($connstr = "::", $user = "", $pass = "", $persistent = 1, $noinc = false)
	{
		$this->noinc = $noinc;
		$this->_init($connstr);
		$this->user = $user;
		$this->pass = $pass;
		$this->persistent = $persistent;

        $driver = IWARE."MySQL_Driver.php";

        if (file_exists ($driver))
        {
        	require_once($driver);
        }
        else
            trigger_error('Database::Database - Cant include driver ('.$driver.')', E_USER_ERROR);

		$driver = $this->driver . "_Driver";
		$this->dbd = new $driver ($this->name, $this->host, $this->user, $this->pass, $this->persistent);

		$this->connection =& $this->dbd->connection;
	}


	/**
	 * Parses the given connect string and sets the private driver, host and name variables
	 */
	function _init($connstr)
	{
		if (ereg ("^([^:]+):([^:]*):(.+)$", $connstr, $regs))
		{
			$this->driver = $regs[1];
			$this->host = str_replace("|",":",$regs[2]);
			$this->name = $regs[3];
		}
		else
			trigger_error("Not a valid connstr($connstr)", E_USER_ERROR);
	}


	/**
	 *	Creates and returns a Query object. $sql is the SQL query you wish to execute with this object.
	 *
	 *	@param string $sql
	 *	@return MySQL_Driver
	 */
	function query ($sql = "")
	{
		$classname = $this->driver . "_Query";

		$q = new $classname ($sql, $this);
		$q->noCommitMode = $this->noCommitMode;
		$q->outputFile = $this->outputFile;
		return $q;
	}

	function connect()
	{
		return $this->dbd->connect();
	}

	function getDataBaseName()
	{
		return $this->name;
	}

	/**
	 * @return phptype://username:password@hostspec/database
	 */
	function getPEARDBDsn()
	{
		return strtolower($this->driver)
			.'://'.$this->user
			.':'.$this->pass
			.'@'.$this->host
			.'/'.$this->name;
	}

	function rollback()
	{
		$q = $this->query();
		$q->rollback();
	}

	function begin()
	{
		$q = $this->query();
		$q->begin();
	}

	function commit()
	{
		$q = $this->query();
		$q->commit();
	}
}

/**
 *	Query is the counterpart class to Database.  It provides the framework for querying a database.
 *
 *	@package I-Ware
 *	@author John Luxford <lux@simian.ca>
 *	@since 2001-05-16
 *	@version 1.0
 *	@copyright 2001, Simian Systems
 *	@link http://www.simian.ca/license.php
 *	@abstract
 */
class Query
{
	/**
	 * @var database $db Database on which this query is running
	 */
	var $db;
	/**
	 *	@var string $sql
	 */
	var $sql = "";
	/**
	 *	@var Result $result
	 */
	var $result = "";
	/**
	 *	@var string $field
	 */
	var $field = "";
	/**
	 *	@var Resource $connection
	 */
	var $connection = "";
	var $noCommitMode = false; // See documentation @ database class
	var $outputFile = "";  // See documentation @ database class

	/**
	 *	Constructor method.
	 *
	 *	@param string $sql
	 *	@param Resource $connection
	 */
	function Query ($sql = "", &$db) {
		$this->sql = $sql;
		$this->tmp_sql = $sql;
		$this->db =& $db;
		$this->connection = $db->connection;
	}

	/**
	 *	Replaces any occurrences of ? in $sql with the proper value from $values. Called automatically just prior to executing the current query.
	 *
	 *	@param array $values
	 */
	function bind_values ()
	{
        if (func_num_args() == 0)
            trigger_error('SQL::addWhereText() - Need atleast one argument (array with values or string1, string2, ...) ', E_USER_ERROR);

        $this->tmp_sql = $this->sql;
		$replacements = array();

		if (is_array( func_get_arg(0)) ) {
		    foreach ( func_get_arg(0) as $el)
		    {
		    	if (is_null($el))
		    		$replacements[] = 'NULL';
		    	else
					$replacements[] = "'" . mysql_escape_string($el) . "'";
			}
		}
		else {
            for ($i=0; $i < func_num_args(); $i++) {
		    	if (is_null(func_get_arg($i)))
		    		$replacements[] = 'NULL';
		    	else
					$replacements[] = "'" . mysql_escape_string(func_get_arg($i)) . "'";
			}
		}

		$offset = 0;
		foreach ($replacements as $replacement)
		{
			$pos = strpos($this->tmp_sql, '?', $offset);
			if ($pos === false)
				break;

			$this->tmp_sql = substr_replace($this->tmp_sql, $replacement, $pos, 2); // 2 = strlen('?')
			$offset = $pos + strlen($replacement);
		}

        return $this->result;
	}

	/**
	 *	Executes the current SQL query.
	 **/
	function execute () {}

	/**
	 *	Returns the name of the specified column in the table currently being queried.
	 *
	 *	@param integer $num
	 */
	function field ($num = 0) {}

	/**
	 *	Returns the number of rows affected or found by the current query.
	 */
	function rows () {}

	/**
	 *	Returns the row ID generated by the database during the previous SQL insert query.
	 */
	function lastid () {}

	/**
	 * Returns the next row of data from the current query, always in the form of an object, with each column as its properties.
	 */
	function fetch () {}

	/**
	 *	Tells the database system to let go of its data from the previous query.
	 */
	function free () {}

	function clear()
	{
		$this->sql = "";
		$this->result = "";
		$this->tmp_sql = "";
		$this->field = "";
	}
	/**
	 *	Seeks the rownumber provided
	 *
	 *	@param integer $rownum
	 */
	function data_seek($rownum = 0) {}

}


Class DB_manager
{
	var $sourceDb;
	var $targetDb;

	function DB_manager()
	{

	}

	function setDatabases($sourceDb,$targetDb)
	{
		$this->sourceDb = $sourceDb;
		$this->targetDb = $targetDb;
	}

	function getDatabases($db)
	{
		$sql = "SHOW DATABASES";
		$ex = $db->query($sql);
		$ex->execute();
		$dbArray = array();
		while($row = $ex->fetch())
		{
			$dbArray[] = $row->{"Database"};
		}

		return $dbArray;
	}

	function getTables()
	{
		$tableArray = array();
		$sql = "SHOW TABLES";
		$ex= $this->sourceDb->query($sql);
		$ex->execute();
		while($row = $ex->fetch())
		{
			$tableArray["SOURCE"][] = $row->{$ex->field (0)};
		}

		$ex= $this->targetDb->query($sql);
		$ex->execute();
		while($row = $ex->fetch())
		{
			$tableArray["TARGET"][] = $row->{$ex->field (0)};
		}

		return $tableArray;
	}



}

?>