<?php
/**
 * Form Element
 * @author Martijn van den Broek <mjbroek@xs4all.nl>
 * @copyright Copyright � 2005, Martijn van den Broek
 * @version $Id$
 * @package I-Ware
 * @subpackage SQL
*/

/**
 * Abstract SQL class
 * Handles low level sql stuff
 * @abstract
 * @package I-Ware
 * @subpackage SQL
 */
class SQL
{
	/**#@+
	 * @access private
	 */
	var $sql = '';
	var $table;

	var $is_build      = false;
	var $arrWheres     = array();
	var $arrWhereTexts = array();

	var $limitOffset   = null;
	var $limitRows     = null;
	/**#@-*/

	/**
	 * Constructor
	 */
	function SQL() { }

	/**
	 * returns a html formatted sql string
	 * @return string
	 */
	function getHtml()
	{
		$this->build();
		return $this->syntaxHighlight($this->sql);
	}

	/**
	 * returns the plain sql string, use this for the database
	 * @return string
	 */
	function getText()
	{
		$this->build();
		return str_replace("\n", ' ', $this->sql);
	}

	/**
	 * removes the last ", " of a string
	 * @access protected
	 * @param string string
	 * @return string
	 */
	function removeCommaSpace($string)
	{
		return substr($string, 0, strlen($string)-2);
	}

	/**
	 * returns a comma seperated list and starts whith string $name
	 * @access protected
	 * @param string $name
	 * @param array @arrList
	 * @param string @str_close
	 * @return string
	 */
	function addCommaList($name, $arrList, $str_close='')
	{
		if (sizeof($arrList) == 0)
			return '';

		$retval = $name.' ';
		foreach ($arrList as $item)
			$retval .= $item .', ';

		$retval = $this->removeCommaSpace($retval);
		$retval .= ' '.$str_close."\n";

		return $retval;
	}

	/**
	 * converts a string to html and adds some colours
	 * @param string $sql
	 * @return string HTML
	 */
	function syntaxHighlight($sql)
	{

		//-- add blue colors
		$arrBlueWords = array('SELECT ', 'FROM ', 'INNER JOIN ', 'LEFT JOIN ','RIGHT JOIN ', 'GROUP BY ',
		                      'ORDER BY ', 'WHERE ', ' ON ', ' ASC', ' DESC', 'LIMIT ',
		                      'INSERT INTO ', 'VALUES ', 'DELETE ', 'UPDATE ', 'SET ');

		foreach ($arrBlueWords as $blueWord)
			$sql = str_replace($blueWord, '<font color="#0000ff"><b>'.$blueWord.'</b></font>', $sql);

		//-- add dark-blue colors
		$arrBlueWords = array('AND ', 'OR ');
		foreach ($arrBlueWords as $blueWord)
			$sql = str_replace($blueWord, '<font color="#000066"><b>'.$blueWord.'</b></font>', $sql);

		//-- create red colors
		$arrRedWords = array('(', ')', '[', ']');
		foreach ($arrRedWords as $redWord)
			$sql = str_replace($redWord, '<font color="#ff0000">'.$redWord.'</font>', $sql);


		//-- make quoted strings green
		$new_sql = '';
		$chunks = explode("'", $sql);

		$i = 0;
		foreach ($chunks as $chunk)
		{
			if($i % 2 == 0)
				$new_sql .= $chunk;
			else
				$new_sql .= '<font color ="#006600">\''.$chunk.'\'</font>';

			$i++;
		}
		$sql = '<span style="font-family: Verdana; font-size: 10px;">'.$new_sql.'</span>';

		//-- replace newlines with br's
		$sql = str_replace("\n", "<br>", $sql);

		return $sql.'<br>';
	}

	/**
	 * quotes a string but makes sure its not quoated twice (by php magic quotes)
	 * @param string $value string to quote
	 * @param bool quote set to false if u dont want to quote
	 * @return string
	 */
	function quote_smart($value, $quote=true)
	{
		if (get_magic_quotes_gpc())
			trigger_error('Magic quotes is on', E_USER_WARNING);

		if (is_array($value))
			trigger_error('Cant add array as value', E_USER_ERROR);

		if (is_null($value))
		    return 'NULL';

		$value = str_replace( "\n", "\\n", $value );

		$value = str_replace( "'", "\'", $value );

		if ($quote)
			$value = "'" . $value . "'";

		return $value;
	}

	/**
	 * Sets a where clause for the SQL statement. Only for SQL_Select, SQL_Update and SQL_Delete
	 * Variable number of arguments.. Example:
	 * <code>$sql->addWhere('Col1', '100');    //-- WHERE Col1 = 100</code>
	 * or
	 * <code>$sql->addWhere('OR',  'Col2', '>=', '100'); //-- OR Col2 >= 100 </code>
	 * or
	 * <code>$sql->addWhere('AND', 'Col3', '=', 'MD5(10)', false); //-- AND Col3=MD5(10)</code>
	 */
    function addWhere()
    {
    	if (func_num_args() == 2)
    	{
    		$type = 'AND';
    		$col = func_get_arg(0);
    		$operator = '=';
    		$value = func_get_arg(1);
    		$quote = true;
    	}
    	else if (func_num_args() >= 4)
    	{
    		$type = func_get_arg(0);
    		$col = func_get_arg(1);
    		$operator = func_get_arg(2);
    		$value = func_get_arg(3);
    		$quote = (func_num_args() == 5) ? func_get_arg(4) : true;
    	}
    	else
    		trigger_error('SQL_Select::addWhere() - invalid number of arguments', E_USER_ERROR);

    	$operator = (strtolower($operator) == 'is') ? ' is ' : $operator;

        if ($col == '' || $operator == '')
            trigger_error ("error while adding 'where' ($col, $value, $operator)", E_USER_ERROR);

        $this->arrWheres[] = array('col'=>$col, 'value'=>$value, 'operator'=>$operator, 'type'=>$type, 'quote'=>$quote);
    }

	/**
	 * Manual adds a WHERE line
	 * Can be used for complex where queries.
	 * <code>$sql->addWhereText('AND ( ( (Col1=Col2) OR (Col3<Col4) ) OR Col5>Col6 )'); </code>
	 * the special ? can be used to set external values
	 * <code>$sql->addWhereText( "AND (Col1=? and Col2=?) OR Col3=? )", 'aap', 'bier', 'kwak');</code>
	 * gives: AND (Col1='aap' and Col2='bier') OR Col3='kwak' )
	 */
    function addWhereText()
    {
			if (func_num_args() == 0)
				trigger_error('SQL::addWhereText() - Need atleast one argument', E_USER_ERROR);

			$subject = func_get_arg(0);

			if (func_num_args() >  1)
			{
				$args = array();

				for ($i=1; $i<func_num_args(); $i++)
					$args[] = func_get_arg($i);

				$this->arrWhereTexts[] = $this->parse_args($subject, $args);;
			}
			else
				$this->arrWhereTexts[] = $subject;
    }

	/**
	 * builds the WHERE statement
	 * @access protected
	 */
	function buildWhere()
	{
		for ($i=0; $i<sizeof($this->arrWheres); $i++)
		{
			$this->sql .= ($i == 0) ? 'WHERE ' : $this->arrWheres[$i]['type'].' ';
			$value = $this->quote_smart($this->arrWheres[$i]['value'], $this->arrWheres[$i]['quote'] ) ;

			$this->sql .= $this->arrWheres[$i]['col'].$this->arrWheres[$i]['operator'].$value;
			$this->sql .= "\n";
		}

		foreach ($this->arrWhereTexts as $str)
			$this->sql .= $str."\n";
	}

	/**
	 * Sets the SQL limit
	 * Variable number of arguments. If only one argument is supplied we take it as the 'rows' variable. If two arguments are given the first will be 'offset' and second 'rows'
	 * Example 1. We only want the first 5 rows:
	 * <code>$sql->setLimit(5); //-- LIMIT 5</code>
	 * Example 2. We want 5 rows starting from number 3:
	 * <code>$sql->setLimit(3, 5); //-- LIMIT 3, 5</code>
	 */
    function setLimit()
    {
    	if (func_num_args() == 1)
    		$this->limitRows = func_get_arg(0);
		else if (func_num_args() == 2)
		{
	        $this->limitOffset = func_get_arg(0);
	        $this->limitRows   = func_get_arg(1);
		}
		else
		    trigger_error('SQL::setLimit() - Invalid number of arguments');
    }

    /**
	 * builds the limit sql
	 * @access protected
	 */
    function buildLimit()
    {
		if (isset($this->limitRows))
		{
			$this->sql .= 'LIMIT ';

			if (isset($this->limitOffset))
			{
				$this->sql .= $this->limitOffset .', ';
				$this->sql .= $this->limitRows;
			}
			else
				$this->sql .= $this->limitRows;

			$this->sql .= "\n";
		}
	}

	/**
	 * replaces ? in a string with $args and quotes it
	 * @access private
	 * @param string subject
	 * @param array $args
	 */
	function parse_args($subject, $args)
	{
		if (!is_array($args))
			trigger_error('SQL::parse_args() - $args is not a valid array', E_USER_ERROR);

        $replacement = array();
        $pattern     = array();

        for ($i=0; $i<sizeof($args); $i++) {
			$pattern[] = "/\?\?/";
			$replacement[] = "'" . mysql_escape_string( $args[$i] ) . "'";
		}

		$retval = preg_replace($pattern, $replacement, $subject, 1);

		return $retval;
	}
}



/**
 * SQL Select
 * Creates SQL SELECT statments
 *
 * <b>Notes</b>
 * <ul><li>If no addSelect() is called, 'SELECT *' will be used.</li>
 * <li>Agrument in the constructor can be used to add a FROM table.</li></ul>
 *
 * Example 1:
 * <code>
 * $sql = new SQL_Select('tblTest'); //-- SELECT * FROM tblTest
 * $sql->addWhere('Col1', '100');    //-- WHERE Col1 = 100
 * echo $sql->getHtml();
 * </code>
 *
 * Example 2:
 * <code>
 * $sql = new SQL_Select();
 * $sql->addSelect('*');
 * $sql->addSelect("DATE_FORMAT('colDate', '%Y-%m-%d')");
 * $sql->addFrom('tblTest');
 * $sql->addJoin('tblJoin1', 'FKCol1', 'PKCol1');
 * $sql->addJoin('tblJoin2', 'FKCol1', 'PKCol1', 'LEFT');
 * $sql->addWhere('AND', 'Col1', '=',  'Col2', false);
 * $sql->addWhere('OR',  'Col2', '>=', '100');
 * $sql->addOrderBy('Col1', false);
 * $sql->addOrderBy('Col2', true);
 * $sql->groupBy(array('Col3', 'Col2', 'Col1'));
 * echo $sql->getHtml();
 * </code>
 * will produce:
 * <code>
 * SELECT *, DATE_FORMAT('colDate', '%Y-%m-%d')
 * FROM tblTest
 * INNER JOIN tblJoin1 ON FKCol1=PKCol1
 * LEFT JOIN tblJoin2 ON FKCol1=PKCol1
 * WHERE Col1=Col2
 * OR Col2>='100'
 * GROUP BY Col3, Col2, Col1
 * ORDER BY Col1 DESC, Col2 ASC
 * </code>
 *
 * @package I-Ware
 * @subpackage SQL
 */
class SQL_Select extends SQL
{

	/**#@+
	 * @access private
	 */
    var $arrSelects   = array();
    var $arrFroms     = array();
    var $arrJoins     = array();
    var $arrGroupBys  = array();
    var $arrOrderBys  = array();
	/**#@-*/

	/**
	 * Constructor
	 * Starts building the SQL Select statement. If no $from_table is given, SELECT * will be used.
	 * @param string from_table
	 */
    function SQL_Select($from_table = null)
    {
        if (isset($from_table))
            $this->addFrom($from_table);
    }

	/**
	 * Adds another entry to the SELECT line
	 * @param string $select
	 */
    function addSelect($select)
    {
        if (empty($select))
            trigger_error ('SQL_Select::addSelect() - Cannot add empty $select', E_USER_ERROR);

        $this->arrSelects[] = $select;
    }

	/**
	 * Adds another entry to the SELECT line
	 * @param string $from
	 */
    function addFrom($from)
    {
        if (empty($from))
            trigger_error ('SQL_Select::addFrom()  - Cannot add empty $from', E_USER_ERROR);

        $this->arrFroms[] = $from;
    }

	/**
	 * Adds a JOIN rule to the query
	 * @param string $table
	 * @param string $col1
	 * @param string $col2
	 * @param string $type possible values: INNER | LEFT | RIGHT
	 */
    function addJoin($table, $col1, $col2, $type='INNER', $condition='', $condition_args=null)
    {
        if ($table == '' || $col1 == '' || $col2 == '')
            trigger_error ("error while adding 'join' ($table, $col1, $col2, $type)", E_USER_ERROR);

		if (!is_null($condition_args) && !is_array($condition_args))
			trigger_error('SQL_Select::addJoin() - condition_args is not an array', E_USER_ERROR);

        $this->arrJoins[] = array('table'=>$table, 'col1'=>$col1, 'col2'=>$col2, 'type'=>$type, 'condition'=>$condition, 'condition_args'=>$condition_args);
    }

    /**
	 * Adds a serie group by columns
	 * @deprecated use addGroupBy
	 * @see addGroupBy
	 */
    function groupBy($arr)
    {
        foreach ($arr as $col)
            $this->addGroupBy($col);
    }

	/**
	 * Adds a GROUP BY column
	 * @param string $col
	 */
	function addGroupBy($col)
    {
        if (empty($col))
            trigger_error ('SQL_Select::addGroupBy() - cannot add empty $col', E_USER_ERROR);

        $this->arrGroupBys[] = $col;
    }

    /**
	 * Adds a serie order by columns ascending
	 * @deprecated use addOrderBy
	 * @see addOrderBy
	 */
    function orderBy($arr)
    {
        foreach ($arr as $col)
            $this->addOrderBy($col);
    }

	/**
	 * Adds a ORDER BY column
	 * @param string $col
	 * @param string $asc if true the column will be sorted ascending, else descending
	 */
	function addOrderBy($col, $asc=true)
    {
        if ($col == '')
            trigger_error ("cannot add empty 'OrderBy'", E_USER_ERROR);

        $k = ($asc) ? 'ASC' : 'DESC';
        $this->arrOrderBys[] = $col.' '.$k;
    }

    function build()
    {
        //-- set 'SELECT *' is no select is given
        if (sizeof($this->arrSelects) == 0)
            $this->addSelect('*');

        //-- select
        $this->sql = $this->addCommaList('SELECT', $this->arrSelects);

        //-- from
        $this->sql .= $this->addCommaList('FROM', $this->arrFroms);

        //-- join
        foreach ($this->arrJoins as $join)
        {
            $this->sql .= $join['type'].' JOIN '.$join['table'].' ON '.$join['col1'].'='.$join['col2'];

            if (!empty($join['condition']))
            	$this->sql .= ' ' . $this->parse_args($join['condition'], $join['condition_args']);

            $this->sql .= "\n";
        }

        //-- where
        $this->buildWhere();

        //-- groupby
        $this->sql .= $this->addCommaList('GROUP BY', $this->arrGroupBys);

        //-- orderby
        $this->sql .= $this->addCommaList('ORDER BY', $this->arrOrderBys);

		//-- limit
		$this->buildLimit();
    }
}



/**
 * SQL Insert
 * Creates SQL INSERT statments
 *
 * Example:
 * <code>
 * $sql = new SQL_Insert('tblTest');
 * $sql->add('Col1', '100');
 * $sql->add('Col2', '200');
 * echo $sql->getHtml();
 * </code>
 * gives:
 * INSERT INTO tblTest ( Col1, Col2 )
 * VALUES ( '100', '200' )
 *
 * @package I-Ware
 * @subpackage SQL
 */
class SQL_Insert extends SQL
{

	/**
	 * @access private
	 */
    var $arrCols = array();

	/**
	 * Constructor
	 * @param string $table
	 */
	function SQL_Insert($table)
	{
		if ($table == '')
			trigger_error("empty table ($table)", E_USER_ERROR);

		$this->table = $table;
	}

	/**
	 * Adds a column and value
	 * @param string $col Column
	 * @param string value $value
	 * @param bool $quote If false the value will not be quoted (use for functions)
	 * Example 1. A normal add()
	 * <code>$sql->add('Col1', '100');</code>
	 * Example 2. Add() with a function, dont quote.
	 * <code>$sql->add('Col2', 'MD5(RAND())', false);</code>
	 */
	function add($col, $value, $quote=true)
	{
		if (empty($col))
			trigger_error ("must supply a column", E_USER_ERROR);

		if ( !$quote && empty($value) )
			trigger_error ("cannot add empty value and not quote", E_USER_ERROR);

		$this->arrCols[] = array('col'=>$col, 'value'=>$value, 'quote'=>$quote);
	}

	function build()
	{
		if ($this->is_build)
			return;

		$this->sql = "INSERT INTO ".$this->table.' ';

		//-- build column names list
		$arrTmp = array();
		foreach ($this->arrCols as $col)
			$arrTmp[] = $col['col'];

		$this->sql .= $this->addCommaList('(', $arrTmp, ')');

		//-- build values list
		$arrTmp = array();
		foreach ($this->arrCols as $col)
			$arrTmp[] = $this->quote_smart($col['value'], $col['quote']) ;

		$this->sql .= $this->addCommaList('VALUES (', $arrTmp, ')');
	}
}


/**
 * SQL Update
 * Creates SQL UPDATE statments
 *
 * Example:
 * <code>
 * $sql = new SQL_Update('tblTest');
 * $sql->add('Col1', '100');
 * $sql->add('Col2', 'MD5(RAND())', false); //-- false: dont quote the value
 * $sql->addWhere('Col3', '15');
 * echo $sql->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage SQL
 */

class SQL_Update extends SQL
{
	/**
	 * @access private
	 */
    var $arrCols = array();

	/**
	 * Constructor
	 * @param string table
	 */
    function SQL_Update($table)
    {
        if ($table == '')
            trigger_error("empty table ($table)", E_USER_ERROR);
        $this->table = $table;
    }

    /**
	 * Adds a 'SET' column and value to the SQL statement
	 * @param string $col column
	 * @param string $value value
	 * @param bool $quote If false the value will not be quoted (use for functions)
	 * Example 1. A normal add()
	 * <code>$sql->add('Col1', '100'); //-- SET Col1='100' </code>
	 * Example 2. Add() with a function.
	 * <code>$sql->add('Col2', 'MD5(RAND())', false); //-- SET Col2 = MD5(RAND()); </code>
	 */
    function add($col, $value, $quote=true)
    {
        if ($col == '')
            trigger_error ("must supply a column", E_USER_ERROR);

        if ($quote==false && $value=='')
            trigger_error ("cannot add empty value and not quote", E_USER_ERROR);

        $this->arrCols[] = array('col'=>$col, 'value'=>$value, 'quote'=>$quote);
    }

    function build()
    {
        if ($this->is_build)
            return;

        $this->sql = 'UPDATE '.$this->table."\n";

        //-- set
        $this->sql .= "SET \n";
        for ($i=0; $i<sizeof($this->arrCols); $i++)
        {
            $value = $this->quote_smart($this->arrCols[$i]['value'], $this->arrCols[$i]['quote'] ) ;
            $this->sql .= $this->arrCols[$i]['col'].'='.$value.', ';
        }
        $this->sql = $this->removeCommaSpace($this->sql);
        $this->sql .= "\n";

        //-- where
		$this->buildWhere();
    }
}

/**
 * SQL Delete
 * Creates SQL DELETE statments
 *
 * Example:
 * <code>
 * $sql = new SQL_Delete('tblTest'); //-- DELETE FROM tblTest
 * $sql->addWhere('Col1', '100');    //-- WHERE Col1 = 100
 * $sql->addWhere('Col2', '200');    //--   AND Col1 = 100
 * echo $sql->getHtml();
 * </code>
 * @package I-Ware
 * @subpackage SQL
 */
class SQL_Delete extends SQL
{
	/**
	 * @access private
	 */
    var $arrCols = array();

	/**
	 * Constructor
	 * @param string $table
	 */
    function SQL_Delete($table)
    {
        if ( empty($table) )
            trigger_error('SQL_Delete::SQL_Delete() - Empty argument $table', E_USER_ERROR);

        $this->table = $table;
    }

    function build()
    {
        $this->sql = "DELETE FROM ".$this->table."\n";

        //-- where
        $this->buildWhere();

		//-- limit
		$this->buildLimit();

    }
}



?>
