/**
 * IWARE.Dashboard
 *
 * Extend this for every application
 * @abstract
 *
 * @package IWARE
 * @subpackage Dashboard
 * @since May 10, 2007
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

var DASHBOARD_PATH = '/includes/i-w/Dashboard/'
var WIDGETS_PATH = DASHBOARD_PATH + 'Widgets/';

/**
 * Includes all required files, prepares events and calls init() when done
 * @constructor
 */
IWARE.Dashboard = function(dashboard_container)
{
	// Initialize the temporary Panel to display while waiting for external content to load
	this.waitDlg = MessageBox.prototype.Wait('Please wait while loading dashboard..', 'Initialising..');

	// prepare some events
	this.onbuild = new YAHOO.util.CustomEvent("onbuild", this); // fired when the interface has been built
	this.onloadlayout = new YAHOO.util.CustomEvent("onloadlayout", this); // fired when the layout has been loaded

	// load requirements and call init when done
	var includer = new IWARE.AsyncIncluder(this.includes);
	includer.onInclude.subscribe(function(type, args, me)
	{
		this.debug && dumpln('Dashboard: All files included, calling init()');
		me.init(dashboard_container);
	}, this);

};

/**
 * Counter for widgets
 * @fixme: this should be handled by dashboard itself
 */
IWARE.Dashboard.widgetCounter = 0;

IWARE.Dashboard.prototype =
{
	debug: false,

	/**
	 * Javascript source files to include
	 */
	includes: [
		// yui stuff
		'/yui/dragdrop/dragdrop-min.js',
		'/yui/tabview/tabview-min.js',
		'/yui/tabview/assets/tabview.css',

		// dashboard classes
		DASHBOARD_PATH + 'IWARE.WidgetSettings.js',
		DASHBOARD_PATH + 'DDWidget.js',

		// basic style
		DASHBOARD_PATH + 'css/Dashboard.css',
		DASHBOARD_PATH + 'css/round_tabs.css',

		// abstract / iware widgets
		WIDGETS_PATH + 'AbstractWidget.js',
		WIDGETS_PATH + 'RpcWidget.js',
		WIDGETS_PATH + 'TableWidget.js',
		WIDGETS_PATH + 'DataPilotWidget.js'
	],

	/**
	 * List of Widgets to register
	 */
	registerWidgets: [
		'IWARE.AbstractWidget', // for testing purposes
	],

	/**
	 * Array of widgets classNames and Instances
	 * @private
	 */
	_widgets: [],

	/**
	 * Number of columns to display
	 * @private
	 */
	_columns: 3,

	/**
	 * Reference to the table body that contains all widgets
	 * @private
	 */
	tbodyWidgets: null,

	/**
	 * Pool of translations which will be translated in init
	 * @protected
	 */
	lang_handles: [
		'Add Widgets', 'Add Tab', 'Remove Tab', 'Rename Tab',
		'Initialising Dashboard',
		'New Widgets Intro text', 'Add', 'Done', // frmNewWidgets
		'Tab title?', 'OK', 'Cancel'
	],

	/**
	 * Init
	 * @param nNode to draw dashboard into
	 * @private
	 * @final
	 */
    init: function(dashboard_container)
    {
		// translateHandles returns an event which will be fired when its done
		langIf.translateHandles(this.lang_handles).subscribe(function(type, args, me)
		{
			// clear lang_handles so we can do another run later
			this.lang_handles = [];

			me.initToolbar(dashboard_container);
			me.initTabs(dashboard_container);
			me.initFrmWidgets();
			me.initFrmRenameTab();

			// fire the event that we build the dashboard
			me.debug && dumpln('Dashboard: Dashboard built');
			me.onbuild.fire();

			// FIXME :REMOVE COMMENT
			// register onunload method
			addEvent(window, 'unload', hitch(me, 'saveLayout'));
			setInterval(hitch(me, 'saveLayout'), 5 * 60 * 1000);

			// register widgets
			me.registerWidgets.forEach(function(sWidget, i, a)
			{
				try {
					me.registerWidget(sWidget);
				}
				catch (e) {
					MessageBox.prototype.Critical('Error registring widget', 'Error while registring widget ' +sWidget+': \n\n' + print_r(e));
					throw e
				}
			});

			me.loadLayout();

		}, this);

		// fired when the layout is loaded from the database
		this.onloadlayout.subscribe(function(type, args, me)
		{
			this.waitDlg.hide();
		});

	},

	/**
	 * Draws the toolbar in the container
	 * @param nNode
	 * @private
	 */
	initToolbar: function(dashboard_container)
	{
    	// make a div where we can put buttons in
		var divButtons = document.createElement('DIV');
		dashboard_container.appendChild(divButtons);
		divButtons.className = 'buttons';


		// button that opens the addWidget dialog
		var oBtn = new YAHOO.widget.Button({
			type: "button",
			label: langIf.getText('Add Widgets'),
			container: divButtons
        });
		oBtn.addListener("click", hitch(this, '_showWidgetsDialog'));


		// add tab
		var oBtn = new YAHOO.widget.Button({
			type: "button",
			label: langIf.getText('Add Tab'),
			container: divButtons
        });
		oBtn.addListener("click", hitch(this, 'addTab'));


		// remove current tab
		var oBtn = new YAHOO.widget.Button({
			type: "button",
			label: langIf.getText('Remove Tab'),
			container: divButtons
        });
		oBtn.addListener("click", hitch(this, 'removeTab'));


		// rename current tab
		var oBtn = new YAHOO.widget.Button({
			type: "button",
			label: langIf.getText('Rename Tab'),
			container: divButtons
        });
		oBtn.addListener("click", hitch(this, '_showRenameTabDialog'));


		/*
		// temp save button
		var btn = document.createElement('INPUT');
		btn.type = 'button';
		divButtons.appendChild(btn);
		btn.value = 'Save Layout';
		btn.onclick = hitch(this, 'saveLayout');

		// tmp load
		var btn = document.createElement('INPUT');
		btn.type = 'button';
		divButtons.appendChild(btn);
		btn.value = 'Load Layout';
		btn.onclick = hitch(this, 'loadLayout');
		*/
	},

	/**
	 * Draws the tabView in the container
	 * @private
	 * @param nNode
	 */
	initTabs: function(dashboard_container)
	{
    	// init the tabview
    	dashboard_container.className = 'yui-navset';
	    this.yuiTabView = new YAHOO.widget.TabView(dashboard_container);
	},

	/**
	 * Renders a form which will be containing the widgets the user can add.
	 * A dialog is used to present the user with the widgets. It will not
	 * be shown till the user presses the addWidget button
	 * @private
	 */
	initFrmWidgets: function()
	{
		var frmNewWidgets = document.createElement('FORM');
		frmNewWidgets.className = 'newWidgets';
		document.body.appendChild(frmNewWidgets);

		// add a table to the dialog that will contain Widget descriptions
		var tb = document.createElement('TABLE');
		tb.className = 'WidgetsAvailable';
		frmNewWidgets.appendChild(tb);

		// room for intro text
		var thead = document.createElement('THEAD');
		tb.appendChild(thead);
		var tr = document.createElement('TR');
		thead.appendChild(tr);
		var td = document.createElement('TD');
		tr.appendChild(td);
		td.innerHTML = langIf.getText('New Widgets Intro text');

		this._tbWidgetsAvailable = document.createElement('TBODY');
		tb.appendChild(this._tbWidgetsAvailable);

		this._widgetsDialog = new YAHOO.widget.Dialog(frmNewWidgets,
		{
			width : "300px",
			fixedcenter : true,
			visible : false,
			constraintoviewport : true,
			buttons : [ { text:langIf.getText("Done"), handler:function() { this.cancel(); }  } ]
		} );
		this._widgetsDialog.render();
	},

	/**
	 * Renders a form for the user to rename the current tab
	 * @private
	 */
	initFrmRenameTab: function()
	{
		var frm = document.createElement('FORM');
		frm.className = 'frmRenameTab';
		document.body.appendChild(frm);

		var div = document.createElement('DIV');
		div.style.padding = '10px';
		frm.appendChild(div);

		var lbl = document.createElement('SPAN');
		lbl.htmlFor = 'tabLabel';
		lbl.innerHTML = langIf.getText('Tab title?');
		div.appendChild(lbl);

		div.appendChild(document.createElement('BR'));

		var input = document.createElement('INPUT');
		input.id = input.name = 'tabLabel';
		div.appendChild(input);

		this._dlgRenameTab = new YAHOO.widget.Dialog(frm,
		{
			width : "300px",
			fixedcenter : true,
			visible : false,
			constraintoviewport : true,
			buttons : [
				{ text:langIf.getText("OK"), handler: hitch(this, 'renameTab') } ,
				{ text:langIf.getText("Cancel"), handler:function() { this.cancel(); }  }
			]
		} );

		this._dlgRenameTab.render();
	},

	/**
	 * Called when the user clicks the showWidgets button
	 * @private
	 */
	_showWidgetsDialog: function()
	{
		this._widgetsDialog.show();
		return false;
	},

	/**
	 * Called when the user clicks the renameTab button
	 * @private
	 */
	_showRenameTabDialog: function()
	{
		this._dlgRenameTab.show();

		// give the input field the value of the current tab
		var tab = this.yuiTabView.get('activeTab');
		$('tabLabel').value = tab.get('label');

		return false;
	},

	/**
	 * Saves the Dashboard to the database
	 * @private
	 */
	saveLayout: function()
	{
		/**
		 * save the following:
		 *  - tabs + names
		 *  - widgets + positions
		 *  - widget settings
		 */

		// declare array to save
		var data = [];

		// loop tru tabs
		for (var tabIndex=0; tabIndex < this.yuiTabView.get('tabs').length; tabIndex++)
		{
			var tabLabel = this.yuiTabView.get('tabs')[tabIndex].get('label');
			data[tabLabel] = [];
			var nTable = $('dashboard_tab_'+tabIndex);

			// loop tru table rows
			for (var rowIndex=0; rowIndex < nTable.rows.length; rowIndex++)
			{
				data[tabLabel][rowIndex] = [];
				var tr = nTable.rows[rowIndex];

				// loop tru cells
				for (var cellIndex=0; cellIndex < tr.cells.length; cellIndex++)
				{
					data[tabLabel][rowIndex][cellIndex] = [];
					var td = tr.cells[cellIndex];

					// loop tru widgets
					for (var widgetIndex=0; widgetIndex < td.childNodes.length; widgetIndex++)
					{
						var child = td.childNodes[widgetIndex];
						var widget = this._widgets[child.id];

						var arr = [];
						arr['className'] = widget.className;
						arr['settings'] = widget.instance.settings ? widget.instance.settings.getFormValues() : [];

						data[tabLabel][rowIndex][cellIndex][widgetIndex] = arr;
					}
				}
			}
		}

		jsbroker.call('DashboardHandler.saveLayout', data, function() {
			dumpln('Dashboard: Layout saved');
		}, this);

	},

	/**
	 * Loads the dashboard from the database
	 * @private
	 */
	loadLayout: function()
	{
		jsbroker.call('DashboardHandler.loadLayout', [], function(data)
		{
			try
			{
				var tabIndex = 0;
				data.forEach(function(tabData, tabLabel, a)
				{
					this.addTab(tabLabel);
					tabData.forEach(function(tabRow, i, a)
					{
						tabRow.forEach(function(tabCell, rowIndex, a)
						{
							tabCell.forEach(function(widget, i, a)
							{
								try {
									var wWidget = this.addWidget(widget['className'], tabIndex, rowIndex, widget['settings']);
								}
								catch (e) {
									MessageBox.prototype.Critical('Error loading widget', 'Error while loading widget ' + widget['className']);
									//throw e; // fixme
								}

							}, this);
						}, this);
					}, this);

					tabIndex++;
				}, this);
			}
			catch (e)
			{
				MessageBox.prototype.Critical('Error loading dashboard', 'There was an error while loading the dashboard from the database. <br><br>'+print_r(e) );
				throw e
			}

			// fire the event that the layout is loaded (error or not)
			this.debug && dumpln('Dashboard: layout loaded');
			this.onloadlayout.fire();

		}, this);
	},

	/**
	 * Adds a tab to the tabView
	 * @param label Name of the tab (optional)
	 * @private
	 */
	addTab: function(label)
	{
		var nTabs = this.yuiTabView.get('tabs').length;
		var nTabId = nTabs;

		if (typeof label != 'string')
			label = 'Tab ' + nTabId;

		var yuiTab = new YAHOO.widget.Tab(
		{
			label: label,
			content: '<table id="dashboard_tab_'+nTabId+'"></table>'
		});

		this.yuiTabView.addTab(yuiTab);

		if (nTabId == 0)
			this.yuiTabView.set('activeIndex', 0);

		// get the table we just created
    	var table = $('dashboard_tab_'+nTabId);
    	table.className = 'dashboard';

		// Now the widgets body
		var tbody = document.createElement('TBODY');
		table.appendChild(tbody);

   		var tr = document.createElement('TR');
   		tbody.appendChild(tr);

    	for (var i=0; i<this._columns; i++)
    	{
    		var td = document.createElement('TD');
			td.className = 'column';
			td.id = 'dashboard_td_'+nTabId+'_'+i;
    		tr.appendChild(td);

    		// make this column a drag and drop target
    		new YAHOO.util.DDTarget(td.id);
    	}
	},

	/**
	 * Removes the current tab
	 * @private
	 */
	removeTab: function()
	{
		// fixme: first remove all widgets in the tab

		// remove the tab
		var tab = this.yuiTabView.get('activeTab');
		this.yuiTabView.removeTab(tab);
	},

	/**
	 * called from the dialog
	 * fixme: maybe mame a little more flexible
	 * @private
	 */
	renameTab: function()
	{
		var sLabel = this._dlgRenameTab.getData().tabLabel;
		var tab = this.yuiTabView.get('activeTab');
		tab.set('label', sLabel);
		this._dlgRenameTab.hide();
	},

	/**
	 * Adds an instance of an widget to the dashboard. THe second argument is optional, if not supplied the best column is chosen for you
	 * @param wWidget
	 * @param tabIndex (optional)
	 * @param columnIndex (optional)
	 */
	addWidget: function(sWidget, tabIndex, columnIndex, settings)
	{
		this.debug && dumpln('Dashboard addWidget() - sWidget: '+sWidget+', tabIndex: '+tabIndex+', columnIndex: '+columnIndex);

		var wWidgetDef = eval(sWidget);
		if (typeof wWidgetDef == 'undefined')
			throw 'addWidget: definition of widget does not exist: '+sWidget

		if (this.yuiTabView.get('tabs').length == 0)
			throw 'Error adding widget: There are no tabs available'

		if (typeof tabIndex == 'undefined')
			tabIndex = this.yuiTabView.get('activeIndex');

		if (typeof settings == 'undefined')
			settings = [];

		if (typeof columnIndex == 'undefined')
		{
			// get the best destination column by seeing which has least childs
			var min_childs = null;
			var columnIndex = null;
			for (var i=0; i<this._columns; i++)
			{
				var col = $('dashboard_td_'+tabIndex+'_'+i);
				if (min_childs == null || col.childNodes.length < min_childs)
				{
					min_childs = col.childNodes.length;
					columnIndex = i;
				}
			}
		}

		try
		{
			var wWidget = new wWidgetDef();
			wWidget.init();

			var nNode = wWidget.getNode();
			this._widgets[nNode.id] = {instance: wWidget, className: sWidget};
			destination =$('dashboard_td_'+tabIndex+'_'+columnIndex);
			destination.appendChild(nNode);

			/**
			 * This is a little tricky. It is possible that the settings have values loaded from
			 * an rpc call. In that case wait for the settings to load before applying settings and loading body
			 */
			if (wWidget.settings && wWidget.settings.getLoadCount() > 0)
			{
					wWidget.settings.onLoad.subscribe(function(type, args, me) {
						me.debug && dumpln('AbstractWidget::addWidget: setting widget settings: '+print_r(settings));
						wWidget.settings.setFormValues(settings);

						wWidget.setTitle();
						wWidget.showBody();
					}, this);
			}
			else
			{
				this.debug && dumpln('AbstractWidget::addWidget: directly setting widget settings: '+print_r(settings));
				wWidget.settings && wWidget.settings.setFormValues(settings);
				wWidget.setTitle();
				wWidget.showBody();
			}


			// add drag and drop behaviour to the widget
			wWidget.ddwidget = new WEEE.DDWidget(nNode.id);

			// change the background of the table so the user knows where to drag into
			wWidget.ddwidget.onStartDrag.subscribe(function(type, args, me2) {
				addCssClass($('dashboard_tab_'+tabIndex), 'ondrag');
			}, this);

			wWidget.ddwidget.onEndDrag.subscribe(function(type, args, me2) {
				removeCssClass($('dashboard_tab_'+tabIndex), 'ondrag');
			}, this);
		}
		catch(e)
		{
			throw 'Error while adding the widget: \n\n' + print_r(e)
			throw e

			// try to cleanup
			try {
				removeNode(nNode)
			} catch (e) { }
		}

		return wWidget;
	},

	/**
	 * Removes a widget
	 * @param int wWidgetId
	 */
	removeWidget: function(wWidgetId)
	{
		this._widgets[wWidgetId].instance.ddwidget.unreg();
		removeNode($(wWidgetId));
	},

	/**
	 * Adds a widget to the available widgets by adding a row to the form table
	 * Pass a widget string representation of a class (not an instance!) as the only argument
	 * @param string sWidget
	 */
	registerWidget: function(sWidget)
	{
		if (typeof sWidget != 'string')
			throw 'registerWidget: An error occured while registring widget ' + sWidget

		var wWidget = eval(sWidget);
		if (typeof wWidget == 'undefined')
			throw 'registerWidget: Widget class does not exist: ' + sWidget;

		// translate title and description
		var lang_handles = [];
		lang_handles.push(wWidget.prototype.title);
		lang_handles.push(wWidget.prototype.description);

		// translate widget handles
		lang_handles = lang_handles.concat(wWidget.prototype.lang_handles);

		// include dependencies
		if (wWidget.prototype.includes.count())
		{
			this.debug && dumpln('Dashboard::registerWidget() - including: '+print_r(wWidget.prototype.includes));
			var includer = new IWARE.AsyncIncluder(wWidget.prototype.includes);
			includer.onInclude.subscribe(function(type, args, me) { }, this);
		}

		langIf.translateHandles(lang_handles).subscribe(function(type, args, me)
		{
			me.debug && dumpln('Dashboard::registerWidget() - handles translated, adding widget to form: '+sWidget);

			var tr = document.createElement('TR');
			me._tbWidgetsAvailable.appendChild(tr);

			var td = document.createElement('TD');
			tr.appendChild(td);
			td.innerHTML = '<b>'+langIf.getText(wWidget.prototype.title)+'</b> <br />' + langIf.getText(wWidget.prototype.description);

			var td = document.createElement('TD');
			td.className = 'button';
			tr.appendChild(td);

			var btn = document.createElement('INPUT');
			btn.type = 'button';
			td.appendChild(btn);
			btn.value = langIf.getText('Add');

			YAHOO.util.Event.addListener(btn, "click", function() {
				this.addWidget(sWidget);
			}, me, true);

		}, this);
	}
};

