/**
 * IWARE.WidgetSettingsSettings
 * The default settings class for Widgets
 *
 * @package IWARE
 * @subpackage Dashboard
 * @since May 10, 2007
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

IWARE.WidgetSettings = function()
{
	// fired when all controls have been loaded
	this.onLoad = new YAHOO.util.CustomEvent("onLoad", this);
};

IWARE.WidgetSettings.prototype =
{
	/**
	 * Object containing setting nodes?
	 */
	settings: {},

	/**
	 * Node form containing all the settings
	 */
	htmlForm: null,

	/**
	 * Int counting the number of settings retrieved from the database.
	 * When this is 0 the event onload will be fired.
	 */
	_loadCount: 0,

	/**
	 * Constructor
	 */
	init: function()
	{
		this.htmlForm = document.createElement('form');
	},

	/**
	 *  Gets the loadcount used by IWARE.Dashboard to see if it has to wait before setting form values
	 */
	getLoadCount: function()
	{
		return this._loadCount;
	},

	/**
	 * Builds a select object with the given dataset or rpcCall
	 * @access protected
	 */
	makeSelect: function(name, dataList, rpcCall, onchange)
	{
		// build select
		var nSelect  = document.createElement('select');
		nSelect.setAttribute('name', name);

		// hook onchange
		if(typeof onchange != 'undefined')
			nSelect.onchange = onchange;

		// fill select if we have a dataset
		if (typeof dataList == 'undefined')
			dataList = [];
		dataList.forEach(function(el, i, a)
		{
			var opt = document.createElement('option');
			opt.innerHTML = el[0];
			opt.value=el[1];
			nSelect.appendChild(opt);
		}, this);

		// fill select if we have a rpc call
		if (typeof rpcCall != 'undefined')
			this.fillSelect(nSelect, rpcCall);

		// register select
		this.settings[name] = nSelect;

		return nSelect;
	},

	/**
	 * Fills a select with the given rpcCall.
	 * The rpcCall must return an array in the form of array(value => caption)
	 *
	 * @access private
	 */
	fillSelect: function(nSelect, rpcCall)
	{
		this._loadCount++;

		jsbroker.call(rpcCall, [], function(rv)
		{
			//dumpln(print_r(rv));
			rv.forEach(function(value, key, a) {
				nSelect.options.add(new Option(value, key));
			});

			/**
			 * fixme: doc
			 */
			this._loadCount--;
			if (this._loadCount == 0)
				this.onLoad.fire();

		}, this);
	},

	/**
	 * Returns the container of the settings form. Called from Dashboard
	 * @final
	 */
	getSettingsForm: function()
	{
		var pNode = this.getSettingsNode();

		table = document.createElement('TABLE');
		pNode.appendChild(table);

		this._tbody = document.createElement('TBODY');
		table.appendChild(this._tbody);

		return pNode;
	},

	/**
	 * Use this if you need access to the setting form
	 * @access protected
	 * @return nNode form
	 */
	getSettingsNode: function()
	{
		return this.htmlForm;
	},

	/**
	 * Handy function to add a row to the settings
	 * @param td1 string|nNode (optional)
	 * @param td2 string|nNode (optional)
	 * @return nNode TR
	 *
	 * <example>

	 	// add a single button to the settings
		var btn = new YAHOO.widget.Button({
			type: "button",
			label: "My Button",
			container: this.settings.addRow().cells[0]
		});

	 * </example>
	 * <example>
	 	// add a select that gets its data from an rpc call
		this.settings.addRow(langIf.getText('CRSO'), this.settings.makeSelect('crso[id]', [], 'DashboardHandler.getCRSOs'));
	 * </example>
	 *
	 */
	addRow: function(td1, td2)
	{
		var tr = document.createElement('TR');
		this._tbody.appendChild(tr);

		var td = document.createElement('TD');
		tr.appendChild(td);

		if (typeof td2 != 'undefined')
		{
			if (typeof td1 == 'string')
				td.innerHTML = td1;
			else
				td.appendChild(td1);
		}

		if (typeof td2 != 'undefined')
		{
			var td = document.createElement('TD');
			tr.appendChild(td);

			if (typeof td2 == 'string')
				td.innerHTML = td2;
			else
				td.appendChild(td2);
		}
		else
			td.colSpan = '2';

		return tr;
	},

	/**
	 * @public
	 * returns an array of the current settings.
	 * Used by Dashboard to save settings and can be used to send current form values to the server to refresh
	 */
	getFormValues: function()
	{
		return this._foldForm(this.getSettingsNode())
	},

	/**
	 * Sets the form with values. Used by Dashboard when values are retrieved from the database
	 * @param array
	 * @param string
	 */
	setFormValues: function(data, keys)
	{
		//dumpln('setFormValues() - data: '+print_r(data)+', keys: '+print_r(keys));

		if (typeof keys == 'undefined')
			keys = [];

		data.forEach(function(e, i, a)
		{
			// add the key to the array so we can build the elementName later
			keys.push(i);

			// check if there is another array, then call itself and append the key
			if (e instanceof Array)
				this.setFormValues(e, keys);
			else
			{
				// when we come here we are at the value
				// build the keys and look for the element
				var sElement = '';
				for (var i=0; i<keys.count(); i++)
				{
					if (i == 0)
						sElement += keys[i];
					else
						sElement += '['+keys[i]+']';
				}

				dumpln(this.getSettingsNode())

				var _getEl = function(nParent, sName)
				{
					//dumpln('_getEl: '+nParent)
					//dumpln('childnodes length: '+nParent.childNodes.length);
					for (var i=0; i<nParent.childNodes.length; i++)
					{
						var childNode = nParent.childNodes[i];
						//dumpln('checking '+childNode)

						if (childNode.name != 'undefined' && childNode.name == sName)
						{
							//dumpln('found: '+childNode.name)
							return childNode;
						}

						var rv = _getEl(childNode, sName);
						if(rv)
							return rv;
					}

					return false;
				}

				// we made a key, see if the element exists
				dumpln(this.getSettingsNode().innerHTML);
				var el = _getEl(this.getSettingsNode(), sElement)

				if (!el)
					throw 'Error restoring settings to form: '+sElement+' cannot be found'

				switch (el.tagName)
				{
					case 'INPUT':
						switch (el.type)
						{
							// checkboxes need to be checked if the value true
							case 'checkbox':
								el.checked = (e);
								break;
							default:
								el.value = e;
						}

					break;

					case 'SELECT':
						//alert('setting '+el+ 'to '+e)
						Select_setSelectedValue(el, e);
					break;

					default:
						throw 'Cannot set form value: Type '+el.tagName+' not implemented'
				}

			}

			// done for this level, go one level back
			keys.pop();

		}, this);
	},

	/**
	 * fixme: this function should go to a global IWARE class
	 * @private
	 */
	_foldForm: function( node, arr )
	{
		if (typeof arr == 'undefined')
			arr = new Array();

	    do
	    {
	        if ( node.nodeType != 1 )
	            continue;

			switch (node.tagName)
			{
				case 'INPUT':
					if ( node.name == '' )
						break;

					//-- ignore all radios and checkboxes that are not checked
					if ((node.type == 'radio' || node.type == 'checkbox') && !node.checked)
						break;

					re = /^(.+)\[(.*)\]$/;
					if ( found = node.name.match(re) )
					{
						//-- its an array, lets save it :D
						var x = found[2];
						if (typeof arr[found[1]] == 'undefined')
							arr[found[1]] = new Array();

						if ( x != '')
							arr[found[1]][x] = node.value;
						else
							arr[found[1]].push(node.value);

					}
					else //-- can add it normally
						arr[node.name] = node.value;
				break;

				case 'SELECT':

					if ( node.name != '' )
					{
						var name = node.name;

						var subkey = false;
						//-- check if the name is array, then make it
						//   so we can push the option values
						re = /^(.+)\[(.*)\]$/;
						if (found = node.name.match(re))
						{
							name = found[1];
							if(!arr[name])
								arr[name] = new Array();
							if(found[2])
							{
								subkey = found[2]
								arr[name][subkey] = '';
							}
						}

						//-- loop trough the options and add/set if selected
						for (var i=0; i<node.options.length; i++)
						{
							var opt = node.options[i];
							if (opt.selected)
							{
								var value = (opt.value == '') ? node.name : opt.value;
								if ( arr[name] instanceof Array)
								{
									if(subkey)
										arr[name][subkey] = opt.value;
									else
										arr[name].push(opt.value);
								}
								else
									arr[name] = opt.value;
							}
						}
					}
				break;

	            default:
	                if ( node.hasChildNodes() )
	                    this._foldForm(node.firstChild, arr);

			} //end switch tagName
	    }
	    while ( node = node.nextSibling  );

	    return arr;
	}

};
