/**
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id: FormBehaviour.js 1654 2006-01-18 14:50:47Z Martijn van den Broek $
 * @since 18-11-2005
 */

/**
 * LangIf.. check if there is such a object.. cache translations, else make a dummy
 */
if (typeof langIf == 'undefined')
	langIf = { getText: function(arg) { return arg }, translateNodes: function() {}, translateHandles: function() {} };

/**
 * FormBehaviour
 * usage: <input type="text" name="first_name" displayname="Voornaam" behaviour="required;alphanumeric" />
 */
function FormBehaviour()
{
	//this._nodeErrors = new Array(); // nodes with error texts

	this._plugins = new Array(); // plugin definitions
	this._checks = new Array(); // running plugins (instances)
	this._errors = new Array(); // failed plugins (instances)

	this.nFirstError = null;

	langIf.translateHandles( new Array('FormBehaviour MessageBox Title', 'Please correct the form errors') );

	// init default plugins
	this.addPlugins( [].append({
		required:     FormBehaviourRequiredPlugin,
		email:        FormBehaviourEmailPlugin,
		alphanumeric: FormBehaviourAlphaNumericPlugin,
		alpha:        FormBehaviourAlphaPlugin,
		numeric:      FormBehaviourNumericPlugin,
		decimal:      FormBehaviourDecimalPlugin,
		url:          FormBehaviourUrlPlugin,
		phone:        FormBehaviourPhonePlugin,
		day:          FormBehaviourDayPlugin,
		month:        FormBehaviourMonthPlugin,
		year:         FormBehaviourYearPlugin,
		integer:      FormBehaviourIntegerPlugin,
		date:         FormBehaviourDatePlugin,
		password:     FormBehaviourPasswordPlugin,
		vat:          FormBehaviourVatPlugin,
        len:          FormBehaviourLengthPlugin,
		iban:         FormBehaviourIbanPlugin,
		float:        FormBehaviourDecimalPlugin,
		brand:        FormBehaviourPluginBrand,
		companyname:  FormBehaviourCompanyNamePlugin,
		address:      FormBehaviourAddressPlugin,
		zipcode:      FormBehaviourZipCodePlugin,
		personname:   FormBehaviourPersonNamePlugin,
		cityname:     FormBehaviourCityNamePlugin,
		swiftcode:    FormBehaviourSwiftCodePlugin,
		bankaccount:  FormBehaviourBankAccountPlugin,
		ponumber:     FormBehaviourPONumberPlugin,
		ukpostcode:	  FormBehaviourUKPONumberPlugin,
        ukcountry:	  FormBehaviourUKCountryPlugin,
        nonukcountry:	  FormBehaviourNonUKCountryPlugin,
        ukdependency:	  FormBehaviourUKDependencyPlugin,
        ukphone:      FormBehaviourUKPhonePlugin,
		departmentname:   FormBehaviourDepartmentNamePlugin,
		question:     FormBehaviourQuestionPlugin,
		emailsubject: FormBehaviourEmailSubjectPlugin,
		producerregno:    FormBehaviourProdRegistrationNoPlugin,
		comment:      FormBehaviourCommentPlugin,
		cronmask:     FormBehaviourCronMaskPlugin,
		glaccount:    FormBehaviourGLAccountsPlugin,
		productcategory:  FormBehaviourProductCategoryPlugin,
		productcomponent: FormBehaviourProductComponentPlugin,
		invoicenumber:    FormBehaviourInvoiceNumberPlugin,
		ipwildcard:	  FormBehaviourIPWildcardPlugin,
		approvalatf:	  FormBehaviourPluginApprovalNoATF,
		approvalae:	  FormBehaviourPluginApprovalNoAE
	}));
}

/**
 * usage: <form onsubmit="return fb.validate(this);">
 * @param Object el
 * @return boolean
 */
FormBehaviour.prototype.validate = function(nForm)
{
	if (!nForm)
		throw "supplied form is not a valid Node";

	// reset
	this.nFirstError = null;
	this._checks = new Array(); // running plugins (instances)
	this._errors = new Array(); // failed plugins (instances)

	// start checking
	// FIXME : add textarea and fix tinymce
	['input', 'select'].forEach(function(sTag, iIndex, aArray)
	{
		var nodes = nForm.getElementsByTagName(sTag);
		for (var i=0; i<nodes.length; i++)
		{
			this.clearError(nodes[i]);
			this.behave(nodes[i]);
		}

	}, this);

	if (this._checks.count() == 0 && this._errors.count() == 0)
	{
		dumpln('Form is valid');
		return true; // no more busy checks and no errors, we are valid
	}
	else if (this._checks.count() > 0)
	{
		// some checks are still busy, wait for them to finish
		dumpln('still got running checks: ' + this._checks.count() );
		alert('some checks ('+this._checks.count()+') still running!');
		return false;
	}
	else
	{
		// No more busy checks, only errors. Display them and return.
		this._errors.forEach(function(instPlug, i, array)
		{
			var nNode = instPlug.getNode();
			var sErr  = langIf.getText(instPlug.getMessage());

			this.clearError(nNode);
			this.displayError(nNode, sErr);
		}, this);

		MessageBox.prototype.Critical(
			langIf.getText('FormBehaviour MessageBox Title'),
			langIf.getText('Please correct the form errors') );;

		return false;
	}
}

/**
 * @param Object el
 * @return string rv
 */
FormBehaviour.prototype.behave = function(el)
{
	var attr = el.getAttribute('behaviour');

	if (!attr)
		return;

	if (el.disabled)
		return;

	behaviours = attr.split(';');
	for(var i=0;i<behaviours.length;i++)
	{
		var reg = /([^(]*)(?:\((.*)\))?/;
		var matches = behaviours[i].match(reg);
		var behaviour = matches[1];

		if (el.value == '' && behaviour != 'required')
			continue;

		if (!behaviour)
			continue;

        if(behaviour === 'length') {
            behaviour = 'len';
        }

		// Make, start and save a new instance of a plugin
		try {
			//dumpln('Starting plugin for behaviour ' + behaviour);
			var pInst = new this._plugins[behaviour](el);
		} catch (e) {
			dumpln("ERROR: Could not init clientside plugin " + behaviour + " not found in " + this._plugins);
			continue;
		}

		var args = [];
		if (typeof matches[2] != 'undefined')
			args = matches[2].split(',');

		pInst.check.apply(pInst, args);

		switch (pInst.getStatus())
		{
			case FB_STATUS_BUSY:
				this._checks.push(pInst);
				break;

			case FB_STATUS_INVALID:
				this._errors.push(pInst);
				break;

			case FB_STATUS_VALID:
				// all cool :)
				break;
		}
	}
}

/**
 * Adds an error to an element
 * @param Node nElement
 * @param String sError
 */
FormBehaviour.prototype.displayError = function(nElement, sError)
{

	// If we dont have a nElement we present the user with a nice message box
	if (typeof nElement == 'undefined')
	{
		MessageBox.prototype.Critical('', sError);
		return;
	}


	// else mark the element as error
	addCssClass(nElement, 'error');

	// save a reference so we can remove the error div later
	orgElement = nElement;

	// only show error if the noerrordesc attribute not is set
	if (! nElement.getAttribute('noerrordesc'))
	{
		var div = createEl('DIV');
		div.innerHTML = sError;
		div.className = 'error';

		if (nElement.type.toLowerCase() == 'radio')
		{
			if (label = getLabelByRadio(nElement))
				nElement = label

			try {
				insertAfter(div, nElement);
			} catch (e) {
				//dumpln('ERROR: ' + e.message)
			}
		}
		else {
			try {
				nElement.parentNode.appendChild(div);
			} catch (e) {
				//dumpln('ERROR: ' + e.message)
			}
		}

		// save error div in input node
		orgElement._errorDiv = div;

		// clear references at unload
		addEvent(window, 'unload', function() {
			orgElement._errorDiv = null;
		});
	}
}

FormBehaviour.prototype.clearError = function(nElement)
{
	// remove error span
	if (nElement._errorDiv) {
		removeNode(nElement._errorDiv);
		nElement._errorDiv = null;
	}

	// remove css class from input
	removeCssClass(nElement, 'error');
}

/**
 * Checks if the given node has errors
 * @param Node
 * @return bool valid
 */
FormBehaviour.prototype.nodeIsValid = function(nNode)
{
	var rv = true;
	this._errors.forEach(function(instPlug, i, array)
	{
		if (nNode === instPlug.getNode())
			return rv = false;
	}, this);

	return rv;
}

/**
 * registers a plugin without translating
 * @private
 */
FormBehaviour.prototype._addPlugin = function (sBehaviour, pPlugin)
{
	if (typeof pPlugin.prototype.check == 'undefined')
		throw 'addPlugin() - Plugin '+pPlugin+' must have a check method';

	if (typeof pPlugin.prototype.getMessage == 'undefined')
		throw 'addPlugin() - Plugin '+pPlugin+' must have a getMessage method';

	this._plugins[sBehaviour] = pPlugin;
}

/**
 * registers a single plugin including translating
 * @public
 */
FormBehaviour.prototype.addPlugin = function (sBehaviour, pPlugin)
{
	this._addPlugin(sBehaviour, pPlugin); // call private version
	langIf.translateHandles(pPlugin.prototype.getHandles());
}

/**
 * registers multiple plugins and translates all in once
 * @public
 */
FormBehaviour.prototype.addPlugins = function(aPlugs)
{
	var trHandles = [];
	aPlugs.forEach(function(pPlugin, sBehaviour, array)
	{
		pPlugin.prototype.getHandles().forEach(function(el, i, a) {
			trHandles.push(el);
		}, this);
		this._addPlugin(sBehaviour, pPlugin);
	}, this);

	langIf.translateHandles(trHandles);
}

/**
 * Sets a behaviour string for a node
 * @static
 * @param nNode
 * @param sBehaviour
 */
FormBehaviour.prototype.setBehaviour = function(nNode, sBehaviour)
{
	if (!nNode)
		throw "nNode is not a valid node";

	var attr = nNode.getAttribute('behaviour');
	if (!attr)
		attr = '';

	var behaviours = attr.split(';');
	var found = false;
	for (var i=0; i<behaviours.length; i++)
		if (behaviours[i] == sBehaviour)
			found = true;

	if (!found)
		nNode.setAttribute('behaviour', attr += ( (attr == '') ? sBehaviour : ';'+sBehaviour ) );
}

/**
 * Removes a behaviour string for a node
 * @static
 * @param nNode
 * @param sBehaviour
 */

FormBehaviour.prototype.removeBehaviour = function(nNode, sBehaviour)
{
	var attr = nNode.getAttribute('behaviour');
	if (!attr)
		attr = '';

	var behaviours = attr.split(';');
	var behaviour = '';

	for (var i=0; i<behaviours.length; i++)
	{
		if (behaviours[i] != sBehaviour)
			behaviour += (behaviours[i]+';');
	}

	nNode.setAttribute('behaviour', behaviour);
}

/**
 * PLUGIN (ABSTRACT)
 */
FB_STATUS_BUSY = 0;
FB_STATUS_VALID = 1;
FB_STATUS_INVALID = 2;

function FormBehaviourAbstractPlugin(nNode)
{
	this._nNode = nNode;
}

FormBehaviourAbstractPlugin.prototype =
{
	_status: FB_STATUS_BUSY,
	_message: 'abstract message',
	_handles: null,

	getNode: function() {
		return this._nNode;
	},

	getStatus: function() {
		return this._status;
	},

	getMessage: function()
	{
		return this._message;
	},

	getHandles: function()
	{
		if (this._handles == null)
			return [this._message];

		this._handles.push(this._message);
		return this._handles;
	},

	// object method?
	extend: function(source)
	{
  		for (property in source)
    		this[property] = source[property];
	},

	mkPlugin: function(obj)
	{
		var rv = new Function('this._nNode = arguments[0];');
		rv.prototype = new FormBehaviourAbstractPlugin();
		rv.prototype.extend(obj);
		return rv;
	}
}

/**
 * PLUGIN REGEX
 */

FormBehaviourRegexPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin(
{
	_message: 'regex default message',
	_regex: new RegExp(),

	check: function()
	{
		this._status = this._nNode.value.match(this._regex) ?  FB_STATUS_VALID : FB_STATUS_INVALID;
	},

	mkPlugin: function(regex, msg)
	{
		var rv = new Function('this._nNode = arguments[0];');
		rv.prototype = new FormBehaviourRegexPlugin();
		rv.prototype.extend({_message: msg, _regex: regex});
		return rv;
	}
});

/**
 * REGEX PLUGINS
 */

// email
FormBehaviourEmailPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/,
	'Not a valid Email');

// alphanumeric
//FormBehaviourRegexPlugin.prototype.mkPlugin(/^([a-zA-Z0-9\s])+$/,'Only numbers and letters are valid');
FormBehaviourAlphaNumericPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^&*\(\)_\-\+=\/|\\:;"'<>.\?]+$/,
	'Only numbers and letters are valid');
// block wrong characters from being submited, but the actual check is done on the server side (for utf8, etc...)

// alpha
FormBehaviourAlphaPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^&*\(\)_\-\+=\/|\\:;"'<>.\?]+$/,
	'Only letters are valid');
// block wrong characters from being submited, but the actual check is done on the server side (for utf8, etc...)
// original expression:	/^([a-z\s])+$/i,

// numeric
FormBehaviourNumericPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([0-9\s])+$/,
	'Only numbers are valid');

// decimal
FormBehaviourDecimalPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[0-9]+\.?[0-9]*$/,
	'Only decimal numbers are valid');

// IP
FormBehaviourIPWildcardPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)?)?)?$/,
	'Value is not a valid/allowed IP addresst');

// url
FormBehaviourUrlPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/.*/,
	'Not a valid URL');

// phone
FormBehaviourPhonePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^\+?(?:[\-0-9\s#*]|\(\d+\))+$/,
	'Not a valid phone or fax number');

// day
FormBehaviourDayPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([1-9]|[1-2][0-9]|3[01])$/,
	'Not a valid day');

// month
FormBehaviourMonthPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^(1[0-2]|[1-9])$/,
	'Not a valid month');

// year
FormBehaviourYearPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([0-9]){4}$/,
	'Not a valid year');

// vat
FormBehaviourVatPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([A-Z]{2})([0-9A-Z]{2,12})$/,
	'Invalid VAT format');
// old expression: 	/^[a-zA-Z0-9 ]+$/,

// IBAN
FormBehaviourIbanPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[A-Z0-9 ]+$/,
	'IBAN has invalid format');

// Brand
FormBehaviourPluginBrand = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*_\+=|\\:;"<>\?]+$/,
	'Not a valid brand name');

// Company name
FormBehaviourCompanyNamePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^_\=|\:;<>\?]+$/,
	'Not a valid company name');

// Address
FormBehaviourAddressPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*_\+=|\\:;"<>\?]+$/,
	'Not a valid address');

// zipcode
FormBehaviourZipCodePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[a-zA-Z0-9 -]+$/,
	'Not a valid postal code');

// person name
FormBehaviourPersonNamePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^_\+=|\\:;"'<>\?]+$/,
	'Not a valid name');

// city name
FormBehaviourCityNamePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*_\+=\/|\\:;"'<>\?]+$/,
	'Not a valid city name');

// swift code
FormBehaviourSwiftCodePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[a-zA-Z0-9 \/]{1,12}$/,
	'Not a valid SWIFT code');

// bankaccount
FormBehaviourBankAccountPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@$%^*\(\)_\+=|\\:;"'<>\?]+$/,
	'Not a valid bankaccount');

// PO Number
FormBehaviourPONumberPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[a-zA-Z0-9 ]+$/,
	'Not a valid PO no.');

//UK PO Number
FormBehaviourUKPONumberPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[A-Z]{1,2}[0-9R][0-9A-Z]? [0-9][A-Z]{2}$/,
	'Not a valid UK PO no.');

// UK Phone number
FormBehaviourUKPhonePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
    /^[0-9]{10,35}$/,
    'Not a valid Phone Number.');

// Department name
FormBehaviourDepartmentNamePlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*_\+=|\\:;"'<>\?]+$/,
	'Not a valid department name');

// question
FormBehaviourQuestionPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~!@#$%^*\(\)_\+=|\\<>]+$/,
	'Not a valid question');

// email subject
FormBehaviourEmailSubjectPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]~#$%^*\+=|\\"<>]+$/,
	'Not a valid email subject');

// producer registration number
FormBehaviourProdRegistrationNoPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[a-zA-Z0-9\/\\ -]+$/,
	'Not a valid producer reg. no.');

// comment
FormBehaviourCommentPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*\(\)_\+=\/|\\"'<>]+$/,
	'Not a valid comment');

// cron mask
FormBehaviourCronMaskPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([\d *,-])+$/,
	'Not a valid cron mask');

// GL accounts
FormBehaviourGLAccountsPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^([A-Z0-9_])+$/,
	'Not a valid value');

// Product categories
FormBehaviourProductCategoryPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*\(\)\+=\/|\\"'<>]+$/,
	'Not a valid product category');

// Product Components
FormBehaviourProductComponentPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[^\[\]{}~`!@#$%^*\(\)\+=\/|\\"'<>]+$/,
	'Not a valid product component');

// Invoice numbers
FormBehaviourInvoiceNumberPlugin = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^[a-zA-Z0-9 \/\\]+$/,
	'Not a valid invoice number !');

// ATF Aproval No
FormBehaviourPluginApprovalNoATF = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^((WEE\/)[A-Z]{2}[0-9]{4}[A-Z]{2})(\/ATF)?$/,
	'Not a valid ATF Approval Number!');
// AE Aproval No
FormBehaviourPluginApprovalNoAE = FormBehaviourRegexPlugin.prototype.mkPlugin(
	/^((WEE\/)[A-Z]{2}[0-9]{4}[A-Z]{2})(\/AE|\/EXP)?$/,
	'Not a valid AE Approval Number!');


/**
 * PLUGIN REQUIRED
 */
FormBehaviourRequiredPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin(
{
	_message: 'Field is required',

	/**
	 * The name and value of an element to depend on can be given.
	 * That depend element must have a value, or the optional value given by sDependValue
	 *
	 * @param String sDependEl Optional
	 * @param Bool bEmpty Optional
	 * @param String sDependValue Optional
	 */
	check: function(sDependEl, bEmpty, sDependValue)
	{
		if (typeof bEmpty == 'undefined' || bEmpty == 'false' )
			bEmpty = false;
		else
			bEmpty = true;

		if (typeof sDependEl != 'undefined' && sDependEl != '' )
		{
			var required = true;

			try
			{
				var dependEl = $$(sDependEl);
			} catch (e)
			{
				alert(e);
			}
			if ( dependEl === UNDEFINED )
				return this._status = FB_STATUS_VALID;
			switch (dependEl.type)
			{
				case 'radio':

					if (typeof sDependValue != 'undefined')
					{
						required = false;

						var radios = document.getElementsByName(sDependEl);
						for (var i=0; i<radios.length; i++)
						{
							var radio = radios[i];

							if (radio.checked && sDependValue.split('|').in_array(radio.value) )
								required = true;
						}
					}

				break;

				case 'checkbox':
					required = (dependEl.checked == bEmpty);
				break;

				default:
					if (bEmpty)
						required = (dependEl.value != '');
					else
					{
						if (typeof sDependValue == 'undefined')
							required = (dependEl.value == '');
						else
						{
							dumpln(sDependValue.split('|'));
							required = sDependValue.split('|').in_array(dependEl.value);
						}
					}
			}
			if (required) {
				if (this._nNode.type == 'checkbox')
					this._status = (this._nNode.checked) ? FB_STATUS_VALID : FB_STATUS_INVALID;
				else {
					// trim input value
					var v = this._nNode.value.replace(/^\s+|\s+$/g,"");
					this._status = (v !== '') ? FB_STATUS_VALID : FB_STATUS_INVALID;
				}
			}
			else
				this._status = FB_STATUS_VALID;
		}
		else
		{
			/**
			 * Radioboxes get special attention:
			 *  - always set this.node to the first radio node of the group
			 *  - only set status to valid if one of the radios is checked
			 */
			if (this._nNode.tagName == 'INPUT' && this._nNode.type == 'radio')
			{
				var radios = Radio_getNodesByName(this._nNode.name);
				this._nNode = radios[0];

				this._status = FB_STATUS_INVALID;
				for (var i=0; i<radios.length; i++) {
					if (radios[i].checked)
						this._status = FB_STATUS_VALID;
				}
				return;
			}

			// trim input value
			var vv = this._nNode.value.replace(/^\s+|\s+$/g,"")
			this._status = vv ? FB_STATUS_VALID : FB_STATUS_INVALID;
		}
	}
});


/**
 * PLUGIN INTEGER
 */
FormBehaviourIntegerPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin(
{
	_message: 'Not an integer',
	_handles: ['Integer too small', 'Integer too big'],

	check: function(minNum, maxNum)
	{
		if ((!minNum || minNum == 'null' || minNum == 'false') && minNum !== '0')
			minNum = false;
		else
			minNum = parseInt(minNum, 10);

		if ((!maxNum || maxNum == 'null' || maxNum == 'false') && maxNum !== '0')
			maxNum = false;
		else
			maxNum = parseInt(maxNum, 10)

		dumpln('minNum: ' + minNum);
		dumpln('maxNum: ' + maxNum);
		dumpln('value: ' + this._nNode.value);

		var el = this._nNode;
		if(el.value !== parseInt(el.value, 10).toString()) {
			this._status = FB_STATUS_INVALID;
			return;
		}

		var value = parseInt(el.value, 10);

		if(minNum!==false && value < minNum) {
			this._status = FB_STATUS_INVALID;
			this._message = this._handles[0];
			return;
		}

		if(maxNum!==false && value > maxNum) {
			this._status = FB_STATUS_INVALID;
			this._message = this._handles[1];
			return;
		}

		this._status = FB_STATUS_VALID;
	}
});


/**
 * PLUGIN DATE
 */
FormBehaviourDatePlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin(
{
	_message: 'Not a valid date',
	_handles: ['Invalid format', 'Invalid date'],

	check: function(minNum, maxNum)
	{
		if(typeof DATE_FORMAT == 'undefined')
		{
			DATE_FORMAT = /(\d{2})\-(\d{2})\-(\d{4})/
			DATE_FORMAT_STRING = 'dd-mm-yyyy';
		}

		if (matches = DATE_FORMAT.exec(this._nNode.value))
		{
			var year = matches[3];
			var month = matches[2];
			var day = matches[1];

			var date = new Date(year, month-1, day);

			if (date.getFullYear() != year ||
				date.getMonth()+1 != month ||
				date.getDate() != day )
			{
				this._message = this._handles[1];
				return this._status = FB_STATUS_INVALID;
			}
		}
		else {
			this._message = this._handles[0];
			return this._status = FB_STATUS_INVALID;
		}

		this._status = FB_STATUS_VALID;
	}
});


/**
 * PLUGIN DATE
 */
FormBehaviourPasswordPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin(
{
	_message: 'Password do not match',
	_handles: ['Password do not match', 'Invalid Password'],

	check: function(sDependEl)
	{
		if (typeof sDependEl != 'undefined')
		{
			var dependEl = getElByName(sDependEl);

			if (this._nNode.value != dependEl.value)
			{
				this._status = FB_STATUS_INVALID;
				return
			}
		}

		this._status = FB_STATUS_VALID;
	}
});


/**
 * PLUGIN LENGTH
 */
FormBehaviourLengthPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin({
    _message: 'Incorrect length',
    _handles: ['Value is too short', 'Value is too long'],

    check : function (minLength, maxLength) {
        if(this._nNode.value.length < minLength) {
            this._message = this._handles[0];
            this._status = FB_STATUS_INVALID;
            return;
        }
        if(this._nNode.value.length > maxLength) {
            this._message = this._handles[1];
            this._status = FB_STATUS_INVALID;
            return;
        }
        this._status = FB_STATUS_VALID;
    }
});

/**
 * UK COUNTRIES
 */
FormBehaviourUKCountryPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin({
    _message: 'This is not a valid UK country',
    check : function () {
        this._status = (this._nNode.value.substr(0, 4) != 'UK -') ? FB_STATUS_INVALID : FB_STATUS_VALID;
    }
});

FormBehaviourNonUKCountryPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin({
    _message: 'This should be a country outside of the UK',
    check : function () {
        this._status = (this._nNode.value.substr(0, 4) == 'UK -') ? FB_STATUS_INVALID : FB_STATUS_VALID;
    }
});

FormBehaviourUKDependencyPlugin = FormBehaviourAbstractPlugin.prototype.mkPlugin({
    _message: 'Country should be outside of the UK',
    check : function (field, group) {

        var ar = document.getElementById('ifRadiohas_authorized_representative2').checked;

        // if country and group is business
        if(group == 'business' && field == 'country') {
            if(ar) {
                this._message = 'Country must be outside of UK because of the AR';
                this._status = (this._nNode.value.substr(0, 4) == 'UK -') ? FB_STATUS_INVALID : FB_STATUS_VALID;
                return;
            } else {
                this._message = 'Country must be in the UK, else you need an AR';
                this._status = (this._nNode.value.substr(0, 4) != 'UK -') ? FB_STATUS_INVALID : FB_STATUS_VALID;
                return;
            }
        }

        if(field == 'postcode') {

            var country = document.getElementsByName(group+'[address][country]')[0].value;
            if(country.substr(0, 4) == 'UK -') {
                var reg = /^[A-Z]{1,2}[0-9R][0-9A-Z]? [0-9][A-Z]{2}$/;
                this._message = 'Not a valid UK Postal Code';
                this._status = this._nNode.value.match(reg) ?  FB_STATUS_VALID : FB_STATUS_INVALID;
                return;
            }
        }

        this._status = FB_STATUS_VALID;

    }
});

/**
 * An object to post a form via AJAX and get any validation errors back from the server.
 */
FormBehaviourSubmission = function(formId)
{
	if(typeof formId == 'string' || typeof formId == 'String')
		this.formElement = getElById(formId);
	else
		this.formElement = formId;
	if(!this.formElement)
	{
		throw "FormBehaviourSubmission::constructor - Form: " + formId + " does not exist!";
	}
}

FormBehaviourSubmission.prototype.foldForm = function(node, arr)
{
    if(typeof node == 'undefined')
    	node = this.formElement;
    if(typeof arr == 'undefined')
    	arr = new Array();
    do
    {
        if ( node.nodeType != 1 )
            continue;

		var normalizedName = node.tagName.toLowerCase();
		if(normalizedName in FormBehaviourSubmissionElementSerializers && !node.disabled)
		{
			var result = FormBehaviourSubmissionElementSerializers[normalizedName](node);
			if(result) //we have a result which is always an Array
			{
				//check if we have multiple values ie. multi-select, multi-checkboxes etc
				if(result[0] instanceof Array)
				{

					for(var i = 0; i < result.length; i++)
					{
						arr.push(result[i]);
					}
				}
				else
				{
					arr.push(result);
				}
			}
		}
		else
		{
			if ( node.hasChildNodes() )
        		this.foldForm(node.firstChild, arr);
        }

    }
    while (node = node.nextSibling);
    return arr;
}


FormBehaviourSubmissionElementSerializers = {

	input: function(node)
	{
		switch(node.type.toLowerCase())
		{
			case 'submit':
			case 'hidden':
			case 'password':
			case 'text':
				return FormBehaviourSubmissionElementSerializers.text(node);
				break;
			case 'checkbox':
			case 'radio':
				return FormBehaviourSubmissionElementSerializers.inputBoolean(node);
				break;
		}
		return false;
	},

	inputBoolean: function(node)
	{
		if (node.checked)
      		return [node.name, node.value];
      	return false;
	},

  	text: function(node)
  	{
		return [node.name, node.value];
	},

	select: function(node)
	{
    	var values = new Array();
    	for(var i = 0; i < node.length; i++)
    	{
    		var opt = node.options[i];
    		if(!opt.selected)
    			continue;
    		values.push([node.name, opt.value]);
    	}
    	if(!values.length)
    		return false;
    	return values;
	},

	textarea: function(node)
	{
		//first we check if this is a tinyMCE field
		if (typeof tinyMCE != 'undefined' && (inst = FormBehaviourSubmissionElementSerializers.isTinyMCEField(node)))
			return FormBehaviourSubmissionElementSerializers.getTinyMCEContent(node, inst);
		return [node.name, node.value];
	},

	isTinyMCEField: function(node)
	{
		for(i in tinyMCE.instances)
		{
			var inst = tinyMCE.instances[i];
			if (!tinyMCE.isInstance(inst))
				continue;
			if(inst.formElement == node)
				return inst;
		}
		return false;
	},

	getTinyMCEContent: function(node, inst)
	{
		inst.switchSettings(); // FIXME: i think we need this line... not sure why
		return [node.name, tinyMCE.getContent(inst.editorId)];
	}
};
