/**
 * LangAdmin
 * @author Martijn van den Broek
 * @version $Id$
*/

/**
 * Constructor
 * @param array<array> dataset (optional)
 * @param LanguageInterface (optional)
 */
function TableFactory(dataset, langIf)
{
	if (typeof(dataset) != 'undefined')
		this.setData(dataset);
	else
		this.dataset = new Array( new Array() );

	if (typeof(langIf) != 'undefined')
		this.langIf = langIf;
	else
		this.langIf = null;

	this._tableClassName = '';
	this._hiddenColumns = new Array();
	this._valueProcessors = new Array();
	this._rowProcessors = new Array();
	this._caption = null;
	this._modulus = 2;
	this._sortingMap = [];

	this.addRowProcessor( function(tRow) {
		tRow.className += ' tr'+((tRow.sectionRowIndex) % this._modulus);
	}, this);
}

/**
 * @param array<array> dataset
 */
TableFactory.prototype.setData = function(dataset)
{
	if (! dataset instanceof Array)
		throw new Error('TableFactory::TableFactory() - supplied argument dataset is not a valid');

	this.dataset = dataset;
}

/**
 * @param string caption
 */
TableFactory.prototype.setCaption = function(caption)
{
	this._caption = caption;
}

/**
 * @param string className
 */
TableFactory.prototype.setClassName = function(className)
{
	this._tableClassName = className;
}

/**
 * @param int modulus
 */
TableFactory.prototype.setModulus = function(modulus)
{
	this._modulus = modulus;
}

/**
 * @param Array<string> columns
 */
TableFactory.prototype.hideColumns = function(columns)
{
	this._hiddenColumns = columns;
}

/**
 * Adds a value processor for the given column.
 * The callback will be executed for every row for that column.
 * The callback has one argument with a reference to the TD.
 *
 * Example. To add a euro sign after each value in the PRICE column:
 * <code>
 * 	table.addValueProcessor( function(nTd) { 'PRICE', nTd.innerHTML += '&euro;'; } );
 * </code>
 *
 * @param string column
 * @param function callback
 */
TableFactory.prototype.addValueProcessor = function(column, func)
{
	if (getObjectType(this._valueProcessors[column]) == 'undefined')
		this._valueProcessors[column] = new Array();

	this._valueProcessors[column].push(func);
}

/**
 * Adds a row processor callback which gets executed for every row.
 * The callback has an argument with a reference to the row.
 * @param function callback
 */
TableFactory.prototype.addRowProcessor = function(func)
{
	this._rowProcessors.push(func);
}

/**
 * @param array<(string) columnname => (function/string)compare >
 * <code>table.setSortingMap( [].append( {'tr1_translation':'String', 'tr2_translation':'String'} ) );</code>
 */
TableFactory.prototype.setSortingMap = function(sortingMap)
{
	if (typeof SortingHat == 'undefined')
		throw 'Undefined class SortingHat';

	if (typeof sortingMap == 'undefined')
		sortingMap = [];

	this._sortingMap = sortingMap;
}

/**
 * Looks up the TH by the _head attribute and replaces the innerHTML with sTitle
 * @param string sHead
 * @param string sTitle
 */
TableFactory.prototype.renameColumn = function(sHead, sTitle)
{
	var th = false;
	for (var i=0; i<this.nTheadRow.cells.length; i++)
	{
		if (this.nTheadRow.cells[i]._head == sHead)
			th = this.nTheadRow.cells[i];
	}

	if (!th)
		throw new Error('TableFactory::renameColumn() - head '+sHead+' does not exist');

	th.innerHTML = sTitle;
}

/**
 * Creates the Table and returns a reference to the Node.
 * @return HtmlNodeTable
 */
TableFactory.prototype.create = function()
{
	this._debug('creating table');

	var nTable = document.createElement('TABLE');
	nTable.className = this._tableClassName;

	var sortingHat = new SortingHat(nTable);
	functionMap = [];

	if (this._caption != null)
	{
		var nCaption = document.createElement('CAPTION');
		nCaption.innerHTML = this._caption;
		nTable.appendChild(nCaption);
	}

	nThead = document.createElement('THEAD');
	nTable.appendChild(nThead);

	var nTheadRow = document.createElement('TR');
	this.nTheadRow = nTheadRow; // save for future reference (e.g. to rename)
	nThead.appendChild(nTheadRow);

	var nTbody = document.createElement('TBODY');
	nTable.appendChild(nTbody);

	var header_built = false;
	var aTranslations = new Array();

	this._debug('looping dataset');

	// rows
	this.dataset.forEach( function(aRow, i, array)
	{
		this._debug('start row');

		var nTr = document.createElement('TR');
		nTbody.appendChild(nTr);

		if ((getObjectType(aRow) != 'Array'))
			throw new Error('TableFactory::create() - Dataset child is not an array ('+print_r(aRow)+')');

		// columns
		aRow.forEach(function(cData, head, array2)
		{
			// hidden columns
			if (this._hiddenColumns.in_array(head))
				return; // alternative continue

			// headers
			if (!header_built)
			{
				var th = document.createElement('TH');
				nTheadRow.appendChild(th);
				th._head = head; // so we can find it back later (at rename for example)
				th.innerHTML = head; // will be replaced by translation
				aTranslations[head] = th;

				// apply sorting
				if (typeof this._sortingMap[head] != 'undefined')
				{
					functionMap[th.cellIndex] = this._sortingMap[head];
					addEvent(th, 'click', hitch(sortingHat, 'sort', th.cellIndex));
					addEvent(th, 'mouseover', function() { this.style.cursor = 'pointer'; } );
				}
			}

			// TD
			var nData = document.createElement('TD');
			nTr.appendChild( nData );

			// set td contents to string or add childs if its a node
			if (cData != null && (typeof cData.nodeType != 'undefined') && (cData.nodeType == 1) )
				nData.appendChild(cData);
			else
			{
				if (cData == null) // IE FIX
					cData = '';

				nData.innerHTML = cData;
			}

			// execute all value processors for this column
			if ( this._valueProcessors[head] )
			{
				this._valueProcessors[head].forEach( function(func, i3, arr3) {
					this._debug('starting value processor');
					func.apply(this, [nData, aRow]);
					this._debug('finished value processor');
				}, this);
			}
		}, this);

		// set header translations
		if (!header_built && this.langIf)
			langIf.translate(aTranslations);

		header_built = true;

		// execute all row processors
		this._rowProcessors.forEach( function(func, i3, arr3) {
			this._debug('starting row processor');
			func.apply(this, [nTr, aRow]);
			this._debug('finished row processor');
		}, this);

		this._debug('finished row');

	}, this);

	sortingHat.setFunctionMap(functionMap);
	this._nTable = nTable;

	return nTable;
}

/**
 * Creates and inserts the table at the given node
 * @param HtmlNode node
 */
TableFactory.prototype.insertAt = function(node)
{
	if (!node)
		throw 'Supplied argument node is not a valid node.';

	node.appendChild(this.create());
}

TableFactory.prototype._debug = function(sStr)
{
	//dumpln(sStr);
}

