#!/usr/bin/php
<?php
error_reporting(E_ALL);
/**
 * Created by PhpStorm.
 * User: marten
 * Date: 26/10/15
 * Time: 9:45 AM
 */

define('REQUIRED_YEAR_FOR_PROVISIONAL_REPOPULATE', 2015);

class RepopulateGermanyException extends Exception {
	// Germany has been wiped out, we need to repopulate germany.
}

/**
 * Class RepopulateGermanProvisional
 *
 * Germany introduces Categories from the 1st of November 2015, this means we are required to alter the provisional
 * for the last quarter (months, 10, 11 & 12).
 *
 * We will add the current numbers to GDL category and add an additional line for LED (with 0).
 *
 * Germany only has WEEE Category LAMPS (Product = 1)
 */
class RepopulateGermanProvisional {

	var $year = REQUIRED_YEAR_FOR_PROVISIONAL_REPOPULATE;
	var $monthsWithBothCategories = array(10, 11, 12);

	public function __construct($db, $crso_id, $verbose=false, $dry_run=false) {
		$this->db =& $db;
		$this->crso = new CRSO($this->db,array('relation'=> new Relation($this->db,$crso_id, true)), true);;
		$this->verbose = $verbose;
		$this->execute = !$dry_run;
	}

	public function repopulate() {
		$this->validateRequirements();
		$this->populate();
		if(!$this->execute) {
			$this->output("\n\nDry-run was enabled, please rerun without the --dry-run to commit the changes.");
		}
	}

	/**
	 * We validate country settings and if provisional has been triggered yet.
	 */
	private function validateRequirements() {

		$this->crso->relation->set();
		$this->output(sprintf('Starting validation process for CRSO `%s`:', $this->crso->relation->name));
		require_once(ELC_CLASSES.'Country.php');
		// Verify the COUNTRY SETTINGS:
		$country = new Country($this->db, $this->crso->getCountry(), true);
		$this->output(' - country settings ...', false);

		if(!$country->has(HasDeclByCategory, 1)) {
			throw new RepopulateGermanyException('`Product category` is not enabled for this CRSO country.');
		}
		if(!$country->has(HasDeclOfWeight, 1)) {
			throw new RepopulateGermanyException('`Weight` is not enabled for this CRSO country.');
		}
		if(!$country->has(HasDeclByBrand, 1)) {
			throw new RepopulateGermanyException('`Brand` is not enabled for this CRSO country.');
		}
		if($country->has(HasDeclarationDimension, 1)) {
			throw new RepopulateGermanyException('`Dimension` is enabled for this CRSO country, deny repopulate.');
		}
		$this->output("[OK]");

		// We should have two categories. GDL & LED
		$this->output(' - product categories .', false);

		// We validate if the product category descriptions have been added.
		$sql = "SELECT PRODUCT_ID, PRODUCT_CATEGORY_ID, NAME, DESCRIPTION FROM TBL_PRODUCT_CATEGORY WHERE CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id));
		$q->execute();

		$cnt = 0;
		while($r = $q->fetch()) {
			$cnt++;
			if($r->NAME == 'GDL') {
				define('CAT_GDL', $r->PRODUCT_CATEGORY_ID);
				continue;
			}
			if($r->NAME == 'LED') {
				define('CAT_LED', $r->PRODUCT_CATEGORY_ID);
			}
		}
		if($cnt != 2) {
			throw new RepopulateGermanyException(sprintf('`%s` Product Categories found, we need exactly 2. LED & GDL', $cnt));
		}
		if(!defined('CAT_GDL') || !defined('CAT_GDL')) {
			throw new RepopulateGermanyException(sprintf('Cannot find category %s', (defined("CAT_GDL") ? 'LED' : 'GDL')));
		}

	}

	private function populate() {
		require_once(ELC_CLASSES . 'ProvisionalDeclaration.php');

		$this->output(sprintf('Starting repopulation process for year `%s`:', $this->year));

		// Get all active products
		$sql = "SELECT p.PRODUCT_ID, p.NAME
			FROM TBL_PRODUCT as p
			INNER JOIN TBL_CRSO_PRODUCT as cp
				ON p.PRODUCT_ID = cp.PRODUCT_ID
					AND cp.CRSO_RELATION_ID = ?
			ORDER BY p.PRODUCT_ID";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id));
		$q->execute();

		// WE CAN ONLY HAVE ONE (LAMPS)

		$r = $q->fetch();
		$this->output(sprintf('Product `%s`', $r->NAME));

		$line_sql = "UPDATE TBL_DECLARATION_LINE SET PRODUCT_CATEGORY_ID = ? WHERE DECLARATION_ID = ?";

		$q = $this->getActiveProducers($r->PRODUCT_ID);
		while($row = $q->fetch()) {
			$this->output(sprintf(' - producer `%s` .', substr($row->NAME, 0, 30)));

			$pd = new ProvisionalDeclaration($this->db);
			$pd->set($this->crso->relation->id, $row->PARTICIPANT_RELATION_ID, $this->year, $trust_id='', $product_id=$r->PRODUCT_ID);

			$newpd = new ProvisionalDeclaration($this->db);
			$newpd->crso = $pd->crso;
			$newpd->participant = $pd->participant;
			$newpd->product = $r->PRODUCT_ID;
			$newpd->year = $pd->year;

			$this->output('   - Provisional declaration .', false);

			foreach ($pd->getLines() as $line) {
				$this->output('.', false);
				// Add GDL category to each line.
				$newpd->addLine(array(
					'month' => $line->month,
					'brand'=>$line->brand,
					'product_category' => new ProductCategory($this->db, CAT_GDL),
					'quantity' => $line->quantity,
					'weight' => $line->weight
				));
				// Add LED category to the correct months
				if(!in_array($line->month, $this->monthsWithBothCategories)) {
					continue;
				}
				$this->output('N', false);
				$newpd->addLine(array(
					'month' => $line->month,
					'brand'=>$line->brand,
					'product_category' => new ProductCategory($this->db, CAT_LED),
					'quantity' => 0,
					'weight' => 0
				));
			}

			// update new provisional by deleting it and inserting again
			$this->execute && $pd->delete(true);
			$this->execute && $newpd->insert();

			$this->output(' [OK]');
			$this->output('   - Put on the Market declarations .', false);

			$sql = "SELECT DECLARATION_ID FROM TBL_DECLARATION as D
				WHERE D.CRSO_RELATION_ID = ?
					AND D.PARTICIPANT_RELATION_ID = ?
					AND D.PRODUCT_ID = ?
					AND D.YEAR = ?
			";
			$q1 = $this->db->query($sql);
			$q1->bind_values(array($this->crso->relation->id, $row->PARTICIPANT_RELATION_ID, $r->PRODUCT_ID, $this->year));
			$q1->execute();

			while($row1 = $q1->fetch()) {
				$q2 = $this->db->query($line_sql);
				$q2->bind_values(array(CAT_GDL, $row1->DECLARATION_ID));
				if($this->execute) {
					$q2->execute();
				}
				$this->output('.', false);
			}


			$this->output(' [OK]');
		}

	}

	private function getActiveProducers($product_id) {
		$sql = "SELECT cpp.PARTICIPANT_RELATION_ID, r.NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as cpp
			INNER JOIN TBL_RELATION as r
				ON cpp.PARTICIPANT_RELATION_ID = r.RELATION_ID
			WHERE cpp.CRSO_RELATION_ID = ?
				AND cpp.PRODUCT_ID = ?
				AND cpp.PRODUCER_EXPORTER & 1 = 1
				AND cpp.STATUS = 5";

		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id, $product_id));
		$q->execute();
		return $q;
	}

	public function output($line, $nl=true) {
		if(!$this->verbose) return;
		echo "$line";
		echo $nl?"\n":'';
		flush();
		//sleep(1);
	}

	public function __toString() {
		return '<RepopulateGermanProvisional>';
	}

}


function displayUsage() {
	echo "Usage: Update_v2.10_to_v2.11.php [options] crso_id:\n";
	echo "\n";
	echo "Options:\n";
	echo "  --site-settings\tPath to site settings file.\n";
	echo "  --dry-run\t\tDoesn't commit the changes to the database.\n";
	echo "  --verbose\t\tBe verbose.\n";
	echo "\n";
	exit();
}

if(sizeof($argv) < 2) {
	displayUsage();
}

$crso_id = array_pop($argv);
if(!is_numeric($crso_id)) {
	displayUsage();
}

$params = array('verbose'=>false, 'site-settings'=>false, 'dry-run'=>false);
foreach($params as $varname => $bitbucket) {
	foreach($argv as $arg) {
		if(preg_match('/--'.$varname.'=([^ ].*)/',$arg,$match)) {
			$params[$varname] = $match[1];
		} elseif(preg_match('/--'.$varname.'/',$arg,$match)) {
			$params[$varname] = true;
		}
	}
}
if(!$params['site-settings']) {
	displayUsage();
}

$site_homedir = '../../';
$_REQUEST = array();
$_SERVER = array(
	'SCRIPT_NAME'=>isset($params['script-path']) ? $params['script-path'] : $argv[0],
	'SERVER_NAME'=>$params['site-settings'],
	'HTTP_USER_AGENT'=>'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.2) Gecko/20060308 Firefox/1.5.0.2',
	'PHP_SELF'=>'IWSync.php',
	'REMOTE_ADDR'=>'127.0.0.1',
	'HTTP_HOST'=>isset($params['http-host']) ? $params['http-host'] : $params['site-settings'],
	'DOCUMENT_ROOT'=>str_replace('related/iwsync','',getcwd())
);

$noValidate = true;
require_once('../../scripts/i-w/default/init.php');
require_once(ELC_CLASSES.'CRSO.php');

restore_error_handler();
restore_exception_handler();

$db->begin();
$inst = new RepopulateGermanProvisional($db, $crso_id, $params['verbose'], $params['dry-run']);
try {
	$inst->repopulate();
	$db->commit();
} catch(RepopulateGermanyException $e) {
	$inst->output('[Fail]');
	echo "\n\nSomething went wrong: \n\n";
	echo " *\t".$e->getMessage();
	echo "\n\n";
	$db->rollback();
	exit();
} catch(Exception $e) {
	$db->rollback();
	throw $e;
}


?>