#!/usr/bin/php
<?php
error_reporting(E_ALL);

/**
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2014) BaseSoft B.V.
 */

define('REQUIRED_YEAR_FOR_PROVISIONAL_REPOPULATE', 2015);

// hungarian categories for #2535
define('CAT_A', 38);
define('CAT_B', 39);
define('CAT_TF', 90);
define('CAT_CFL', 91);
define('CAT_SPEC', 92);
define('CAT_LED', 93);

class RepopulateException extends Exception {

}

/**
 * This is a requirement for HU/RO to (p)repopulate the Provisional Declaration over 2015 with newly introduced settings:
 *  - New Product Categories have been added.
 *  - Weight has been enabled (HU has Weight including Decimals)
 *
 * The following rules apply:
 *  - The new Provisional for previously unused categories is 0 for quantity and thus 0 for weight/weight_dec
 *  - The new Provisional for existing used categories is the original quantity and the "average_weight" (description field in PC)
 *    time that quantity for weight/weight_dec
 *
 *
 * CRSO_ID's:
 *  * HU = 'Electro-Coord Magyarország Nonprofit Kft.' = '221' (Kovacs0954)
 *  * RO = Asociatia Recolamp = 353 (Bruma0001)
 *
 */
class RepopulateProvisional {

	var $year = REQUIRED_YEAR_FOR_PROVISIONAL_REPOPULATE;
	var $field = 'WEIGHT_DEC'; // Depends on Country setting

	public function __construct($db, $crso_id, $verbose=false, $dry_run=false) {
		$this->db =& $db;
		$this->crso = new CRSO($this->db,array('relation'=> new Relation($this->db,$crso_id, true)), true);;
		$this->verbose = $verbose;
		$this->execute = !$dry_run;
		$this->_pc_average_weight = array();
	}

	public function repopulate() {
		$this->validateRequirements();
		$this->populate();
		if(!$this->execute) {
			$this->output("\n\nDry-run was enabled, please rerun without the --dry-run to commit the changes.");
		}
	}

	private function populate() {
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');

		$this->output(sprintf('Starting repopulation process for year `%s`:', $this->year));

		// Get all active products
		$sql = "SELECT p.PRODUCT_ID, p.NAME
			FROM TBL_PRODUCT as p
			INNER JOIN TBL_CRSO_PRODUCT as cp
				ON p.PRODUCT_ID = cp.PRODUCT_ID
					AND cp.CRSO_RELATION_ID = ?
			ORDER BY p.PRODUCT_ID";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id));
		$q->execute();

		$prop = strtolower($this->field);

		while($r = $q->fetch()) {
			$this->output(sprintf('Product `%s`', $r->NAME));

			// Loop all active producers for this CRSO/Product
			$q1 = $this->getActiveProducers($r->PRODUCT_ID);

			$averages = $this->_pc_average_weight[$r->PRODUCT_ID];

			while($row = $q1->fetch()) {
				$this->output(sprintf(' - producer `%s` .', substr($row->NAME, 0, 30)), false);

				// Check provisional for already existing categories
				$pd = new ProvisionalDeclaration($this->db);
				$pd->set($this->crso->relation->id, $row->PARTICIPANT_RELATION_ID, $this->year, $trust_id='', $product_id=$r->PRODUCT_ID);
				$used_categories = array();
				foreach($pd->getLines() as $line) {
					$line->$prop = $averages[$line->product_category->id]*$line->quantity;
					$this->output('.', false);
					$used_categories[] = $line->product_category->id;
				}


				foreach($averages as $cat_id => $avg) {

					if(in_array($cat_id, $used_categories)) {
						continue;
					}

					$this->output('N', false);

					// loop months
					for($i = 1; $i < 13; $i++) {


						/**
						 * function that calculates a percentage over the provisional quantity
						 * in the current context for the given category. part of #2535
						 */
						$_this = $this; $get_quantity = function($percentage, $category) use ($_this, $i, $cat_id, $row, $r) {

							$sql = "SELECT QUANTITY FROM TBL_PROVISIONAL_DECLARATION
								WHERE YEAR = ? AND MONTH=? AND PRODUCT_CATEGORY_ID = ?
								AND CRSO_RELATION_ID = ? AND PARTICIPANT_RELATION_ID = ?
								AND PRODUCT_ID = ?";

							$q = $_this->db->query($sql);
							$q->bind_values(array($_this->year, $i, $category, $_this->crso->relation->id,
								$row->PARTICIPANT_RELATION_ID, $r->PRODUCT_ID));
							$q->execute();

							$rows = $q->fetchAll();
							if (count($rows) > 1)
								throw new Exception('multiple rows');
							if (count($rows) == 0)
								return 0;
							return floor($rows[0]->QUANTITY * $percentage);

						};

						// switch HU categories that need a special calculation.. see #2535
						switch ($cat_id) {

							case CAT_TF:
								$quantity = $get_quantity(0.8, CAT_A);
								break;
							case CAT_CFL:
								$quantity = $get_quantity(0.2, CAT_A);
								$prop_v = 0;
								break;
							case CAT_SPEC:
								$quantity = $get_quantity(1, CAT_B);
								$prop_v = 0;
								break;
							case CAT_LED:
								$quantity = 100;
								$prop_v = 0;
								break;

							default:
								$quantity = 0;
								$prop_v = 0;
								break;
						}


						$pd->addLine(array(
							'month' => $i,
							'product_category' => new ProductCategory($this->db, $cat_id),
							'quantity' => $quantity,
							$prop => $averages[$cat_id]*$quantity
						));
						$this->output('.', false);
					}
				}

				// update new provisional by deleting it and inserting again
				$this->execute && $pd->delete(true);
				$this->execute && $pd->insert();

				$this->output("[OK]");
			}

		}

	}

	private function getActiveProducers($product_id) {
		$sql = "SELECT cpp.PARTICIPANT_RELATION_ID, r.NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as cpp
			INNER JOIN TBL_RELATION as r
				ON cpp.PARTICIPANT_RELATION_ID = r.RELATION_ID
			WHERE cpp.CRSO_RELATION_ID = ?
				AND cpp.PRODUCT_ID = ?
				AND cpp.PRODUCER_EXPORTER & 1 = 1
				AND cpp.STATUS = 5";

		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id, $product_id));
		$q->execute();
		return $q;
	}

	/**
	 * We validate country settings and if provisional has been triggered yet.
	 */
	private function validateRequirements() {

		$this->crso->relation->set();
		$this->output(sprintf('Starting validation process for CRSO `%s`:', $this->crso->relation->name));
		require_once(ELC_CLASSES.'Country.php');
		// Verify the COUNTRY SETTINGS:
		$country = new Country($this->db, $this->crso->getCountry(), true);
		$this->output(' - country settings ...', false);

		if(!$country->has(HasDeclByCategory, 1)) {
			throw new RepopulateException('`Product category` is not enabled for this CRSO country.');
		}
		if(!$country->has(HasDeclOfWeight, 1) && !$country->has(HasDeclOfWeightWithDecimals, 1)) {
			throw new RepopulateException('`Weight` or `Weight in Decimals` is not enabled for this CRSO country.');
		}
		if($country->has(HasDeclOfWeight, 1)) {
			$this->field = 'WEIGHT';
		}
		if($country->has(HasDeclByBrand, 1)) {
			throw new RepopulateException('`Brand` is enabled for this CRSO country, deny repopulate.');
		}
		if($country->has(HasDeclarationDimension, 1)) {
			throw new RepopulateException('`Dimension` is enabled for this CRSO country, deny repopulate.');
		}
		$this->output("[OK]");
		$this->output(' - product categories .', false);

		// We validate if the product category descriptions have been added.
		$sql = "SELECT PRODUCT_ID, PRODUCT_CATEGORY_ID, NAME, DESCRIPTION FROM TBL_PRODUCT_CATEGORY WHERE CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id));
		$q->execute();

		while($r = $q->fetch()) {
			if(is_null($r->DESCRIPTION)) {
				throw new RepopulateException(sprintf('No description (average weight) set for Product category %s.', $r->NAME));
			}
			if(!is_numeric($r->DESCRIPTION)) {
				throw new RepopulateException(sprintf('Non-numerical description (average weight) set for Product category %s.', $r->NAME));
			}
			if(!isset($this->_pc_average_weight[$r->PRODUCT_ID])) {
				$this->_pc_average_weight[$r->PRODUCT_ID] = array();
			}
			$this->_pc_average_weight[$r->PRODUCT_ID][$r->PRODUCT_CATEGORY_ID] = $r->DESCRIPTION;
			$this->output('.', false);
		}
		$this->output("[OK]");

		// We validate the requirements:
		$sql = "SELECT MAX(YEAR) as `year` FROM TBL_PROVISIONAL_DECLARATION WHERE CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id));
		$q->execute();
		$r = $q->fetch();
		$this->output(' - provisional year ...', false);

		if($r->year!=$this->year) {
			throw new RepopulateException(
				sprintf(
					'The maximum provisional year %s does not match the required year %s',
					is_null($r->year)?'NULL':$r->year,
					$this->year
				)
			);
		}
		$this->output("[OK]\n");

	}

	public function output($line, $nl=true) {
		if(!$this->verbose) return;
		echo "$line";
		echo $nl?"\n":'';
		flush();
		//sleep(1);
	}

	public function __toString() {
		return '<RepopulateProvisional>';
	}
}

function displayUsage() {
	echo "Usage: Update_v2.6_to_v2.7.php [options] crso_id:\n";
	echo "\n";
	echo "Options:\n";
	echo "  --site-settings\tPath to site settings file.\n";
	echo "  --dry-run\t\tDoesn't commit the changes to the database.\n";
	echo "  --verbose\t\tBe verbose.\n";
	echo "\n";
	exit();
}

if(sizeof($argv) < 2) {
	displayUsage();
}

$crso_id = array_pop($argv);
if(!is_numeric($crso_id)) {
	displayUsage();
}

$params = array('verbose'=>false, 'site-settings'=>false, 'dry-run'=>false);
foreach($params as $varname => $bitbucket) {
	foreach($argv as $arg) {
		if(preg_match('/--'.$varname.'=([^ ].*)/',$arg,$match)) {
			$params[$varname] = $match[1];
		} elseif(preg_match('/--'.$varname.'/',$arg,$match)) {
			$params[$varname] = true;
		}
	}
}
if(!$params['site-settings']) {
	displayUsage();
}

$site_homedir = '../../';
$_REQUEST = array();
$_SERVER = array(
	'SCRIPT_NAME'=>isset($params['script-path']) ? $params['script-path'] : $argv[0],
	'SERVER_NAME'=>$params['site-settings'],
	'HTTP_USER_AGENT'=>'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.0.2) Gecko/20060308 Firefox/1.5.0.2',
	'PHP_SELF'=>'IWSync.php',
	'REMOTE_ADDR'=>'127.0.0.1',
	'HTTP_HOST'=>isset($params['http-host']) ? $params['http-host'] : $params['site-settings'],
	'DOCUMENT_ROOT'=>str_replace('related/iwsync','',getcwd())
);

$noValidate = true;
require_once('../../scripts/i-w/default/init.php');
require_once(ELC_CLASSES.'CRSO.php');

restore_error_handler();
restore_exception_handler();

$db->begin();
$inst = new RepopulateProvisional($db, $crso_id, $params['verbose'], $params['dry-run']);
try {
	$inst->repopulate();
	$db->commit();
} catch(RepopulateException $e) {
	$inst->output('[Fail]');
	echo "\n\nSomething went wrong: \n\n";
	echo " *\t".$e->getMessage();
	echo "\n\n";
	$db->rollback();
	exit();
} catch(Exception $e) {
	$db->rollback();
	throw $e;
}

?>
